import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.util.ArrayList;
import java.io.Serializable;

import javax.swing.*;

public class Canvas_Memo{
	public static final int MODE_NORMAL = 0;
	public static final int MODE_FOCUSED = 1;
	public static final int MODE_ADDING = 2;
	
	static final int BG_MARGIN_W = 3;
	static final int BG_MARGIN_H = 2;
	static final Color TEXT_COLOR = Color.BLACK;
	static final Color BG_COLOR = new Color(255, 255, 200);
	static final Color EDGE_COLOR = Color.DARK_GRAY;
	static final Color FOCUS_RECT_COLOR = Color.BLACK;
	protected int mode;
	protected MyCanvas canvas;
	protected Point mouse_last_pos;
	
	int bg_w;
	int bg_h;
		
	Model_Root data;
	Model_Memo model;
	boolean is_draged = false;
	
	int click_num;
	
	Canvas_Memo(MyCanvas _canvas, Model_Root _data, Model_Memo m, int _mode){
		canvas = _canvas;
		data = _data;
		model = m;
		mode = _mode;
		mouse_last_pos = null;
		click_num = 0;
		
		bg_w = -1;
		bg_h = -1;
	}
	
	public void dispose(){
		canvas = null;
		data = null;
		model = null;
		mode = MODE_NORMAL;
		mouse_last_pos = null;
	}
	
	public void move_pos(int dx, int dy){
		data.move_memo_pos(model, dx, dy);
	}
	
	public void paint(Graphics g){
		switch (mode){
		case MODE_NORMAL :
			paint_local(g, BG_COLOR, EDGE_COLOR, TEXT_COLOR, false, false);
			break;
		case MODE_FOCUSED :
			paint_local(g, BG_COLOR, EDGE_COLOR, TEXT_COLOR, true, false);
			break;
		case MODE_ADDING :
			paint_local(g, BG_COLOR, EDGE_COLOR, TEXT_COLOR, false, true);
			break;
		}
	}
	
	public boolean is_in(Rectangle r){
		if(r.x <= pos().x && pos().x + bg_w <= r.x + r.width && r.y <= pos().y && pos().y + bg_h <= r.y + r.height) return true;
		else return false;
	}
	
	public void set_pos(Point p){
		data.set_memo_pos(model, p);
	}
		
	public boolean isHit(Point p){
		if(pos().x <= p.x &&
		   p.x < pos().x + bg_w &&
		   pos().y <= p.y &&
		   p.y < pos().y + bg_h
		){
			return true;
		}
		
		return false;
	}
	
	public Point max_point(){
		return new Point(pos().x + bg_w, pos().y + bg_h);
	}
	
	public void setMode(int _mode){
		if(mode != _mode) click_num = 0;
		mode = _mode;
		//変更通知
		canvas.need_repaint();
	}
	
	public Model_Memo get_model(){
		return model;
	}
		
	public void mouseReleased(MouseEvent e, boolean is_multi_focused){
		set_cursor(e, is_multi_focused);
	}

    public void mousePressed(MouseEvent e, boolean is_multi_focused){
    	mouse_last_pos = e.getPoint();
    	setMode(MODE_FOCUSED);
    	
    	is_draged = false;
    	
    	set_cursor(e, is_multi_focused);
    }
  
    public void mouseClicked(MouseEvent e, boolean is_multi_focused){
    	if(e.getClickCount() == 1) click_num = 1;
    	else click_num ++;
  
    	switch(click_num){
    	//クリック
    	case 1 :
    		break;
    	//ダブルクリック
    	case 2 :
    		canvas.get_root_frame().show_memo_dialog(model);
    		break;
    	}
    	
    	set_cursor(e, is_multi_focused);
    }

	public void mouseEntered(MouseEvent e, boolean is_multi_focused){}
	
	public void mouseExited(MouseEvent e, boolean is_multi_focused){}
	
	public void mouseDragged(MouseEvent e, boolean is_multi_focused){
		move_pos(e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
		
		mouse_last_pos = e.getPoint();
		click_num = 0;
		is_draged = true;
		set_cursor(e, is_multi_focused);
	}
	
	public void set_cursor(MouseEvent e, boolean is_multi_focused){
		if(isHit(e.getPoint()) == true){
			canvas.request_set_cursor(Cursor.CROSSHAIR_CURSOR);
		}
	}
	
	public void mouseMoved(MouseEvent e, boolean is_multi_focused){
		click_num = 0;
		
		set_cursor(e, is_multi_focused);
	}
	
	public void keyPressed(KeyEvent e, boolean is_multi_focused) {}
	
	/******* private *******/
	private Point pos(){
		return model.get_pos();
	}
	
	protected int bg_margin_w(){
		return scaled(BG_MARGIN_W);
	}
	
	protected int bg_margin_h(){
		return scaled(BG_MARGIN_H);
	}
			
	protected float scale(){
		return canvas.get_scale();
	}
	
	protected int scaled(int x){
		return (int)(x * scale());
	}
	
	protected int unscaled(int x){
		return (int)(x / scale());
	}
	
	protected float scaled_f(float x){
		return x * scale();
	}
	
	protected Point scaled_p(Point _p){
		return new Point(scaled(_p.x), scaled(_p.y));
	}
	
	protected void paint_local(Graphics g, Color bg_c, Color edge_c, Color label_c, boolean is_rect, boolean is_transparent){
		Graphics2D g2 = (Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		float alpha = 1.0f;
		if(is_transparent) alpha = 0.5f;
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));	
		
		String text = model.get_memo();	
		Font font = new Font(canvas.get_font_name(), g.getFont().getStyle(), scaled(canvas.get_org_font_size()));
		
		//文字列サイズを取得
		int text_w = canvas.multiline_string_width(g2, text);
		int text_h = canvas.multiline_string_height(g2, text);
				
		bg_w = text_w + bg_margin_w() * 2;		
		bg_h = text_h + bg_margin_h() * 2;		
		
		//背景を描く
		g2.setColor(bg_c);
		g2.fillRect(pos().x, pos().y, bg_w, bg_h);
		g2.setColor(edge_c);
		g2.setStroke(new BasicStroke(scaled_f(1.0f)));
		g2.drawRect(pos().x, pos().y, bg_w, bg_h);
		
		//テキストを描く
		g2.setColor(label_c);
		canvas.draw_multiline_string(g2, new Point(pos().x + bg_margin_w(), pos().y + bg_margin_h()), text, false);
		
		if(is_rect){
			//枠線
			Graphics2D g4 = (Graphics2D)g2;
			BasicStroke stroke2;
			g4.setColor(FOCUS_RECT_COLOR);
			stroke2 = new BasicStroke(scaled_f(2.0f),		// 太さ
                    BasicStroke.CAP_BUTT,		// 端は残す
                    BasicStroke.JOIN_MITER,		// 継ぎ目は斜め
                    1.0f,						// 継ぎ目の限界
                    new float[] {scaled_f(5.0f), scaled_f(5.0f)},	// 5書いて5書かないダッシュ
                    scaled_f(0.0f));						// ダッシュの開始フェーズ
			g4.setStroke(stroke2); //線の種類を設定
			g4.drawRect(pos().x, pos().y, bg_w, bg_h);			
		}
	}
}
