import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.util.ArrayList;
import java.io.Serializable;

import javax.swing.*;

public class Canvas_State{	  
	public static final int MODE_NORMAL = 0;
	public static final int MODE_FOCUSED = 1;
	public static final int MODE_ADDING = 2;
	
	public static final int NONE = -1;
	public static final int POINT_LEFT_TOP = 0;
	public static final int POINT_LEFT_BOTTOM = 1;
	public static final int POINT_RIGHT_TOP = 2;
	public static final int POINT_RIGHT_BOTTOM = 3;
	
	private static final int MIN_R = 10;	
	
	protected static final Color BG_COLOR = new Color(240, 240, 255);
	protected static final Color EDGE_COLOR = Color.BLUE;
	protected static final Color LABEL_COLOR = Color.BLACK;
	protected static final Color ABSTRACT_BG_COLOR = BG_COLOR;//new Color(250, 250, 255);
	protected static final Color ABSTRACT_EDGE_COLOR = new Color(128, 128, 255);
	protected static final Color ABSTRACT_LABEL_COLOR = LABEL_COLOR;//new Color(96, 96, 96);
	protected static final float ABSTRACT_ALPHA = 0.5f;
	
	protected static final Color FOCUS_RECT_COLOR = Color.BLACK;
	protected static final int HIT_MARGIN_SIZE = 4;
	static final int HIT_MARGIN_POINT = 0;
	protected int mode;
	protected MyCanvas canvas;
	protected Point mouse_last_pos;
	protected int focused_point;
	
	static final int POINT_SIZE = 8;
	static final Color POINT_COLOR = Color.RED;
	static final Color POINT_EDGE_COLOR = Color.BLACK;
	protected static final int FONT_SIZE = 12;
	
	ArrayList<Point> point_list;
	ArrayList<Point> connect_point_list;
	Model_Root data;
	Model_State model;
	boolean is_draged = false;
	
	int click_num;
	
	int mouse_point_offset_x = 0;
	int mouse_point_offset_y = 0;
	
	Canvas_State(MyCanvas _canvas, Model_Root _data, Model_State s, int _mode){
		canvas = _canvas;
		data = _data;
		model = s;
		mode = _mode;
		mouse_last_pos = null;
		click_num = 0;
		
		focused_point = NONE;
		
		update_point_list();
	}
	
	public void dispose(){
		canvas = null;
		data = null;
		model = null;
		mode = MODE_NORMAL;
		mouse_last_pos = null;
	}
	
	public Point get_center_pos(){
		return model.get_pos();
	}
	
	public void move_pos_with_trans(int dx, int dy){
		data.move_state_with_trans(model, dx, dy);
	}
	
	public void move_pos_without_focused_trans(int dx, int dy){
		ArrayList<Model_Trans> focused_trans_list = new ArrayList<Model_Trans>();
		
		for(int i = 0; i < canvas.get_focused_trans_list().size(); i ++){
			focused_trans_list.add(canvas.get_focused_trans_list().get(i).get_model());
		}
		
		data.move_state_without_focused_trans(model, dx, dy, focused_trans_list);
	}
	
	public Point get_connect_trans_point(Point p){
		Point new_p = new Point();
		
		//中心からpの延長線上で円と交わるところにpを移動させる
		double len = p.distance(pos());
		new_p.x = (int)(pos().x + (p.x - pos().x) * r() / len);
		new_p.y = (int)(pos().y + (p.y - pos().y) * r() / len);
				
		return new_p;
	}
		
	public void paint(Graphics g){
		Color bg_color = BG_COLOR;
		Color edge_color = EDGE_COLOR;
		Color label_color = LABEL_COLOR;
		
		switch (mode){
		case MODE_NORMAL :
			paint_local(g, bg_color, edge_color, label_color, false, false);
			break;
		case MODE_FOCUSED :
			paint_local(g, bg_color, edge_color, label_color, true, false);
			break;
		case MODE_ADDING :
			paint_local(g, bg_color, edge_color, label_color, false, true);
			break;
		}
	}
	
	public boolean is_in(Rectangle r){
		if(r.x <= pos().x - r() && pos().x + r() <= r.x + r.width && r.y <= pos().y - r() && pos().y + r() <= r.y + r.height) return true;
		else return false;
	}
	
	public void set_pos_with_trans(Point p){
		data.set_state_pos_with_trans(model, p);
	}
	
	public void set_pos_without_trans(Point p){
		data.set_state_pos_without_trans(model, p);
	}
	
	public boolean isHit(Point p){
		if(p.distance(pos()) <= r() + HIT_MARGIN_SIZE) return true;
	
		if(mode == MODE_FOCUSED){
			if(getHitPoint(p) != NONE) return true;
		}
		return false;
	}
	
	public Point max_point(){
		return new Point(pos().x + r(), pos().y + r());
	}
	
	public void setMode(int _mode){
		if(mode != _mode) click_num = 0;
		mode = _mode;
		//変更通知
		canvas.need_repaint();
	}
	
	public int get_mode(){
		return mode;
	}
	
	public Model_State get_model(){
		return model;
	}
	
	public int getHitPoint(Point p){
		if(point_list.get(0).distance(p) <= point_size() + HIT_MARGIN_POINT) return POINT_LEFT_TOP;
		if(point_list.get(1).distance(p) <= point_size() + HIT_MARGIN_POINT) return POINT_LEFT_BOTTOM;
		if(point_list.get(2).distance(p) <= point_size() + HIT_MARGIN_POINT) return POINT_RIGHT_TOP;
		if(point_list.get(3).distance(p) <= point_size() + HIT_MARGIN_POINT) return POINT_RIGHT_BOTTOM;
		return NONE;
	}
	
	public void mouseReleased(MouseEvent e, boolean is_multi_focused){
		set_cursor(e, is_multi_focused);
	}

    public void mousePressed(MouseEvent e, boolean is_multi_focused){
    	mouse_last_pos = e.getPoint();
    	setMode(MODE_FOCUSED);
    	
    	is_draged = false;
    	if(is_multi_focused == true) focused_point = NONE;
    	else{
    		focused_point = getHitPoint(e.getPoint());
    		
    		switch(focused_point){
			case POINT_LEFT_TOP:
				mouse_point_offset_x = e.getX() - (pos().x - r());
	    		mouse_point_offset_y = e.getY() - (pos().y - r());
				break;
			case POINT_LEFT_BOTTOM:
				mouse_point_offset_x = e.getX() - (pos().x - r());
	    		mouse_point_offset_y = e.getY() - (pos().y + r());
				break;
			case POINT_RIGHT_TOP:
				mouse_point_offset_x = e.getX() - (pos().x + r());
	    		mouse_point_offset_y = e.getY() - (pos().y - r());
				break;
			case POINT_RIGHT_BOTTOM:
				mouse_point_offset_x = e.getX() - (pos().x + r());
	    		mouse_point_offset_y = e.getY() - (pos().y + r());
				break;
			}
    	}
    	
    	set_cursor(e, is_multi_focused);
    }
  
    public void mouseClicked(MouseEvent e, boolean is_multi_focused){
    	if(e.getClickCount() == 1) click_num = 1;
    	else click_num ++;
  
    	switch(click_num){
    	//クリック
    	case 1 :
    		break;
    	//ダブルクリック
    	case 2 :
    		canvas.get_root_frame().show_state_dialog(model);
    		break;
    	}
    	
    	set_cursor(e, is_multi_focused);
    }

	public void mouseEntered(MouseEvent e, boolean is_multi_focused){}
	
	public void mouseExited(MouseEvent e, boolean is_multi_focused){}
	
	public void mouseDragged(MouseEvent e, boolean is_multi_focused){
		if(focused_point == NONE){
			canvas.request_set_cursor(Cursor.CROSSHAIR_CURSOR);
			if(is_multi_focused == true){
				move_pos_without_focused_trans(e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
			}
			else{
				move_pos_with_trans(e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
			}
		}
		else{
			int r = MIN_R;
			switch(focused_point){
			case POINT_LEFT_TOP:
				canvas.request_set_cursor(Cursor.NW_RESIZE_CURSOR);
				r = Math.max(Math.max(pos().x - (e.getX() - mouse_point_offset_x), pos().y - (e.getY() - mouse_point_offset_y)), min_r());
				break;
			case POINT_LEFT_BOTTOM:
				canvas.request_set_cursor(Cursor.SW_RESIZE_CURSOR);
				r = Math.max(Math.max(pos().x - (e.getX() - mouse_point_offset_x), (e.getY() - mouse_point_offset_y) - pos().y), min_r());
				break;
			case POINT_RIGHT_TOP:
				canvas.request_set_cursor(Cursor.NE_RESIZE_CURSOR);
				r = Math.max(Math.max((e.getX() - mouse_point_offset_x) - pos().x, pos().y - (e.getY() - mouse_point_offset_y)), min_r());
				break;
			case POINT_RIGHT_BOTTOM:
				canvas.request_set_cursor(Cursor.SE_RESIZE_CURSOR);
				r = Math.max(Math.max((e.getX() - mouse_point_offset_x) - pos().x, (e.getY() - mouse_point_offset_y) - pos().y), min_r());
				break;
			}
			
			data.set_state_r(model, r);
		}
		
		mouse_last_pos = e.getPoint();
		click_num = 0;
		is_draged = true;
	}
	
	public void set_cursor(MouseEvent e, boolean is_multi_focused){
		if(mode == MODE_FOCUSED){
			if(is_multi_focused == true){
				if(isHit(e.getPoint()) == true || getHitPoint(e.getPoint()) != NONE){
					canvas.request_set_cursor(Cursor.MOVE_CURSOR);
				}
			}
			else{
				switch(getHitPoint(e.getPoint())){
				case POINT_LEFT_TOP:
					canvas.request_set_cursor(Cursor.NW_RESIZE_CURSOR);
					break;
				case POINT_LEFT_BOTTOM:
					canvas.request_set_cursor(Cursor.SW_RESIZE_CURSOR);
					break;
				case POINT_RIGHT_TOP:
					canvas.request_set_cursor(Cursor.NE_RESIZE_CURSOR);
					break;
				case POINT_RIGHT_BOTTOM:
					canvas.request_set_cursor(Cursor.SE_RESIZE_CURSOR);
					break;
				case NONE :
					if(isHit(e.getPoint()) == true){
						canvas.request_set_cursor(Cursor.CROSSHAIR_CURSOR);
					}
					break;
				}
			}
		}
	}
	
	public void mouseMoved(MouseEvent e, boolean is_multi_focused){
		click_num = 0;
		
		set_cursor(e, is_multi_focused);
	}
	
	public void keyPressed(KeyEvent e, boolean is_multi_focused) {}
	
	/******* protected *******/
	protected String label(){
		return ((Model_State)model).base_state.get_label_or_name();
	}
	
	protected Point pos(){
		return model.get_pos();
	}
		
	protected int r(){
		return model.get_r();
	}
	
	protected int r2(){
		return r() * 2;
	}
	
	protected float scale(){
		return canvas.get_scale();
	}
	
	protected int scaled(int x){
		return (int)(x * scale());
	}
	
	protected int unscaled(int x){
		return (int)(x / scale());
	}
	
	protected float scaled_f(float x){
		return x * scale();
	}
	
	protected Point scaled_p(Point _p){
		return new Point(scaled(_p.x), scaled(_p.y));
	}
	
	protected int point_size(){
		return scaled(POINT_SIZE);
	}
	
	protected int min_r(){
		return scaled(MIN_R);
	}
	
	protected int font_size(){
		return scaled(FONT_SIZE);
	}
	
	protected void paint_local(Graphics g, Color bg_c, Color edge_c, Color label_c, boolean is_rect, boolean is_transparent){
		Graphics2D g2 = (Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		float alpha = 1.0f;
		if(is_transparent) alpha = 0.5f;
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		
		g2.setColor(bg_c);
		g2.fillOval(pos().x - r(), pos().y - r(), r2(), r2());
		
		Graphics2D g3 = (Graphics2D)g2;
		BasicStroke stroke;
		g3.setColor(edge_c);
		stroke = new BasicStroke(scaled_f(2.0f)); //線の太さを変更
		g3.setStroke(stroke); //線の種類を設定
		g3.drawOval(pos().x - r(), pos().y - r(), r2(), r2());
		
		g3.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
		
		if(is_rect){
			//枠線
			Graphics2D g4 = (Graphics2D)g2;
			BasicStroke stroke2;
			g4.setColor(FOCUS_RECT_COLOR);
			stroke2 = new BasicStroke(scaled_f(2.0f),		// 太さ
                    BasicStroke.CAP_BUTT,		// 端は残す
                    BasicStroke.JOIN_MITER,		// 継ぎ目は斜め
                    1.0f,						// 継ぎ目の限界
                    new float[] {scaled_f(5.0f), scaled_f(5.0f)},	// 5書いて5書かないダッシュ
                    scaled_f(0.0f));						// ダッシュの開始フェーズ
			g4.setStroke(stroke2); //線の種類を設定
			g4.drawRect(pos().x - r(), pos().y - r(), r2(), r2());			
			
			//コントロールポイント
			g4.setStroke(new BasicStroke(scaled_f(2.0f)));
			
			update_point_list();
			for (int i = 0; i < point_list.size(); i ++){
				Point p = point_list.get(i);
					
				g4.setColor(POINT_COLOR);
				g4.fillOval(p.x - point_size(), p.y - point_size(), point_size() * 2, point_size() * 2);
				
				g4.setColor(POINT_EDGE_COLOR);
				g4.drawOval(p.x - point_size(), p.y - point_size(), point_size() * 2, point_size() * 2);
			}
		}
		
		if(label().length() > 0){
			Font font = new Font(canvas.get_font_name(), g.getFont().getStyle(), scaled(canvas.get_org_font_size()));
			g.setFont(font);
			
			int text_w = canvas.multiline_string_width(g, label());
			int text_h = canvas.multiline_string_height(g, label());
			g.setColor(label_c);
	        canvas.draw_multiline_string(g, new Point(pos().x - text_w / 2, pos().y - text_h / 2), label(), true);
			//g.drawString(label(), pos().x - strWidth / 2, pos().y + ascent / 2);
		}
	}
			
	protected void update_point_list(){
		if(point_list == null){
			point_list = new ArrayList<Point>();
			point_list.add(new Point());
			point_list.add(new Point());
			point_list.add(new Point());
			point_list.add(new Point());			
		}
		
		point_list.get(0).setLocation(pos().x - r(), pos().y - r());
		point_list.get(1).setLocation(pos().x - r(), pos().y + r());
		point_list.get(2).setLocation(pos().x + r(), pos().y - r());
		point_list.get(3).setLocation(pos().x + r(), pos().y + r());
	}
}
