import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.util.ArrayList;
import java.io.Serializable;

import javax.swing.*;

public class Canvas_Trans{	
	private int mode;
	private MyCanvas canvas;
	private Point mouse_last_pos;
	private Point focused_point;
	private Canvas_State src_state;
	private Canvas_State dest_state;
	private Canvas_Trans_Label label;
	private boolean is_draged;
	private Model_Trans model;
	private Model_Root data;
	int click_num;
	
	private static final Color LINE_COLOR_CONNECT = Color.DARK_GRAY;
	private static final Color LINE_COLOR_UNCONNECT = Color.LIGHT_GRAY;
	private static final int ALLOW_LEN = 15;
	private static final double ALLOW_RAD = Math.toRadians(20.0f);
	public static final int MODE_NORMAL = 0;
	public static final int MODE_FOCUSED_LINE = 1;
	public static final int MODE_ADDING = 2;
	public static final int MODE_FOCUSED_LABEL = 3;
	static final int POINT_SIZE = 8;
	static final int HIT_MARGIN_LINE = 8;
	static final int HIT_MARGIN_POINT = 0;
	static final Color POINT_COLOR_CONNECT = Color.BLUE;
	static final Color POINT_COLOR_UNCONNECT = Color.RED;
	static final Color POINT_EDGE_COLOR = Color.BLACK;
	static final String DEFAULT_LABEL = "Label";
	protected static final int FONT_SIZE = 12;
	
	static final int MAX_POINTS = 10;
	static final int LOCAL_MAX_POINTS = MAX_POINTS + 2;
	
	Canvas_Trans(MyCanvas _canvas, Model_Root _data, Model_Trans _model, int _mode){
		canvas = _canvas;
		model = _model;
		data = _data;
			
		mode = _mode;
		mouse_last_pos = null;
		focused_point = null;
		click_num = 0;
			
		src_state = canvas.get_state(model.get_src_state());
		dest_state = canvas.get_state(model.get_dest_state());
		label = new Canvas_Trans_Label(canvas, this, data, model);
		is_draged = false;
		
		curve = new Curve();
		update_curve();
	}
	
	public void dispose(){
		canvas = null;
		model = null;
		data = null;
			
		mode = MODE_NORMAL;
		mouse_last_pos = null;
		focused_point = null;
			
		src_state = null;
		dest_state = null;
		label.dispose();
		label = null;
		is_draged = false;
	}
	
	protected float scale(){
		return canvas.get_scale();
	}
	
	protected int scaled(int x){
		return (int)(x * scale());
	}
	
	protected int unscaled(int x){
		return (int)(x / scale());
	}
	
	protected float scaled_f(float x){
		return x * scale();
	}
	
	protected double scaled_d(double x){
		return x * scale();
	}
	
	protected Point scaled_p(Point _p){
		return new Point(scaled(_p.x), scaled(_p.y));
	}
	
	protected Point unscaled_p(Point _p){
		return new Point(unscaled(_p.x), unscaled(_p.y));
	}
	
	protected int point_size(){
		return scaled(POINT_SIZE);
	}
	
	protected int font_size(){
		return scaled(FONT_SIZE);
	}
	
	protected int hit_allow_len(){
		return scaled(ALLOW_LEN);
	}
	
	public Point max_point(){
		Point max = new Point(0, 0);
		for(int i = 0; i < point_list().size() - 1; i ++){
			max.x = Math.max(max.x, point_list().get(i).x + point_size());
			max.y = Math.max(max.y, point_list().get(i).y + point_size());
		}
		max.x = Math.max(max.x, label.max_point().x + point_size());
		max.y = Math.max(max.y, label.max_point().y + point_size());
		return max;
	}
	
	public void change_prop_label(){
		canvas.repaint();
	}
	
	public Model_Trans get_model(){
		return model;
	}
		
	public boolean isHit(Point p){
		/*直線
		for(int i = 0; i < point_list().size() - 1; i ++){
			Line2D.Float line = new Line2D.Float(point_list().get(i), point_list().get(i + 1));
			if(line.ptSegDist(p) <= hit_margin_line()) return true;
		}
		*/
		//spline
		if(sprine_is_hit(p) == true) return true;
		
		if(getHitPoint(p) != null) return true;
		if(label.isHit(p) == true) return true; 
		return false;
	}
	
	public boolean is_in(Rectangle r){
		for(int i = 0; i < point_list().size(); i ++){
			if(!is_point_in(r, point_list().get(i))) return false;
		}
		
		return true;
	}
	
	public boolean is_point_in(Rectangle r, Point p){
		if(r.x <= p.x && p.x <= r.x + r.width && r.y <= p.y && p.y <= r.y + r.height) return true;
		else return false;
	}
	
	public Point getHitPoint(Point p){
		for(int i = 0; i < org_point_list().size(); i ++){
			Point p2 = org_point_list().get(i);
			if(scaled_p(p2).distance(p) <= point_size() + HIT_MARGIN_POINT) return p2;
		}
		return null;
	}
	
	public void setMode(int _mode){
		if(mode != _mode) click_num = 0;
		mode = _mode;
		
		//変更通知
		canvas.need_repaint();
	}
	
	public void set_dest_pos(Point p){
		data.set_trans_dest_point(model, p);
	}
	
	public void set_src_state(Canvas_State cs){
		src_state = cs;
		if(cs != null){
			Point new_p = cs.get_connect_trans_point(src_point());
			data.connect_trans_src_point(model, cs.get_model(), new_p);
		}
		else{
			data.unconnect_trans_src_point(model);
		}
	}
	
	protected void set_src_state_if_other_state(Canvas_State cs){
		src_state = cs;
		if(cs != null){
			if(cs != get_src_state()){
				Point new_p = cs.get_connect_trans_point(src_point());
				data.connect_trans_src_point(model, cs.get_model(), new_p);
			}
		}
		else{
			data.unconnect_trans_src_point(model);
		}
	}
	
	public void set_dest_state(Canvas_State cs){
		dest_state = cs;
		
		if(cs != null){
			Point new_p = cs.get_connect_trans_point(dest_point());
			data.connect_trans_dest_point(model, cs.get_model(), new_p);
		}
		else{
			data.unconnect_trans_dest_point(model);
		}
	}
	
	protected void set_dest_state_if_other_state(Canvas_State cs){
		dest_state = cs;
		
		if(cs != null){
			if(cs != get_dest_state()){
				Point new_p = cs.get_connect_trans_point(dest_point());
				data.connect_trans_dest_point(model, cs.get_model(), new_p);
			}
		}
		else{
			data.unconnect_trans_dest_point(model);
		}
	}
	
	public Canvas_State get_src_state(){
		return src_state;
	}
	
	public Canvas_State get_dest_state(){
		return dest_state;
	}
	
	public void del_state(Canvas_State cs){
		if(src_state == cs) src_state = null;
		if(dest_state == cs) dest_state = null;
	}
	
	public void paint_label(Graphics g){
		boolean is_transparent = (mode == MODE_ADDING);
		
		Graphics2D g2 = (Graphics2D)g;
		float alpha = 1.0f;
		if(is_transparent) alpha = 0.5f;
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		
		//ラベルを描画
		label.paint(g2);
	}
	
	public void paint_line(Graphics g){
		boolean is_transparent = (mode == MODE_ADDING);
		boolean is_draw_point = (mode == MODE_FOCUSED_LINE || mode == MODE_FOCUSED_LABEL || mode == MODE_ADDING);
		
		Color line_c = LINE_COLOR_UNCONNECT;
		Color src_point_c = POINT_COLOR_UNCONNECT;
		Color dest_point_c = POINT_COLOR_UNCONNECT;
		Color middle_point_c = POINT_COLOR_UNCONNECT;
		Color point_edge_c = POINT_EDGE_COLOR;
		if(src_state != null && dest_state != null){
			line_c = LINE_COLOR_CONNECT;
		}
		if(src_state != null){
			src_point_c = POINT_COLOR_CONNECT;
		}
		if(dest_state != null){
			dest_point_c = POINT_COLOR_CONNECT;
		}
		
		paint_line_local(g, is_transparent, is_draw_point, line_c, src_point_c, dest_point_c, middle_point_c, point_edge_c);
	}
	public void paint_line_local(Graphics g, boolean is_transparent, boolean is_draw_point, Color line_c, Color src_point_c, Color dest_point_c, Color middle_point_c, Color point_edge_c){
		Graphics2D g2 = (Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		float alpha = 1.0f;
		if(is_transparent) alpha = 0.5f;
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		
		//線を描く
		g2.setColor(line_c);
		g2.setStroke(new BasicStroke(scaled_f(2.0f)));
		/* 直線
		int [] xs = new int[point_list().size()];
		int [] ys = new int[point_list().size()];
		for(int i=0; i < point_list().size(); i ++){
			xs[i] = point_list().get(i).x;
			ys[i] = point_list().get(i).y;
		}
		g2.drawPolyline(xs, ys, point_list().size());
		*/
		//スプライン
		update_curve();
		PolyBezier(g2, curve.pv, 3 * (curve.npt - 1) + 1);
		
		//矢印の先っぽを描く
		drow_line_arrow(g2);
				
		//頂点を描画
		if(is_draw_point){
			g2.setStroke(new BasicStroke(scaled_f(2.0f)));
			for (int i = 0; i < org_point_list().size(); i ++){
				Point p = point_list().get(i);
				
				if(org_point_list().get(i) == org_src_point()){
					g2.setColor(src_point_c);
				}
				else if(org_point_list().get(i) == org_dest_point()){
					g2.setColor(dest_point_c);
				}
				else{
					g2.setColor(middle_point_c);
				}								
				g2.fillOval(p.x - point_size(), p.y - point_size(), point_size() * 2, point_size() * 2);
				
				g2.setColor(point_edge_c);
				g2.drawOval(p.x - point_size(), p.y - point_size(), point_size() * 2, point_size() * 2);
			}
		}
	}
	
	protected void drow_line_arrow(Graphics2D g){
		Point from;
		Point to;
		
		if(get_dest_state() != null){
			from = new Point(point_list().get(point_list().size() - 1));	
			to = new Point(get_dest_state().get_center_pos());
			int dx = from.x - to.x;
			int dy = from.y - to.y;
			from.translate(dx, dy);
			to.translate(dx, dy);
		}
		else{
			from = new Point(point_list().get(point_list().size() - 2));
			to = new Point(point_list().get(point_list().size() - 1));
		}
		
		Point p3 = new Point();
		p3.x = (int)(to.x + Math.cos(ALLOW_RAD) * (from.x - to.x) - Math.sin(ALLOW_RAD) * (from.y - to.y));
		p3.y = (int)(to.y + Math.sin(ALLOW_RAD) * (from.x - to.x) + Math.cos(ALLOW_RAD) * (from.y - to.y));
		double l3 = p3.distance(to);
		if((int)l3 != 0){
			p3.x = (int)(to.x + (p3.x - to.x) * hit_allow_len() / l3);
			p3.y = (int)(to.y + (p3.y - to.y) * hit_allow_len() / l3);
			
			Point p4 = new Point();
			p4.x = (int)(to.x + Math.cos(-ALLOW_RAD) * (from.x - to.x) - Math.sin(-ALLOW_RAD) * (from.y - to.y));
			p4.y = (int)(to.y + Math.sin(-ALLOW_RAD) * (from.x - to.x) + Math.cos(-ALLOW_RAD) * (from.y - to.y));
			double l4 = p4.distance(to);
			if((int)l4 != 0){
				p4.x = (int)(to.x + (p4.x - to.x) * hit_allow_len() / l4);
				p4.y = (int)(to.y + (p4.y - to.y) * hit_allow_len() / l4);
				
				int [] xs2 = {p3.x, to.x, p4.x, p3.x};
				int [] ys2 = {p3.y, to.y, p4.y, p3.y};
				g.fillPolygon(xs2, ys2, 3);
				
				g.setStroke(new BasicStroke(scaled_f(2.5f)));
				
				int [] xs3 = {(p3.x + p4.x) / 2, p3.x, to.x, p4.x, (p3.x + p4.x) / 2};
				int [] ys3 = {(p3.y + p4.y) / 2, p3.y, to.y, p4.y, (p3.y + p4.y) / 2};
				g.drawPolyline(xs3, ys3, 5);
			}
		}
	}
	
	public void add_point(Point p){
		//最も近い線分に追加する
		double d = -1;
		int target_i = 0;
		for(int i = 0; i < point_list().size() - 1; i ++){
			Line2D.Float line = new Line2D.Float(point_list().get(i), point_list().get(i + 1));
			if(d < 0 || line.ptSegDist(p) < d){
				d = line.ptSegDist(p);
				target_i = i;
			}
		}
		
		data.add_trans_middle_point(model, p, org_point_list().get(target_i));
	}
	
	private void move_point(Point p, int dx, int dy){
		if(p == org_src_point()){
			data.move_trans_src_point(model, dx, dy);
		}
		else if(p == org_dest_point()){
			data.move_trans_dest_point(model, dx, dy);
		}
		else{
			data.move_trans_middle_point(model, p, dx, dy);			
		}
	}
	
	public Point get_point(int i){
		if(0 <= i && i < point_list().size()) return point_list().get(i);
		return null;
	}

	public void connect_src(){
		//遷移のsrcが状態の上に乗っているか？
		Canvas_State src_cs = canvas.get_hit_state(src_point());
		//乗っていれば状態に接続する
		set_src_state(src_cs);
	}
	
	protected void connect_src_if_other_state(){
		//遷移のsrcが状態の上に乗っているか？
		Canvas_State src_cs = canvas.get_hit_state(src_point());
		//乗っていれば状態に接続する
		set_src_state_if_other_state(src_cs);
	}
	
	public void connect_dest(){
		//遷移のdestが状態の上に乗っているか？
		Canvas_State dest_cs = canvas.get_hit_state(dest_point());
		//乗っていれば状態に接続する
		set_dest_state(dest_cs);
	}
	
	protected void connect_dest_if_other_state(){
		//遷移のdestが状態の上に乗っているか？
		Canvas_State dest_cs = canvas.get_hit_state(dest_point());
		//乗っていれば状態に接続する
		set_dest_state_if_other_state(dest_cs);
	}
	
	public void mouseReleased(MouseEvent e, boolean is_multi_focused){
		if(mode == MODE_FOCUSED_LABEL){
      		label.mouseReleased(e);
    	}
		else{
			if(is_draged == false){
				if(mode == MODE_ADDING){
					connect_src();
					connect_dest();
				}
				else{
					//何もしない
				}
			}
			else{
				if(is_multi_focused == false){
					//遷移全体を動かしたとき
					if(focused_point == null){
						connect_src();
						connect_dest();
					}
					else{
						//srcを動かしたとき
						if(focused_point == org_src_point()){
							connect_src();
						}
						//destを動かしたとき
						else if(focused_point == org_dest_point()){
							connect_dest();
						}
						//その他のコントロールポイントを動かしたとき
						else{
							//何もしない
						}
					}
				}
				else{
					Canvas_State src_cs = canvas.get_hit_state(src_point());
					if(src_cs != src_state) set_src_state(null);
					else connect_src();
					
					Canvas_State dest_cs = canvas.get_hit_state(dest_point());
					if(dest_cs != dest_state) set_dest_state(null);
					else connect_dest();
				}
				
			}
			focused_point = null;
		}
		
		set_cursor(e, is_multi_focused);
	}

    public void mousePressed(MouseEvent e, boolean is_multi_focused){
    	if(label.isHit(e.getPoint()) == true && is_multi_focused == false){
      		label.mousePressed(e);
      		setMode(MODE_FOCUSED_LABEL);
    	}
    	else{
    		is_draged = false;
	    	mouse_last_pos = e.getPoint();
	    	setMode(MODE_FOCUSED_LINE);
	    	if(is_multi_focused == true) focused_point = null;
	    	else focused_point = getHitPoint(e.getPoint());
    	}
    	
    	set_cursor(e, is_multi_focused);
    }
    
    private boolean isMiddlePoint(Point p){
    	if(p == src_point()) return false;
    	if(p == dest_point()) return false;
    	return true;
    }
  
    public void mouseClicked(MouseEvent e, boolean is_multi_focused){
    	if(e.getClickCount() == 1) click_num = 1;
    	else click_num ++;
    	
		switch(click_num){
    	//クリック
    	case 1 :
    		if(mode == MODE_FOCUSED_LABEL){
          		label.mouseClicked(e);
        	}
        	else{
	    		//+ ALT
	    		if(e.isAltDown()){
	    			Point p = getHitPoint(e.getPoint());
	    			if(p == null){
	    				if(model.get_point_list().size() < MAX_POINTS){
		    				//コントロールポイント追加
		    				add_point(e.getPoint());
	    				}
	    			}
	    			else{
	    				if(isMiddlePoint(p)){
	    					//コントロールポイント削除
	    					data.del_trans_middle_point(model, p);
	    				}
	    			}
	    		}
	    		else{
	   
	    		}
    		}
    		break;
    	//ダブルクリック
    	case 2 :
    		if(model.is_connect_point_src() == false){
    			canvas.get_root_frame().show_trans_dialog(model);
    		}
    		break;
    	}
		
		set_cursor(e, is_multi_focused);
    }

	public void mouseEntered(MouseEvent e, boolean is_multi_focused){}
	public void mouseExited(MouseEvent e, boolean is_multi_focused){}
	
	public void mouseDragged(MouseEvent e, boolean is_multi_focused){
		click_num = 0;
		
		if(mode == MODE_FOCUSED_LABEL){
      		label.mouseDragged(e);
    	}
		else{
			if(focused_point == null){
				//全体を移動
				for (int i = 0; i < point_list().size(); i ++){
					move_point(org_point_list().get(i), e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
					connect_src_if_other_state();
					connect_dest_if_other_state();
				}
			}
			else{
				//focused_pointのみを移動
				move_point(focused_point, e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
				//srcを動かしたとき
				if(focused_point == org_src_point()){
					connect_src_if_other_state();
				}
				//destを動かしたとき
				else if(focused_point == org_dest_point()){
					connect_dest_if_other_state();
				}
				//その他のコントロールポイントを動かしたとき
				else{
					//何もしない
				}
			}
			mouse_last_pos = e.getPoint();
			is_draged = true;
		}
		
		set_cursor(e, is_multi_focused);
	}
	
	private void set_cursor(MouseEvent e, boolean is_multi_focused){
		if(getHitPoint(e.getPoint()) != null){
			canvas.request_set_cursor(Cursor.MOVE_CURSOR);
		}
		else if(isHit(e.getPoint()) == true){
			canvas.request_set_cursor(Cursor.CROSSHAIR_CURSOR);
		}
	}
	
	public void mouseMoved(MouseEvent e, boolean is_multi_focused){
		click_num = 0;
		
		set_cursor(e, is_multi_focused);
	}
	
	/******* private *******/
	private String pred(){
		return model.get_pred();
	}
	
	private Model_State src_state(){
		return model.get_src_state();
	}
	
	private Model_State dest_state(){
		return model.get_dest_state();
	}
	
	private ArrayList<Point> point_list(){
		return model.get_point_list();
	}
	
	private ArrayList<Point> org_point_list(){
		return model.get_org_point_list();
	}
	
	private Point src_point(){
		return model.get_src_point();
	}
	
	private Point dest_point(){
		return model.get_dest_point();
	}
	
	private Point org_src_point(){
		return model.get_org_src_point();
	}
	
	private Point org_dest_point(){
		return model.get_org_dest_point();
	}
	
	/*** spline ***/

	class Curve{
		int npt;
		Point2D.Double [] pt;
		Point2D.Double [] pv;
		double [] p;
		double [] ax;
		double [] bx;
		double [] cx;
		double [] ay;
		double [] by;
		double [] cy;
		
		Curve(){
			npt = 0;
			pt = new Point2D.Double[LOCAL_MAX_POINTS];
			pv = new Point2D.Double[3 * (LOCAL_MAX_POINTS - 1) + 1];
			p = new double[LOCAL_MAX_POINTS];
			ax = new double[LOCAL_MAX_POINTS];
			bx = new double[LOCAL_MAX_POINTS];
			cx = new double[LOCAL_MAX_POINTS];
			ay = new double[LOCAL_MAX_POINTS];
			by = new double[LOCAL_MAX_POINTS];
			cy = new double[LOCAL_MAX_POINTS];
		}
	};

	Curve curve;
/*	
	void update_curve(){
		Point cur_p = model.get_point_list().get(0);
		curve.pt[0] = new Point2D.Double(cur_p.x, cur_p.y);
		curve.npt = 1;
				
		for (int i = 0; i < model.get_point_list().size(); ++i){
			if(cur_p.distance(model.get_point_list().get(i)) > 0){
				curve.pt[curve.npt] = new Point2D.Double(model.get_point_list().get(i).x, model.get_point_list().get(i).y);
				curve.npt ++;
			}
		}

		if(curve.npt >= 2) calc_curve();
	}
*/	
	void update_curve(){
		ArrayList<Point> point_list = model.get_point_list();
		
		if(get_src_state() != null){
			point_list.add(0, get_src_state().get_center_pos());
		}
		if(get_dest_state() != null){
			point_list.add(point_list.size(), get_dest_state().get_center_pos());
		}
		
		Point cur_p = point_list.get(0);
		curve.pt[0] = new Point2D.Double(cur_p.x, cur_p.y);
		curve.npt = 1;
				
		for (int i = 1; i < point_list.size(); ++i){
			if(cur_p.distance(point_list.get(i)) > 0){
				curve.pt[curve.npt] = new Point2D.Double(point_list.get(i).x, point_list.get(i).y);
				curve.npt ++;
			}
			cur_p = point_list.get(i);
		}

		if(curve.npt >= 2) calc_curve();
	}
	
	void calc_curve()
	{
		int i;
		double [] x = new double[LOCAL_MAX_POINTS];
		double [] y = new double[LOCAL_MAX_POINTS];
		int [] xi = new int[3 * (LOCAL_MAX_POINTS - 1) + 1];
		int [] yi = new int[3 * (LOCAL_MAX_POINTS - 1) + 1];

		for (i = 0; i < curve.npt; ++i)
		{
			x[i] = curve.pt[i].x;
			y[i] = curve.pt[i].y;
		}

		maketable2(curve.npt, curve.p, x, y, curve.bx, curve.by);
		maketable3(curve.npt, curve.p, x, curve.bx, curve.ax, curve.cx, xi);
		maketable3(curve.npt, curve.p, y, curve.by, curve.ay, curve.cy, yi);

		for (i = 0; i < 3 * (curve.npt - 1) + 1; ++i){
			curve.pv[i] = new Point2D.Double(xi[i], yi[i]);
		}
	}
	
	void maketable(int n, double p[], double x[], double b[]){
		int i;
		double t;
		double [] h = new double[LOCAL_MAX_POINTS];
		double [] d = new double[LOCAL_MAX_POINTS];

		b[0] = 0;  b[n - 1] = 0;
	    for (i = 0; i < n - 1; i++) {
			h[i    ] =  p[i + 1] - p[i];
			d[i + 1] = (x[i + 1] - x[i]) / h[i];
		}
		b[1] = d[2] - d[1] - h[0] * b[0];
		d[1] = 2 * (p[2] - p[0]);
		for (i = 1; i < n - 2; i++) {
			t = h[i] / d[i];
			b[i + 1] = d[i + 2] - d[i + 1] - b[i] * t;
			d[i + 1] = 2 * (p[i + 2] - p[i]) - h[i] * t;
		}
		b[n - 2] -= h[n - 2] * b[n - 1];
		for (i = n - 2; i > 0; i--)
			b[i] = (b[i] - h[i] * b[i + 1]) / d[i];
	}

	
	void maketable2(int n, double p[], double x[], double y[], double bx[], double by[]){
		int i;
		double t1, t2;

		p[0] = 0;
		for (i = 1; i < n; i++) {
			t1 = x[i] - x[i - 1];
			t2 = y[i] - y[i - 1];
			p[i] = p[i - 1] + Math.sqrt(t1 * t1 + t2 * t2);
		}
		for (i = 1; i < n; i++) p[i] /= p[n - 1];
		maketable(n, p, x, bx);
		maketable(n, p, y, by);
	}

	void maketable3(int n, double p[], double x[], double b[], double a[], double c[], int v[]){
		int i;
		double d, h;

		for (i = 0; i < n - 1; i++)
		{
			h = p[i + 1] - p[i];
			d = x[i + 1] - x[i];

			a[i] = (b[i + 1] - b[i]) / h;
			c[i] = (d / h - (2 * b[i] + b[i + 1]) * h) / 3;

			v[3 * i] = (int)(x[i]);
			v[3 * i + 1] = (int)(c[i] * h + x[i]);
			v[3 * i + 2] = (int)(b[i] * h * h + 2 * c[i] * h + x[i]);
		}

		v[3 * (n - 1)] = (int)(x[n - 1]);
	}
/*
	private void PolyBezier(Graphics2D g, Point2D.Double [] pt, int point_num){
		GeneralPath p = new GeneralPath();
		
		if(curve.npt < 2) return;
		
		p.moveTo(pt[0].x, pt[0].y);
	    
		for(int i = 1; i < point_num; i += 3){
			p.curveTo(pt[i].x, pt[i].y, pt[i+1].x, pt[i+1].y, pt[i+2].x, pt[i+2].y);
		}
	    g.draw(p);
	}
*/	
	private void PolyBezier(Graphics2D g, Point2D.Double [] pt, int point_num){
		GeneralPath p = new GeneralPath();
		
		int start_inxex = 0;
		int last_index = point_num;
		
		if(get_src_state() != null){
			start_inxex = 3;
		}
		if(get_dest_state() != null){
			last_index = last_index - 3;
		}
		
		if(curve.npt < 2) return;
		
		p.moveTo(pt[start_inxex].x, pt[start_inxex].y);
	    
		for(int i = start_inxex + 1; i < last_index; i += 3){
			p.curveTo(pt[i].x, pt[i].y, pt[i+1].x, pt[i+1].y, pt[i+2].x, pt[i+2].y);
		}
	    g.draw(p);
	}
	
	boolean sprine_is_hit(Point p){
		int i;
		int start_inxex = 0;
		int last_index = curve.npt - 1;
		
		if(get_src_state() != null){
			start_inxex = 1;
		}
		if(get_dest_state() != null){
			last_index = last_index - 1;
		}

		for (i = start_inxex; i < last_index; ++i){
			if (near_point_arc(i, p)){
				return true;
			}
		}

		return false;
	}
	
	boolean near_point_arc(int i, Point p){
		double dx_min, dy_min;

		dx_min = min_dist(p.x, (int)curve.pt[i].x, curve.p[i], curve.p[i + 1], curve.ax[i], curve.bx[i], curve.cx[i]);
		dy_min = min_dist(p.y, (int)curve.pt[i].y, curve.p[i], curve.p[i + 1], curve.ay[i], curve.by[i], curve.cy[i]);

		if (dx_min > HIT_MARGIN_LINE * 2 || dy_min > HIT_MARGIN_LINE * 2)
		{
			return false;
		}

		return min_dist_curve(HIT_MARGIN_LINE, p.x, p.y,
								curve.pt[i].x, curve.pt[i].y,
								curve.p[i], curve.p[i + 1],
								curve.ax[i], curve.bx[i], curve.cx[i],
								curve.ay[i], curve.by[i], curve.cy[i]);
	}

	boolean min_dist_curve(double d_min, double xc, double yc, double x0, double y0, double p0, double p1, double ax, double bx, double cx, double ay, double by, double cy){
		double h, ph, x, y, dx, dy, dr;

		h = p1 - p0;

		for (ph = 0; ph <= h; ){
			x = x0 + 3 * cx * ph + 3 * bx * ph * ph + ax * ph * ph * ph;
			y = y0 + 3 * cy * ph + 3 * by * ph * ph + ay * ph * ph * ph;

			dx = x - xc;
			dy = y - yc;
			dr = Math.sqrt(dx * dx + dy * dy);
			if (dr <= d_min){
				return true;
			}
			dx = 3 * (cx + 2 * bx * ph + ax * ph * ph);
			dy = 3 * (cy + 2 * by * ph + ay * ph * ph);
			dr = Math.sqrt(dx * dx + dy * dy);
			if (dr < 100){
				dr = 100;
			}
			ph += 1 / dr;
		}

		return false;
	}

	double min_dist(int x, int x0, double p0, double p1, double a, double b, double c){
		double A, B, C, D, p, pa, pb, q0, q1, q2, q3, h, h2;

		h = p1 - p0;
		q0 = x0 - x;
		q1 = x0 + 3 * c * h + 3 * b * h * h + a * h * h * h - x;

		if (q0 * q1 <= 0)
		{
			return 0;
		}

		if (a == 0)
		{
			if (b == 0)
			{
				return Math.min(Math.abs(q0), Math.abs(q1));
			}
			else
			{
				h2 = - c / (2 * b);
				p = p0 + h2;
				q2 = x0 + 3 * c * h2 + 3 * b * h2 * h2 + a * h2 * h2 * h2 - x;
				if (p >= p0 && p <= p1)
				{
					if (q0 * q2 <= 0)
					{
						return 0;
					}
					else
					{
						return Math.min(Math.abs(q2), Math.min(Math.abs(q0), Math.abs(q1)));
					}
				}
				else
				{
					return Math.min(Math.abs(q0), Math.abs(q1));
				}
			}
		}
		else
		{
			A = a;
			B = b - a * p0;
			C = c - 2 * b * p0 + a * p0 * p0;

			D = B * B - A * C;
			if (D >= 0)
			{
				pa = (-B + Math.sqrt(D)) / A;
				pb = (-B - Math.sqrt(D)) / A;

				h2 = pa - p0;
				q2 = x0 + 3 * c * h2 + 3 * b * h2 * h2 + a * h2 * h2 * h2 - x;
				h2 = pb - p0;
				q3 = x0 + 3 * c * h2 + 3 * b * h2 * h2 + a * h2 * h2 * h2 - x;

				if (q0 * q2 <= 0 || q0 * q3 <= 0)
				{
					return 0;
				}
				else
				{
					return Math.min(Math.min(Math.abs(q0), Math.abs(q1)), Math.min(Math.abs(q2), Math.abs(q3)));
				}
			}
			else
			{
				return Math.min(Math.abs(q0), Math.abs(q1));
			}
		}
	}

	
}
