import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.geom.*;
import java.util.ArrayList;
import java.io.Serializable;

import javax.swing.JOptionPane;


public class Canvas_Trans_Label{	
	static final int BG_MARGIN_W = 3;
	static final int BG_MARGIN_H = 2;
	static final Color TEXT_COLOR = Color.BLACK;
	static final Color BG_COLOR = new Color(240, 255, 240);
	static final Color EDGE_COLOR = Color.DARK_GRAY;
	protected static final int FONT_SIZE = 12;
	
	private Model_Trans model;
	private Model_Root data;
	
	int bg_w;
	int bg_h;
	Point mouse_last_pos;
	Canvas_Trans parent;
	MyCanvas canvas; 
	
	Canvas_Trans_Label(MyCanvas _canvas, Canvas_Trans _parent, Model_Root _data, Model_Trans _model){
		canvas = _canvas;
		parent = _parent;
		data = _data;
		model = _model;
		
		bg_w = -1;
		bg_h = -1;
		mouse_last_pos = null;
	}
	
	public void dispose(){
		canvas = null;
		parent = null;
		data = null;
		model = null;
		
		bg_w = -1;
		bg_h = -1;
		mouse_last_pos = null;
	}
	
	public void move_pos(int dx, int dy){
		data.move_trans_label_offset(model, dx, -dy);
	}
	
	protected float scale(){
		return canvas.get_scale();
	}
	
	protected int scaled(int x){
		return (int)(x * scale());
	}
	
	protected Point scaled_p(Point _p){
		return new Point(scaled(_p.x), scaled(_p.y));
	}
	
	protected float scaled_f(float x){
		return x * scale();
	}
	
	protected int bg_margin_w(){
		return scaled(BG_MARGIN_W);
	}
	
	protected int bg_margin_h(){
		return scaled(BG_MARGIN_H);
	}
	
	protected int font_size(){
		return scaled(FONT_SIZE);
	}
	
	public void paint(Graphics g){
		if(model.is_connect_point_src() == true) return;
			
		String text = label();
		
		Font font = new Font(canvas.get_font_name(), g.getFont().getStyle(), scaled(canvas.get_org_font_size()));
		Graphics2D g2 = (Graphics2D)g;
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		//文字列サイズを取得
		int text_w = canvas.multiline_string_width(g2, text);
		int text_h = canvas.multiline_string_height(g2, text);
				
		bg_w = text_w + bg_margin_w() * 2;		
		bg_h = text_h + bg_margin_h() * 2;		
		
		//接続線を描く
		g2.setColor(EDGE_COLOR);
		BasicStroke stroke2;
		stroke2 = new BasicStroke(scaled_f(1.0f),		// 太さ
                BasicStroke.CAP_BUTT,		// 端は残す
                BasicStroke.JOIN_MITER,		// 継ぎ目は斜め
                1.0f,						// 継ぎ目の限界
                new float[] {scaled_f(5.0f), scaled_f(5.0f)},	// 5書いて5書かないダッシュ
                0.0f);						// ダッシュの開始フェーズ
		g2.setStroke(stroke2); //線の種類を設定
		g2.drawLine(base_p().x, base_p().y, base_p().x + offset_x() + bg_margin_w() + bg_w / 2, base_p().y - offset_y() - bg_h / 2);
		
		//背景を描く
		g2.setColor(BG_COLOR);
		g2.fillRect(base_p().x + offset_x(), base_p().y - offset_y() - bg_h, bg_w, bg_h);
		g2.setColor(EDGE_COLOR);
		g2.setStroke(new BasicStroke(scaled_f(1.0f)));
		g2.drawRect(base_p().x + offset_x(), base_p().y - offset_y() - bg_h, bg_w, bg_h);
		
		//テキストを描く
		g2.setColor(TEXT_COLOR);
		//g2.drawString(text, base_p().x + offset_x() + bg_margin(), base_p().y - offset_y() - descent);	

		canvas.draw_multiline_string(g2, new Point(base_p().x + offset_x() + bg_margin_w(), base_p().y - offset_y() - text_h - bg_margin_h()), text, false);
	}
	
	public Point max_point(){
		return new Point(base_p().x + offset_x() + bg_w, base_p().y - offset_y());
	}
	
	public boolean isHit(Point p){
		if(model.is_connect_point_src() == true) return false;
		
		if(base_p().x + offset_x() <= p.x &&
		   p.x < base_p().x + offset_x() + bg_w &&
		   base_p().y - offset_y() - bg_h <= p.y &&
		   p.y < base_p().y - offset_y()
		){
			return true;
		}
		
		return false;
	}
	
	public void mouseReleased(MouseEvent e){
		mouse_last_pos = null;
		
		set_cursor(e);
	}

    public void mousePressed(MouseEvent e){
    	mouse_last_pos = e.getPoint();
    	
    	set_cursor(e);
    }
   
    public void mouseClicked(MouseEvent e){
    	switch(e.getClickCount()){
    	//クリック
    	case 1 :
    		break;
    	//ダブルクリック
    	case 2 :
    		break;
    	}
    	
    	set_cursor(e);
    }

	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	
	public void mouseDragged(MouseEvent e){
		move_pos(e.getPoint().x - mouse_last_pos.x, e.getPoint().y - mouse_last_pos.y);
		mouse_last_pos = e.getPoint();
		set_cursor(e);
	}
	public void mouseMoved(MouseEvent e){
		set_cursor(e);
	}
	
	private void set_cursor(MouseEvent e){
		if(isHit(e.getPoint()) == true){
			canvas.request_set_cursor(Cursor.CROSSHAIR_CURSOR);
		}
	}
	
	
	/******* private *******/
	private Point base_p(){
		return model.get_src_point();
	}
	private String label(){
		return model.get_label();
	}
	
	private int offset_x(){
		return model.get_label_offset_x();
	}
	
	private int offset_y(){
		return model.get_label_offset_y();
	}
}
