import java.awt.Event;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;
import javax.swing.undo.UndoManager;


public abstract class Dialog_Base extends JDialog {
	protected MyFrame frame = null;
	protected Model_Root model_root = null;
	
	Dialog_Base(MyFrame _frame, Model_Root _model_root, String _title){
		super(_frame, false);
		frame = _frame;
		model_root = _model_root;
		
		setTitle(_title);
		setLocationByPlatform(true);
		
		set_shortcut_key();		
	}
	
	protected void set_shortcut_key(){
		InputMap imap = getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_G, java.awt.event.InputEvent.CTRL_MASK), "hide");
		getRootPane().getActionMap().put("hide", new Shortcut_Action_T(frame, "hide"));
		
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK), "save");
		getRootPane().getActionMap().put("save", new Shortcut_Action_T(frame, "save"));
		
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK), "export_alloy");
		getRootPane().getActionMap().put("export_alloy", new Shortcut_Action_T(frame, "export_alloy"));
		
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Z, java.awt.event.InputEvent.CTRL_MASK), "undo");
		getRootPane().getActionMap().put("undo", new Shortcut_Action_T(frame, "undo"));
		
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Y, java.awt.event.InputEvent.CTRL_MASK), "redo");
		getRootPane().getActionMap().put("redo", new Shortcut_Action_T(frame, "redo"));
	}
	
	//画面レイアウトを作成する。
	abstract protected void create_view();
	
	//ダイアログに表示しているデータを更新する。
	abstract public void update_data();
	
	abstract protected void update_font();

	public Font get_font(){
		return new Font(frame.get_font_name(), Font.PLAIN, frame.get_org_font_size());
	}
	
	protected void init_undo_mgr(JTextComponent tc) {
		UndoManager manager = new UndoManager();
		init_undo_mgr(tc, manager);
	}
	
	protected void init_undo_mgr(JTextComponent tc, UndoManager manager) {
		tc.getDocument().addUndoableEditListener(manager);
		tc.getActionMap().put("undo", new UndoAction(manager));
		tc.getActionMap().put("redo", new RedoAction(manager));
		InputMap imap = tc.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.CTRL_MASK), "undo");
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Event.CTRL_MASK), "redo");
	}
	
	protected void text_update(JTextComponent tc, String new_text){
		if(tc.getText().equals(new_text) == false){
			tc.setText(new_text);
		}
	}
	
	protected void text_update(JLabel label, String new_text){
		if(label.getText().equals(new_text) == false){
			label.setText(new_text);
		}
	}
}


class Shortcut_Action_T extends AbstractAction{
	MyFrame frame;
	String type;
	
	Shortcut_Action_T(MyFrame _frame, String _type){
		frame = _frame;
		type = _type;
	}
	
	public void actionPerformed(ActionEvent e) {
		if(type.equals("hide")){
			frame.hide();
		}
		else if(type.equals("save")){
			frame.save_data_xml();
		}
		else if(type.equals("export_alloy")){
			frame.export_alloy();
		}
		else if(type.equals("undo")){
			frame.undo();
		}
		else if(type.equals("redo")){
			frame.redo();
		}
	}
}