import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.undo.UndoManager;


public class Dialog_Event_List extends Dialog_2Pain implements DocumentListener {
	protected JLabel l_name_title;
	protected JTextField tf_name;
	protected UndoManager tf_name_mgr;
	protected JLabel l_param_title;
	protected JTextField tf_param;
	protected UndoManager tf_param_mgr;
	
	protected Model_Event sel_event = null;
	
	protected boolean is_change_listener_enable = true;
	
	
	Dialog_Event_List(MyFrame _frame, Model_Root _model_root){
		super(_frame, _model_root, "イベントの一覧");
		
		create_view();
		update_data();
		
		this.repaint();
	}
	
	@Override
	protected void create_view() {
		super.create_view();
		this.setSize(500, 300);
		
		
		//event panel
		JPanel event_panel = new JPanel();
		event_panel.setLayout(new BorderLayout());
		panel_right.add(event_panel, BorderLayout.NORTH);
		{
			//event名
			JPanel name_panel = new JPanel();
			name_panel.setLayout(new BoxLayout(name_panel, BoxLayout.LINE_AXIS));
			event_panel.add(name_panel, BorderLayout.NORTH);
			{
				l_name_title = new JLabel("イベント名 : ", JLabel.LEFT);
				l_name_title.setAlignmentX(Component.LEFT_ALIGNMENT);
				name_panel.add(l_name_title);
				
				tf_name = new JTextField("");
				tf_name.getDocument().addDocumentListener(this);
				tf_name_mgr = new UndoManager();
				init_undo_mgr(tf_name, tf_name_mgr);
				name_panel.add(tf_name);
				
			}
			
			//param
			JPanel param_panel = new JPanel();
			param_panel.setLayout(new BoxLayout(param_panel, BoxLayout.LINE_AXIS));
			event_panel.add(param_panel, BorderLayout.SOUTH);
			{
				l_param_title = new JLabel("引数 : ", JLabel.LEFT);
				l_param_title.setAlignmentX(Component.LEFT_ALIGNMENT);
				param_panel.add(l_param_title);
				
				tf_param = new JTextField("");
				tf_param.getDocument().addDocumentListener(this);
				tf_param_mgr = new UndoManager();
				init_undo_mgr(tf_param, tf_param_mgr);
				param_panel.add(tf_param);	
			}
		}
		
		this.getRootPane().setDefaultButton(b_close);
	}
	
	@Override
	public void update_data() {
		is_change_listener_enable = false;
		list_left_model.clear();
		
		ArrayList<Model_Event> list_item = model_root.get_event_list();
	    for (int i = 0 ; i < list_item.size() ; i++){
	    	list_left_model.addElement(new String(list_item.get(i).get_full_name()));
	    }
	    
	    if(model_root.get_event_list().contains(sel_event)){
	    	list_left.setSelectedIndex(model_root.get_event_list().indexOf(sel_event));
	    }
	    else {
	    	sel_event = null;
	    }
		
	    is_change_listener_enable = true;
	    
	    update_right_view();
	    update_font();
	}
	
	public void show(){
		update_data();
		
		super.show();
	}
	
	protected void update_right_view(){
		Model_Event selected = get_selected_event();
		
		l_name_title.setVisible(selected != null);
		tf_name.setVisible(selected != null);
		l_param_title.setVisible(selected != null);
		tf_param.setVisible(selected != null);

	    if(selected != null){
	    	is_change_listener_enable = false;
	    	
	    	text_update(tf_name, selected.get_name());
	    	text_update(tf_param, selected.get_param());
	    	
    		is_change_listener_enable = true;
	    }
	}
	
	protected Model_Event get_selected_event(){
		int i = get_selected_event_index();
		
		if(model_root.get_event_list().size() > i && i >= 0){
			return model_root.get_event_list().get(i);
		}
		else return null;
	}
	
	protected int get_selected_event_index(){
		int[] index = list_left.getSelectedIndices();
		if(index.length == 1){
			return index[0];
		}
		else return -1;
	}
	
	public void set_model(Model_Root _model_root){
		model_root = _model_root;
		update_data();
	}
	
	public void show(Model_Event e){
		this.sel_event = e;
		
		update_data();
		super.setVisible(true);
	}
	
	@Override
	protected void update_font() {
		super.update_font();
		
		Font font = get_font();

		l_name_title.setFont(font);
		tf_name.setFont(font);
		l_param_title.setFont(font);
		tf_param.setFont(font);
	}

	@Override
	protected void select_add() {
		model_root.add_event(model_root.create_unique_event_name(), "");
		
		update_data();
		list_left.setSelectedIndex(list_left_model.getSize() - 1);
	}

	@Override
	protected void select_del() {
		int[] index = list_left.getSelectedIndices();
		if (index.length == 0){
			return;
		}
		else{
			model_root.remove_event(index);
		}
	}

	@Override
	protected void select_dup() {
		int[] index = list_left.getSelectedIndices();
		if (index.length > 0){
			ArrayList<Model_Event> list_item = model_root.get_event_list();
			
			for(int i = 0; i < index.length; i ++){
				model_root.add_event("copy_" + list_item.get(index[i]).get_name(), list_item.get(index[i]).get_param());
			}
		}
		else{
			return;
		}
	}

	@Override
	protected void select_left_list(int index) {
		if(is_change_listener_enable){
			sel_event = get_selected_event();

			change_another_event_view();
		}
	}
	
	protected void change_another_event_view(){
		update_right_view();
	    
		tf_name_mgr.discardAllEdits();
		tf_param_mgr.discardAllEdits();
	}

	public void changedUpdate(DocumentEvent e) {
		apply_text_change();
	}

	public void insertUpdate(DocumentEvent e) {
		apply_text_change();
	}

	public void removeUpdate(DocumentEvent e) {
		apply_text_change();
	}
	
	protected void apply_text_change() {
		if(is_change_listener_enable){
			Model_Event selected = get_selected_event();
			
			if(selected != null){
				boolean is_change = false;
				if(tf_name.getText().equals(selected.get_name()) == false){
					is_change = true;
				}
				if(tf_param.getText().equals(selected.get_param()) == false){
					is_change = true;
				}
				
				if(is_change){
					model_root.edit_event(selected, tf_name.getText(), tf_param.getText());
					if(frame != null){
						if(frame.get_canvas() != null) frame.get_canvas().repaint();
					}
				}
			}
		}
	}
	
	
}
