import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


public class Dialog_State extends Dialog_Base implements ActionListener, ItemListener{
	protected Model_State model = null;
	
	protected JLabel l_base_state = null;
	protected JCheckBox cb_is_rect = null;
	protected JButton b_close = null;
	protected JComboBox combo_base_state = null;
	protected JButton b_base_state_list = null;
	
	
	Dialog_State(MyFrame _frame, Model_Root _model_root, Model_State _model){
		super(_frame, _model_root, "状態");
		
		model = _model;
		create_view();
		update_data();
	}

	protected void apply_text_change() {
		if(cb_is_rect.isSelected() != (model.get_shape() == Model_State.SHAPE_RECT)){	
			if(cb_is_rect.isSelected()){
				model_root.set_state_shape(model, Model_State.SHAPE_RECT);
			}
			else{
				model_root.set_state_shape(model, Model_State.SHAPE_CIRCLE);
			}
			if(frame != null){
				if(frame.get_canvas() != null) {
					if(cb_is_rect.isSelected()){
						frame.get_canvas().change_state_shape(model, Model_State.SHAPE_RECT);
					}
					else{
						frame.get_canvas().change_state_shape(model, Model_State.SHAPE_CIRCLE);
					}
					
					frame.get_canvas().repaint();
				}
			}
		}
	}


	@Override
	protected void create_view() {
		this.setSize(400, 100);
		this.setLayout(new BorderLayout(0, 0));
		this.setBackground(Color.WHITE);
		
		//text_boxsパネルを配置
		JPanel text_boxs_panel = new JPanel();
		text_boxs_panel.setLayout(new BorderLayout());
		getContentPane().add(text_boxs_panel, BorderLayout.NORTH);
		{			
			//base_state のパネルを配置
			JPanel base_state_panel = new JPanel();
			base_state_panel.setLayout(new BorderLayout());
			text_boxs_panel.add(base_state_panel, BorderLayout.SOUTH);
			{
				l_base_state = new JLabel("状態 : ", JLabel.LEFT);
				l_base_state.setAlignmentX(Component.LEFT_ALIGNMENT);
				base_state_panel.add(l_base_state, BorderLayout.WEST);
				
				//base stateのドロップダウンリスト
				ArrayList<Model_Base_State> base_state_model_list = model_root.get_base_state_list();
				combo_base_state = new JComboBox();							
				combo_base_state.addItemListener(this);
				base_state_panel.add(combo_base_state, BorderLayout.CENTER);
				
				//Base Stateリストボタン
				b_base_state_list = new JButton("...");
				b_base_state_list.addActionListener(this);
				base_state_panel.add(b_base_state_list, BorderLayout.EAST);				
			}
		}

		//ボタンを配置
		JPanel button_panel = new JPanel();
		button_panel.setLayout(new BoxLayout(button_panel, BoxLayout.LINE_AXIS));
		getContentPane().add(button_panel, BorderLayout.SOUTH);
		{
			button_panel.add(Box.createRigidArea(new Dimension(5,1)));
					
			cb_is_rect = new JCheckBox("長方形", (model.get_shape() == Model_State.SHAPE_RECT));
			cb_is_rect.addActionListener(this);
			button_panel.add(cb_is_rect);
			
			button_panel.add(Box.createGlue());
			
			b_close = new JButton("閉じる");
			b_close.addActionListener(this);
			button_panel.add(b_close);
			
			button_panel.add(Box.createRigidArea(new Dimension(10,1)));
		}
		
		this.getRootPane().setDefaultButton(b_close);
	}


	@Override
	public void update_data() {
		combo_base_state.removeItemListener(this);
		{
			combo_base_state_update();
			combo_base_state.setSelectedIndex(model_root.get_base_state_list().indexOf(model.get_base_state()));
		}
		combo_base_state.addItemListener(this);

		if(cb_is_rect.isSelected() != (model.get_shape() == Model_State.SHAPE_RECT)){	
			cb_is_rect.setSelected(model.get_shape() == Model_State.SHAPE_RECT);
		}

		update_font();
	}
	
	public Model_State get_model(){
		return model;
	}
	
	public void set_model(Model_Root _model_root, Model_State _model){
		model_root = _model_root;
		model = _model;
		update_data();
	}
	
	protected void combo_base_state_update(){
		boolean is_change = false;
		ArrayList<Model_Base_State> base_state_model_list = model_root.get_base_state_list();
	
		if(base_state_model_list.size() == combo_base_state.getItemCount()){
			for(int i = 0; i < base_state_model_list.size(); i ++){
				if(base_state_model_list.get(i).get_state_name().equals(combo_base_state.getItemAt(i).toString().replaceAll(" \\(Root State\\)", "")) == false){
					is_change = true;
				}
			}
		}
		else is_change = true;
		
		if(is_change == true){
			combo_base_state.removeAllItems();
			for(int i = 0; i < base_state_model_list.size(); i ++){
				if(base_state_model_list.get(i).is_root_state()){
					combo_base_state.addItem(new String(base_state_model_list.get(i).get_state_name() + " (Root State)"));
				}
				else{
					combo_base_state.addItem(new String(base_state_model_list.get(i).get_state_name()));
				}
			}
		}
	}


	@Override
	protected void update_font() {
		Font font = get_font();
		
		combo_base_state.setFont(font);
		l_base_state.setFont(font);
		b_close.setFont(font);
		cb_is_rect.setFont(font);
		b_base_state_list.setFont(font);
		this.setFont(font);
	}


	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == b_close){
			setVisible(false);
		}
		else if(e.getSource() == cb_is_rect){
			apply_text_change();
		}
		else if(e.getSource() == b_base_state_list){
			frame.show_Dialog_Base_State_List(model.get_base_state());
		}
	}


	public void itemStateChanged(ItemEvent e) {
		if(e.getItemSelectable() == combo_base_state) {
			if (e.getStateChange() == ItemEvent.SELECTED){
				int k = combo_base_state.getSelectedIndex();
				if(model.get_base_state() != model_root.get_base_state_list().get(k)){
					model_root.set_state_base_state(model, model_root.get_base_state_list().get(k));
					frame.repaint();
				}
			}
		}
	}
	
}
