import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.DocumentListener;
import javax.swing.table.DefaultTableModel;


public class Dialog_State_Event_Matrix extends Dialog_Base implements ActionListener, Dialog_Progress_Listener {
	JTable table;
	DefaultTableModel table_model;

	JButton b_close;
	
	volatile Dialog_Progress d_progress = null;
	volatile Boolean is_canceled = false;
	
	Dialog_State_Event_Matrix(MyFrame _frame, Model_Root _model_root){
		super(_frame, _model_root, "状態遷移表");
		
		create_view();
		update_data();
	}

	@Override
	protected void create_view() {
		d_progress = new Dialog_Progress(this);
		
		this.setSize(400, 200);	
		this.setLayout(new BorderLayout(0, 0));
		this.setBackground(Color.WHITE);
		
		//テーブル本体
		table = new JTable();
		table.setShowGrid(true);
		
		table.setGridColor(Color.GRAY);
		
		JScrollPane scroll_pane1 = new JScrollPane(table);
		getContentPane().add(scroll_pane1, BorderLayout.CENTER);
		
		//Closeボタンを配置
		JPanel button_panel = new JPanel();
		button_panel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		getContentPane().add(button_panel, BorderLayout.SOUTH);
		{
			b_close = new JButton("閉じる");
			b_close.addActionListener(this);
			button_panel.add(b_close);
		}
	}

	@Override
	public void update_data() {
		table_model = new DefaultTableModel();
		table.setModel(table_model);
		
		ArrayList<Model_Event> event_list = model_root.get_event_list();
		ArrayList<Model_State> state_list = model_root.get_state_list();
		
		String[] state_name_list = new String[state_list.size()];
		for(int i = 0; i < state_list.size(); i ++){
			state_name_list[i] = state_list.get(i).get_base_state().get_label_or_name();
		}
		table_model.addColumn("状態名", state_name_list);
		
		for(int i = 0; i < event_list.size(); i ++){
			table_model.addColumn(event_list.get(i).get_full_name());
		}
		
		update_font();
	}

	@Override
	protected void update_font() {
		Font font = get_font();
		
		b_close.setFont(font);
		table.setFont(font);
		this.setFont(font);
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == b_close){
			setVisible(false);
		}
	}
	
	public void set_model(Model_Root _model_root){
		model_root = _model_root;
		update_data();
	}
	
	protected void instance_check(){
		is_canceled = false;
		MyThread thread = new MyThread();
		thread.start();
		d_progress.setTitle("状態遷移表");
		d_progress.set_message("作成中...");
		d_progress.setVisible(true);
		is_canceled = false;
	}
	
	protected void instance_check_local(){
		//create check alloy
		String alloy = model_root.get_alloy();
		
		ArrayList<Model_Event> event_list = model_root.get_event_list();
		ArrayList<Model_State> state_list = model_root.get_state_list();
		
		for(int i = 0; i < state_list.size(); i ++){		
			Model_State state = state_list.get(i);
			
			for(int j = 0; j < event_list.size(); j ++){
				if(is_canceled) {
					d_progress.hide_after_show();
					return;
				}
				
				d_progress.set_message("作成中... " + (event_list.size() * i + j) * 100 / (state_list.size() * event_list.size()) + " %");
				
				Model_Event event = event_list.get(j);
				alloy += get_alloy_pred(i, j + 1, state, event);
				
				Alloy_result ret = new Alloy_result();
				ret = frame.exec_alloy(alloy, "command", get_alloy_pred_name(i, j + 1, state, event));
				
				if(ret.ret == Alloy_result.SYNTAX_TYPE_ERROR){
					d_progress.hide_after_show();
					frame.show_Dialog_Error_Alloy("構文・型チェック NG", alloy, ret.ret_text);
					return;
				}
				else if(ret.ret == Alloy_result.CHECK_OK){
					table_model.setValueAt("◯", i, j + 1);
				}
				else if(ret.ret == Alloy_result.NO_INSTANCE_FOUND){
					table_model.setValueAt("×", i, j + 1);
				}
				
				this.repaint();
			}
		}
		
		d_progress.hide_after_show();
	}
	
	protected String instance_check_local(String alloy, String command){
		Alloy_result ret = new Alloy_result();
		ret = frame.exec_alloy(alloy, "command", command);
		
		if(ret.ret == Alloy_result.SYNTAX_TYPE_ERROR){
			frame.show_Dialog_Error_Alloy("構文・型チェック NG", alloy, ret.ret_text);
		}
		
		return ret.ret_text;
	}
	
	protected String get_alloy_pred(int row, int col, Model_State s, Model_Event e){
		String alloy = "";
		
		alloy += "pred tmp_" + row + "_" + col + e.get_alloy_param_def();
		alloy += "\t" + e.get_name() + " [s, s'" + e.get_param_names() + "]\n";
		alloy += "\t" + "s in " + s.get_base_state_name() + "\n";
		alloy += "}\n";
		alloy += "run tmp_" + row + "_" + col + " for 3\n\n";
		
		return alloy;
	}
	
	protected String get_alloy_pred_name(int row, int col, Model_State s, Model_Event e){
		return "tmp_" + row + "_" + col;
	}
	
	public void progress_cancel_clicked() {
		is_canceled = true;
		frame.cancel_exec_alloy();
	}
	
	class MyThread extends Thread{	
		public void run() {
			instance_check_local();
		}
	}
}
