import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.geom.Rectangle2D;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import java.awt.dnd.*;

public class DnDList extends JList implements DragGestureListener, DragSourceListener, Transferable {
    private static final String NAME = "test";
    private static final Color lineColor = new Color(100,100,255);
    private final Rectangle2D targetLine = new Rectangle2D.Float();
    private int draggedIndex = -1;
    private int targetIndex  = -1;
    private DnDListChangeListener list_change_listener = null;
    private DnDList this_object = null;
    
    public DnDList() {
        super();
        //DropTarget dropTarget =
        new DropTarget(this, DnDConstants.ACTION_COPY_OR_MOVE, new CDropTargetListener(), true);
        //DragSource dragSource = new DragSource();
        new DragSource().createDefaultDragGestureRecognizer((Component)this, DnDConstants.ACTION_COPY_OR_MOVE, (DragGestureListener)this);
        
        this.setCellRenderer(new DefaultListCellRenderer() {
            private final Color ec = new Color(240, 240, 240);
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                super.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);
                if(isSelected) {
                    setForeground(list.getSelectionForeground());
                    setBackground(list.getSelectionBackground());
                }else{
                    setForeground(list.getForeground());
                    setBackground(list.getBackground());
                }
                return this;
            }
        });
        
        this_object = this;
    }
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if(targetIndex>=0) {
            Graphics2D g2 = (Graphics2D)g;
            g2.setPaint(lineColor);
            g2.fill(targetLine);
        }
    }
    private void initTargetLine(Point p) {
        Rectangle2D testArea = new Rectangle2D.Float();
        int cellHeight = (int) getCellBounds(0,0).getHeight();
        int lineWidht  = (int) getCellBounds(0,0).getWidth();
        int lineHeight = 2;
        int modelSize  = getModel().getSize();
        targetIndex = -1;
        for(int i=0;i<modelSize;i++) {
            testArea.setRect(0, cellHeight*i-(cellHeight/2), lineWidht, cellHeight);
            if(testArea.contains(p)) {
                targetIndex = i;
                targetLine.setRect(0, i*cellHeight, lineWidht, lineHeight);
                break;
            }
        }
        if(targetIndex<0) {
            targetIndex = modelSize;
            targetLine.setRect(0, targetIndex*cellHeight-lineHeight, lineWidht, lineHeight);
        }
    }
    
    // Interface: DragGestureListener
    public void dragGestureRecognized(DragGestureEvent e) {
        if(getSelectedIndices().length>1) return;
        draggedIndex = locationToIndex(e.getDragOrigin());
        if(draggedIndex<0) return;
        try{
            e.startDrag(DragSource.DefaultMoveDrop, (Transferable)this, (DragSourceListener)this);
        }catch(InvalidDnDOperationException idoe) { idoe.printStackTrace(); }
    }

    // Interface: DragSourceListener
    public void dragEnter(DragSourceDragEvent e) {
        e.getDragSourceContext().setCursor(DragSource.DefaultMoveDrop);
    }
    public void dragExit(DragSourceEvent e) {
        e.getDragSourceContext().setCursor(DragSource.DefaultMoveNoDrop);
    }
    public void dragOver(DragSourceDragEvent e) {}
    public void dropActionChanged(DragSourceDragEvent e) {}
    public void dragDropEnd(DragSourceDropEvent e) {}

    // Interface: Transferable
    private final DataFlavor FLAVOR = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType, NAME);
    public Object getTransferData(DataFlavor flavor) {
        return this;
    }
    public DataFlavor[] getTransferDataFlavors() {
        DataFlavor[] f = new DataFlavor[1];
        f[0] = this.FLAVOR;
        return f;
    }
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        return flavor.getHumanPresentableName().equals(NAME);
    }
    
    public void addDnDListChangeListener(DnDListChangeListener _list_change_listener){
    	list_change_listener = _list_change_listener;
    }

    class CDropTargetListener implements DropTargetListener{
        // DropTargetListener interface
        public void dragExit(DropTargetEvent e) {
            targetIndex = -1;
            repaint();
        }
        public void dragEnter(DropTargetDragEvent e) {
            if(isDragAcceptable(e)) {
                e.acceptDrag(e.getDropAction());
            }else{
                e.rejectDrag();
            }
        }
        public void dragOver(final DropTargetDragEvent e) {
            if(isDragAcceptable(e)) {
                e.acceptDrag(e.getDropAction());
            }else{
                e.rejectDrag();
                return;
            }
            initTargetLine(e.getLocation());
            repaint();
        }
        public void dropActionChanged(DropTargetDragEvent e) {
            // if(isDragAcceptable(e)) e.acceptDrag(e.getDropAction());
            // else e.rejectDrag();
        }
        public void drop(DropTargetDropEvent e) {
            DefaultListModel model = (DefaultListModel)getModel();
//             Transferable t = e.getTransferable();
//             DataFlavor[] f = t.getTransferDataFlavors();
//             try{
//                 Component comp = (Component) t.getTransferData(f[0]);
//             }catch(UnsupportedFlavorException ex) {
//                     e.dropComplete(false);
//             }catch(java.io.IOException ie) {
//                 e.dropComplete(false);
//             }
            if(isDropAcceptable(e)) {
                Object str = model.getElementAt(draggedIndex);
                if(targetIndex==draggedIndex) {
                    setSelectedIndex(targetIndex);
                }else if(targetIndex<draggedIndex) {
                    model.removeElementAt(draggedIndex);
                    model.insertElementAt(str, targetIndex);
                    setSelectedIndex(targetIndex);
                }else{
                    model.insertElementAt(str, targetIndex);
                    model.removeElementAt(draggedIndex);
                    setSelectedIndex(targetIndex-1);
                }
                e.dropComplete(true);
                list_change_listener.listSortChange(this_object, draggedIndex, targetIndex);
            }else{
                e.dropComplete(false);
            }
            e.dropComplete(false);
            targetIndex = -1;
            repaint();
        }
        public boolean isDragAcceptable(DropTargetDragEvent e) {
            DataFlavor[] f = e.getCurrentDataFlavors();
            return isDataFlavorSupported(f[0]);
        }
        public boolean isDropAcceptable(DropTargetDropEvent e) {
            Transferable t = e.getTransferable();
            DataFlavor[] f = t.getTransferDataFlavors();
            return isDataFlavorSupported(f[0]);
        }
    }
}


interface DnDListChangeListener{
	public void listSortChange(DnDList source, int from, int to);
}
