import java.awt.Point;
import java.io.Serializable;
import java.util.ArrayList;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class Model_Root implements Serializable {
	protected static final String DEFAULT_STATE_NAME = "State";
	protected static final String DEFAULT_EVENT_NAME = "event";

	protected Model_Global_Def g_state = null;
	protected ArrayList<Model_State> state_list = null;
	protected ArrayList<Model_Trans> trans_list = null;
	protected ArrayList<Model_Memo> memo_list = null;
	protected ArrayList<Model_Event> event_list = null;
	protected ArrayList<Model_Base_State> base_state_list = null;
	protected ArrayList<Model_Scenario> scenario_list = null;
	transient protected MyFrame frame = null;
	
	
	Model_Root(MyFrame _frame) {	
		g_state = new Model_Global_Def(this);
		state_list = new ArrayList<Model_State>();
		trans_list = new ArrayList<Model_Trans>();
		memo_list = new ArrayList<Model_Memo>();
		event_list = new ArrayList<Model_Event>();
		base_state_list = new ArrayList<Model_Base_State>();
		Model_Base_State root_st = new Model_Base_State(this, Model_Base_State.ROOT_STATE_NAME, true);
		base_state_list.add(root_st);
		scenario_list = new ArrayList<Model_Scenario>();
		frame = _frame;
	}
	
	Model_Root(MyFrame _frame, Document xml){
		g_state = new Model_Global_Def(this);
		state_list = new ArrayList<Model_State>();
		trans_list = new ArrayList<Model_Trans>();
		memo_list = new ArrayList<Model_Memo>();
		event_list = new ArrayList<Model_Event>();
		base_state_list = new ArrayList<Model_Base_State>();
		scenario_list = new ArrayList<Model_Scenario>();
		frame = _frame;
		
		parse_xml(xml);
	}
	
	public void set_frame(MyFrame _frame) {
		frame = _frame;
	}

	public void add_xml_data(Document xml) {
		parse_xml(xml);
	}

	public void set_scale(float s) {
		for (int i = 0; i < state_list.size(); i++) {
			state_list.get(i).set_scale(s);
		}

		for (int i = 0; i < trans_list.size(); i++) {
			trans_list.get(i).set_scale(s);
		}

		for (int i = 0; i < memo_list.size(); i++) {
			memo_list.get(i).set_scale(s);
		}
	}

	/******* Memoを変更 *******/
	public ArrayList<Model_Memo> get_memo_list() {
		return memo_list;
	}

	public void move_memo_last(Model_Memo m) {
		memo_list.remove(m);
		memo_list.add(m);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_memo_first(Model_Memo m) {
		memo_list.remove(m);
		memo_list.add(0, m);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_memo_pos(Model_Memo m, int dx, int dy) {
		m.move_pos(dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_memo_pos(Model_Memo m, Point p) {
		m.set_pos(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_memo(Model_Memo m, String text) {
		m.set_memo(text);

		// 変更通知
		frame.notify_update_data();
	}

	public void add_memo(Model_Memo m) {
		memo_list.add(m);

		// 変更通知
		frame.notify_update_data();
	}

	public void del_memo(Model_Memo m) {
		memo_list.remove(m);

		// 変更通知
		frame.notify_update_data();
	}

	/******* 状態を変更 *******/
	public void set_state_base_state(Model_State s, Model_Base_State _base){
		s.set_base_state(_base);
		
		frame.notify_update_data();
	}
	
	public void move_state_last(Model_State s) {
		state_list.remove(s);
		state_list.add(s);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_state_first(Model_State s) {
		state_list.remove(s);
		state_list.add(0, s);

		// 変更通知
		frame.notify_update_data();
	}

	public void add_state(Model_State s) {
		state_list.add(s);

		// 変更通知
		frame.notify_update_data();
	}

	public void del_state(Model_State s) {
		state_list.remove(s);

		for (int i = 0; i < trans_list.size(); i++) {
			trans_list.get(i).del_state(s);
		}

		// 変更通知
		frame.notify_update_data();
	}

	public void set_state_pos_with_trans(Model_State s, Point p) {
		int dx = p.x - s.get_pos().x;
		int dy = p.y - s.get_pos().y;

		move_state_with_trans(s, dx, dy);
	}

	public void set_state_pos_without_trans(Model_State s, Point p) {
		int dx = p.x - s.get_pos().x;
		int dy = p.y - s.get_pos().y;

		move_state_without_trans(s, dx, dy);
	}

	public void move_state_with_trans(Model_State s, int dx, int dy) {
		s.move_pos(dx, dy);

		// 接続されている遷移のポイントも移動
		for (int i = 0; i < trans_list.size(); i++) {
			trans_list.get(i).move_state(s, dx, dy);
		}

		// 変更通知
		frame.notify_update_data();
	}

	public void move_state_without_trans(Model_State s, int dx, int dy) {
		s.move_pos(dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_state_without_focused_trans(Model_State s, int dx, int dy,
			ArrayList<Model_Trans> focused_trans_list) {
		s.move_pos(dx, dy);

		// 接続されている遷移のポイントも移動
		for (int i = 0; i < trans_list.size(); i++) {
			if (focused_trans_list.contains(trans_list.get(i)) == false) {
				trans_list.get(i).move_state(s, dx, dy);
			}
		}

		// 変更通知
		frame.notify_update_data();
	}

	public void set_state_label(Model_Base_State s, String _label) {
		s.set_label(_label);

		// 変更通知
		frame.notify_update_data();
	}

	public ArrayList<Model_State> get_state_list() {
		return state_list;
	}

	public void set_state_r(Model_State s, int r) {
		s.set_r(r);

		// 接続されている遷移のポイントも移動
		for (int i = 0; i < get_trans_list().size(); i++) {
			Model_Trans t = get_trans_list().get(i);

			if (t.get_src_state() == s) {
				Point new_pos = new Point((int) (r * Math.cos(t
						.get_src_point_rad()))
						+ s.get_pos().x, (int) (r * Math.sin(t
						.get_src_point_rad()))
						+ s.get_pos().y);
				t.set_src_point(new_pos);
			}
			if (t.get_dest_state() == s) {
				Point new_pos = new Point((int) (r * Math.cos(t
						.get_dest_point_rad()))
						+ s.get_pos().x, (int) (r * Math.sin(t
						.get_dest_point_rad()))
						+ s.get_pos().y);
				t.set_dest_point(new_pos);
			}
		}

		// 変更通知
		frame.notify_update_data();
	}

	public void set_state_wh(Model_State s, int w, int h) {
		int old_w = s.get_w();
		int old_h = s.get_h();
		double scale_w = (double) w / old_w;
		double scale_h = (double) h / old_h;

		s.set_wh(w, h);

		// 接続されている遷移のポイントも移動
		for (int i = 0; i < get_trans_list().size(); i++) {
			Model_Trans t = get_trans_list().get(i);

			if (t.get_src_state() == s) {
				Point new_pos = new Point();
				Point p = t.get_src_point();

				// 上辺
				if (p.y == s.get_pos().y - old_h / 2) {
					new_pos.x = (int) ((p.x - s.get_pos().x) * scale_w + s
							.get_pos().x);
					new_pos.y = s.get_pos().y - h / 2;
				}
				// 下辺
				else if (p.y == s.get_pos().y + old_h / 2) {
					new_pos.x = (int) ((p.x - s.get_pos().x) * scale_w + s
							.get_pos().x);
					new_pos.y = s.get_pos().y + h / 2;
				}
				// 右辺
				else if (p.x == s.get_pos().x + old_w / 2) {
					new_pos.x = s.get_pos().x + w / 2;
					new_pos.y = (int) ((p.y - s.get_pos().y) * scale_w + s
							.get_pos().y);
				}
				// 左辺
				else {
					new_pos.x = s.get_pos().x - w / 2;
					new_pos.y = (int) ((p.y - s.get_pos().y) * scale_w + s
							.get_pos().y);
				}
				t.set_src_point(new_pos);
			}
			if (t.get_dest_state() == s) {
				Point new_pos = new Point();
				Point p = t.get_dest_point();

				// 上辺
				if (p.y == s.get_pos().y - old_h / 2) {
					new_pos.x = (int) ((p.x - s.get_pos().x) * scale_w + s
							.get_pos().x);
					new_pos.y = s.get_pos().y - h / 2;
				}
				// 下辺
				else if (p.y == s.get_pos().y + old_h / 2) {
					new_pos.x = (int) ((p.x - s.get_pos().x) * scale_w + s
							.get_pos().x);
					new_pos.y = s.get_pos().y + h / 2;
				}
				// 右辺
				else if (p.x == s.get_pos().x + old_w / 2) {
					new_pos.x = s.get_pos().x + w / 2;
					new_pos.y = (int) ((p.y - s.get_pos().y) * scale_w + s
							.get_pos().y);
				}
				// 左辺
				else {
					new_pos.x = s.get_pos().x - w / 2;
					new_pos.y = (int) ((p.y - s.get_pos().y) * scale_w + s
							.get_pos().y);
				}

				t.set_dest_point(new_pos);
			}

		}

		// 変更通知
		frame.notify_update_data();
	}

	public void set_state_shape(Model_State s, int _shape) {
		s.set_shape(_shape);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_state_comment(Model_State s, String _label) {
		s.set_comment(_label);

		// 変更通知
		frame.notify_update_data();
	}

	/******* 遷移を変更 *******/
	public void trans_set_change_valiable(Model_Trans t, ArrayList<String> list){		
		t.set_change_valiable(list);
		// 変更通知
		frame.notify_update_data();
	}
	
	public void trans_add_change_valiable(Model_Trans t, String v){	
		t.add_change_valiable(v);
		// 変更通知
		frame.notify_update_data();
	}
	
	public void trans_del_change_valiable(Model_Trans t, String v){	
		t.del_change_valiable(v);
		// 変更通知
		frame.notify_update_data();
	}
	
	public void move_trans_last(Model_Trans t) {
		trans_list.remove(t);
		trans_list.add(t);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_trans_first(Model_Trans t) {
		trans_list.remove(t);
		trans_list.add(0, t);

		// 変更通知
		frame.notify_update_data();
	}

	public void add_trans(Model_Trans t) {
		trans_list.add(t);

		// 変更通知
		frame.notify_update_data();
	}

	public void del_trans(Model_Trans t) {
		trans_list.remove(t);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_trans_middle_point(Model_Trans t, Point p, int dx, int dy) {
		t.move_middle_point(p, dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void connect_trans_dest_point(Model_Trans t, Model_State dest,
			Point p) {
		// 接続角度計算
		double rad = Math.atan2(p.y - dest.get_pos().y, p.x - dest.get_pos().x);
		t.connect_dest_point(dest, p, rad);

		// 変更通知
		frame.notify_update_data();
	}

	public void connect_trans_src_point(Model_Trans t, Model_State src, Point p) {
		// 接続角度計算
		double rad = Math.atan2(p.y - src.get_pos().y, p.x - src.get_pos().x);
		t.connect_src_point(src, p, rad);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_dest_point(Model_Trans t, Point p) {
		t.set_dest_point(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_src_point(Model_Trans t, Point p) {
		t.set_src_point(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void unconnect_trans_dest_point(Model_Trans t) {
		t.unconnect_dest_point();

		// 変更通知
		frame.notify_update_data();
	}

	public void unconnect_trans_src_point(Model_Trans t) {
		t.unconnect_src_point();

		// 変更通知
		frame.notify_update_data();
	}

	public void move_trans_dest_point(Model_Trans t, int dx, int dy) {
		t.move_dest_point(dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_trans_src_point(Model_Trans t, int dx, int dy) {
		t.move_src_point(dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void add_trans_middle_point(Model_Trans t, Point new_p, Point prev_p) {
		t.add_middle_point(new_p, prev_p);

		// 変更通知
		frame.notify_update_data();
	}

	public void del_trans_middle_point(Model_Trans t, Point p) {
		t.del_middle_point(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void move_trans_label_offset(Model_Trans t, int dx, int dy) {
		t.move_label_offset(dx, dy);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_event(Model_Trans t, Model_Event _event) {
		t.set_event(_event);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_pred(Model_Trans t, Model_Event _event, String pred) {
		t.set_pred(_event, pred);

		// 変更通知
		frame.notify_update_data();
	}

	public ArrayList<Model_Trans> get_trans_list() {
		return trans_list;
	}

	public void set_trans_src_point_pos(Model_Trans t, Point p) {
		t.set_src_point(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_dest_point_pos(Model_Trans t, Point p) {
		t.set_dest_point(p);

		// 変更通知
		frame.notify_update_data();
	}

	public void set_trans_comment(Model_Trans t, String text) {
		t.set_comment(text);

		// 変更通知
		frame.notify_update_data();
	}
	
	/******* BaseStateListを変更 *******/
	public ArrayList<Model_Base_State> get_base_state_list() {
		return base_state_list;
	}
	
	public boolean add_base_state(String _state_name, Boolean _is_root){
		base_state_list.add(new Model_Base_State(this, _state_name, _is_root));
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
		
		return true;
	}
	
	public boolean set_base_state_name(Model_Base_State _base_state, String _name){
		_base_state.set_state_name(_name);
	
		// 変更通知
		frame.notify_update_data();
		
		return true;
	}
	
	public boolean set_base_state_extends(Model_Base_State _base_state, String _extends){
		_base_state.set_extends(_extends);
	
		// 変更通知
		frame.notify_update_data();
		
		return true;
	}
	
	public boolean set_base_state_field(Model_Base_State _base_state, String _field){
		_base_state.set_field_usr(_field);
	
		// 変更通知
		frame.notify_update_data();
		
		return true;
	}
	
	public void remove_base_state(int [] index){
		for (int i = index.length - 1 ; i >= 0 ; i--){
			if(base_state_list.get(index[i]).is_root_state() == false){
				base_state_list.remove(index[i]);
			}
		}
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}
	
	/******* ScenarioListを変更 *******/
	public ArrayList<Model_Scenario> get_scenario_list() {
		return scenario_list;
	}
	
	public Model_Scenario add_scenario(String _name){
		Model_Scenario s = new Model_Scenario(this, _name);
		
		scenario_list.add(s);
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
		
		return s;
	}
	
	public void remove_scenario(int [] index){
		for (int i = index.length - 1 ; i >= 0 ; i--){
			scenario_list.remove(index[i]);
		}
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}
	
	public void set_scenario_name(Model_Scenario model, String _name){
		model.name = _name;
		
		// 変更通知
		frame.notify_update_data();
	}
	
	/******* EventListを変更 *******/
	public ArrayList<Model_Event> get_event_list() {
		return event_list;
	}
	
	public Model_Event add_event(String _name, String _param){
		Model_Event event = new Model_Event(this, _name, _param);
		
		event_list.add(event);
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
		
		return event;
	}
	
	public void set_event_name(Model_Event _event, String _name){	
		_event.set_name(_name);
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}
	
	public void set_event_param(Model_Event _event, String _param){	
		_event.set_param(_param);
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}
	
	public void edit_event(Model_Event _event, String _name, String _param){	
		_event.set_name(_name);
		_event.set_param(_param);
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}
	
	public void remove_event(int [] index){
		for (int i = index.length - 1 ; i >= 0 ; i--){
			event_list.remove(index[i]);
		}
		
		// 変更通知
		frame.notify_update_data();
		frame.add_undo_data_if_need();
	}

	/******* その他 *******/
	public String create_unique_base_state_name() {
		String name = new String(DEFAULT_STATE_NAME);

		ArrayList<Integer> free_index = new ArrayList<Integer>();
		for (int i = 0; i <= base_state_list.size(); i++) {
			free_index.add(i);
		}

		for (int i = 0; i < base_state_list.size(); i++) {
			if (base_state_list.get(i).get_state_name().startsWith(
					DEFAULT_STATE_NAME)) {
				String last = base_state_list.get(i).get_state_name().substring(
						DEFAULT_STATE_NAME.length());

				try {
					Integer val = Integer.parseInt(last); // input_valが文字列ならここで例外発生
					free_index.remove(val);
				} catch (NumberFormatException e) {
				} finally {
				}
			}
		}

		return name + free_index.get(0);
	}
	
	public String create_unique_event_name() {
		String name = new String(DEFAULT_EVENT_NAME);

		ArrayList<Integer> free_index = new ArrayList<Integer>();
		for (int i = 0; i <= event_list.size(); i++) {
			free_index.add(i);
		}

		for (int i = 0; i < event_list.size(); i++) {
			if (event_list.get(i).get_name().startsWith(
					DEFAULT_EVENT_NAME)) {
				String last = event_list.get(i).get_name().substring(
						DEFAULT_EVENT_NAME.length());

				try {
					Integer val = Integer.parseInt(last); // input_valが文字列ならここで例外発生
					free_index.remove(val);
				} catch (NumberFormatException e) {
				} finally {
				}
			}
		}

		return name + free_index.get(0);
	}
	
	public String create_unique_scenario_name() {
		String name = "scenario";

		ArrayList<Integer> free_index = new ArrayList<Integer>();
		for (int i = 0; i <= scenario_list.size(); i++) {
			free_index.add(i);
		}

		for (int i = 0; i < scenario_list.size(); i++) {
			if (scenario_list.get(i).get_name().startsWith("scenario")) {
				String last = scenario_list.get(i).get_name().substring("scenario".length());
				try {
					Integer val = Integer.parseInt(last); // input_valが文字列ならここで例外発生
					free_index.remove(val);
				} 
				catch (NumberFormatException e) {
				}
				finally {
				}
			}
		}

		return name + free_index.get(0);
	}
	
	public String get_alloy() {
		String alloy = "";

		alloy += g_state.get_alloy();

		for (int i = 0; i < memo_list.size(); i++) {
			alloy += memo_list.get(i).get_alloy();
		}
		
		for (int i = 0; i < base_state_list.size(); i++) {
			alloy += base_state_list.get(i).get_alloy();
		}

		for (int i = 0; i < state_list.size(); i++) {
			alloy += state_list.get(i).get_alloy();
		}

		for (int i = 0; i < trans_list.size(); i++) {
			alloy += trans_list.get(i).get_alloy();
		}
		
		for (int i = 0; i < event_list.size(); i++) {
			alloy += event_list.get(i).get_alloy();
		}
		
		for (int i = 0; i < scenario_list.size(); i++) {
			alloy += scenario_list.get(i).get_alloy();
		}

		return alloy;
	}

	public Model_Global_Def get_global_state() {
		return g_state;
	}

	public String get_xml() {
		String text = "";

		text += "<?xml version=\"1.0\" encoding=\"Shift-JIS\"?>\n";

		text += "<Alloy_State_Model>\n";
		text += g_state.get_xml();

		text += "\t<Base_State_List>\n";
		for (int i = 0; i < base_state_list.size(); i++) {
			text += base_state_list.get(i).get_xml();
		}
		text += "\t</Base_State_List>\n";
		
		text += "\t<State_List>\n";
		for (int i = 0; i < state_list.size(); i++) {
			text += state_list.get(i).get_xml();
		}
		text += "\t</State_List>\n";
		
		text += "\t<Event_List>\n";
		for (int i = 0; i < event_list.size(); i++) {
			text += event_list.get(i).get_xml();
		}
		text += "\t</Event_List>\n";

		text += "\t<Trans_List>\n";
		for (int i = 0; i < trans_list.size(); i++) {
			text += trans_list.get(i).get_xml();
		}
		text += "\t</Trans_List>\n";

		text += "\t<Memo_List>\n";
		for (int i = 0; i < memo_list.size(); i++) {
			text += memo_list.get(i).get_xml();
		}
		text += "\t</Memo_List>\n";
		
		text += "\t<Scenario_List>\n";
		for (int i = 0; i < scenario_list.size(); i++) {
			text += scenario_list.get(i).get_xml();
		}
		text += "\t</Scenario_List>\n";

		text += "</Alloy_State_Model>\n";

		return text;
	}
	
	public Model_Base_State get_root_state() {
		Model_Base_State s = null;

		for (int i = 0; i < base_state_list.size(); i++) {
			if (base_state_list.get(i).is_root_state()) {
				return base_state_list.get(i);
			}
		}

		return null;
	}
	
	/******* Global State Definitionを変更 *******/
	public void set_global_def_usr(String text){
		get_g_state().set_def_usr(text);
		frame.notify_update_data();
	}
	
	public void set_global_trace_usr(String text){
		get_g_state().set_trace_usr(text);
		frame.notify_update_data();
	}
	
	/******* EventListを変更 *******/
	
	/******* その他 *******/
	private Model_Global_Def get_g_state(){
		return (Model_Global_Def)g_state;
	}
	
	public String get_trace_pred_text(){
		String text = "";
		
		for(int i = 0; i < event_list.size(); i ++){
			Model_Event t = event_list.get(i);
			
			if(t.get_param().length() == 0){
				text += "\t\t\t" + t.get_name() + "[s, s'] ||\n";
			}
			else{
				text += "\t\t\t(some " + t.get_param() + " | " + t.get_name() + "[s, s'" + t.get_param_names() + "]) ||\n";				
			}
		}
		if(text.endsWith(" ||\n") == true){
			text = text.substring(0, text.length() - 4);
		}
		
		return text;
	}
	
	protected void parse_xml(Document xml){
		Element root = xml.getDocumentElement();
		
		//Global Definition
		NodeList global_list = root.getElementsByTagName("Global_Definition");
		for(int i = 0; i < global_list.getLength(); i++){
			get_g_state().set_xml((Element)global_list.item(i));
		}
		
		//Base State List
		NodeList b_list = root.getElementsByTagName("Base_State");
		for(int i = 0; i < b_list.getLength(); i++){
			base_state_list.add(new Model_Base_State(this, (Element)b_list.item(i)));
		}
		
		//State List
		NodeList s_list = root.getElementsByTagName("State");
		for(int i = 0; i < s_list.getLength(); i++){
			Element elem = (Element)s_list.item(i);
			String is_initial_state = elem.getAttribute("model_type");
			if(is_initial_state.equalsIgnoreCase("TYPE_CONNECT_POINT")){
				state_list.add(new Model_Connect_Point(this, elem));
			}
			else{
				state_list.add(new Model_State(this, elem));
			}
		}
		
		//Event List
		NodeList e_list = root.getElementsByTagName("Event");
		for(int i = 0; i < e_list.getLength(); i++){
			event_list.add(new Model_Event(this, (Element)e_list.item(i)));
		}
		
		//Trans List
		NodeList t_list = root.getElementsByTagName("Transition");
		for(int i = 0; i < t_list.getLength(); i++){
			trans_list.add(new Model_Trans(this, (Element)t_list.item(i)));
		}
		
		//Memo List
		NodeList m_list = root.getElementsByTagName("Memo");
		for(int i = 0; i < m_list.getLength(); i++){
			memo_list.add(new Model_Memo(this, (Element)m_list.item(i)));
		}
		
		//Memo List
		NodeList sc_list = root.getElementsByTagName("Scenario");
		for(int i = 0; i < sc_list.getLength(); i++){
			scenario_list.add(new Model_Scenario(this, (Element)sc_list.item(i)));
		}
	}
	
	protected void parse_xml_and_get_models(Document xml, ArrayList<Model_State> out_state_list, 
			ArrayList<Model_Trans> out_trans_list, 
			ArrayList<Model_Memo> out_memo_list){
		Element root = xml.getDocumentElement();
		
		//State List
		NodeList s_list = root.getElementsByTagName("State");
		for(int i = 0; i < s_list.getLength(); i++){
			Element elem = (Element)s_list.item(i);
			String is_initial_state = elem.getAttribute("model_type");
			if(is_initial_state.equalsIgnoreCase("TYPE_CONNECT_POINT")){
				out_state_list.add(new Model_Connect_Point(this, elem));
			}
			else{
				out_state_list.add(new Model_State(this, elem));
			}
		}
				
		//Trans List
		NodeList t_list = root.getElementsByTagName("Transition");
		for(int i = 0; i < t_list.getLength(); i++){
			out_trans_list.add(new Model_Trans(this, (Element)t_list.item(i), out_state_list));
		}
		
		//Memo List
		NodeList m_list = root.getElementsByTagName("Memo");
		for(int i = 0; i < m_list.getLength(); i++){
			out_memo_list.add(new Model_Memo(this, (Element)m_list.item(i)));
		}
	}
}
