import java.util.ArrayList;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class Model_Scenario extends Model_Elem {
	public static final String TEXT_NO_CHECK = "未チェック";
	public static final String TEXT_CHAECK_OK = "チェックOK";
	public static final String TEXT_CHECK_NG = "チェックNG";
	
	protected String name = "シナリオ";
	protected Model_State start_st = null;
	protected Model_State end_st = null;
	protected ArrayList<Model_Event> event_list = null;
	protected String check_state = TEXT_NO_CHECK;
	
	
	Model_Scenario(Model_Root _root, String _name){
		super(_root);
		
		name = _name;
		event_list = new ArrayList<Model_Event>();
	}
	
	Model_Scenario(Model_Root _root, Element xml){
		super(_root);
		
		set_xml(xml);
	}
	
	public void set_name(String _name){
		name = _name;
	}
	
	public void set_start_st(Model_State _st){
		start_st = _st;
	}
	
	public void set_end_st(Model_State _st){
		end_st = _st;
	}
	
	public ArrayList<Model_Event> get_event_list(){
		return event_list;
	}
	
	public String get_name(){
		return name;
	}
	
	protected String get_alloy_run_command() {
		return "run " + get_scenario_pred_name() + " for 3 but exactly " + (event_list.size() + 1) + " " + Model_Base_State.ROOT_STATE_NAME + "\n";
	}
	
	protected String get_scenario_pred_name(){
		return "scenario_" + name;
	}

	@Override
	public String get_alloy() {
		String alloy = "";
		
		alloy += "/*\n" + insert_tab("シナリオ:" + get_name()) + "\n*/\n";

		alloy += "pred " + get_scenario_pred_name() + "(disj ";
		for(int i = 0; i < event_list.size() + 1; i ++){
			if(i > 0) alloy += ", ";
			alloy += "s" + i;
		}
		alloy += " : " + Model_Base_State.ROOT_STATE_NAME + "){\n";
		
		if(start_st != null){
			alloy += "\t" + "s0 in " + start_st.get_base_state_name() + "\n";
		}
		if(end_st != null){
			alloy += "\t" + "s" + event_list.size() + " in " + end_st.get_base_state_name() + "\n";
		}
		alloy += "\n";
		
		for(int j = 0; j < event_list.size(); j ++){
			Model_Event e = event_list.get(j);
			String st1_name = "s" + j;
			String st2_name = "s" + (j + 1);
			
			if(e.get_param().length() == 0){
				alloy += "\t" + e.get_name() + "[" + st1_name + ", " + st2_name + "]\n";
			}
			else{
				alloy += "\t(some " + e.get_param() + " | " + e.get_name() + "[" + st1_name + ", " + st2_name + e.get_param_names() + "])\n";				
			}
		}
		
		alloy += "}\n\n";
		
		return alloy;
	}

	@Override
	public String get_xml() {
		String text = "";
		
		text += "\t\t<Scenario>\n";	
		text += "\t\t\t<name><![CDATA[" + name + "]]></name>\n";
		text += "\t\t\t<start_st_index><![CDATA[" + root.get_state_list().indexOf(start_st) + "]]></start_st_index>\n";
		text += "\t\t\t<end_st_index><![CDATA[" + root.get_state_list().indexOf(end_st) + "]]></end_st_index>\n";
		text += "\t\t\t<event_list>\n";	
		for(int i = 0; i < event_list.size(); i ++){
			text += "\t\t\t\t<event_index><![CDATA[" + root.get_event_list().indexOf(event_list.get(i)) + "]]></event_index>\n";
		}
		text += "\t\t\t</event_list>\n";
		text += "\t\t</Scenario>\n";
		
		return text;
	}

	@Override
	public void set_xml(Element xml) {
		if(is_exist_xml_tag(xml, "name")) name = get_xml_string(xml, "name");
		
		if(is_exist_xml_tag(xml, "start_st_index")){
			int index = Integer.valueOf(get_xml_string(xml, "start_st_index"));
			if(index != -1){
	        	start_st = root.get_state_list().get(index);
	        }
	        else{
	        	start_st = null;
	        }
		}
		else{
			start_st = null;			
		}
		
		if(is_exist_xml_tag(xml, "end_st_index")){
			int index = Integer.valueOf(get_xml_string(xml, "end_st_index"));
			if(index != -1){
	        	end_st = root.get_state_list().get(index);
	        }
	        else{
	        	end_st = null;
	        }
		}
		else{
			end_st = null;			
		}
		
		if(event_list == null) event_list = new ArrayList<Model_Event>();
		NodeList p_list = xml.getElementsByTagName("event_index");
		for(int i = 0; i < p_list.getLength(); i++){
			Element elem = (Element)p_list.item(i);
			int index = Integer.valueOf(elem.getFirstChild().getNodeValue());
			event_list.add(root.get_event_list().get(index));
		}
	}
}
