import java.awt.Point;
import java.util.ArrayList;

import org.w3c.dom.Element;


public class Model_State extends Model_Canvas_Elem {
	public static final int TYPE_NORMAL_STATE = 0;
	public static final int TYPE_INIT_STATE = 1;
	public static final int TYPE_CONNECT_POINT = 2;
	public static final int SHAPE_CIRCLE = 0;
	public static final int SHAPE_RECT = 1;
	
	protected Point pos = null;		//中心座標
	protected int r = 10;
	protected int w = 10;
	protected int h = 10;
	protected int shape = SHAPE_CIRCLE;
	
	protected String comment = "";
	Model_Base_State base_state = null;

	
	Model_State(Model_Root _root, Point _pos, int _r){
		super(_root);
		
		pos = _pos;
		r = _r;
		w = 2 * r;
		h = w;
		comment = "";
		shape = SHAPE_CIRCLE;
		
		base_state = root.get_root_state();
	}
	
	Model_State(Model_Root _root){
		super(_root);
		
		pos = new Point(0, 0);
		r = 10;
		w = 10;
		h = 10;
		comment = "";
		
		base_state = root.get_root_state();
	}
	
	Model_State(Model_Root _root, Element xml){
		super(_root);
		
		set_xml(xml);
	}
	
	//図形
	public void set_shape(int s){
		switch(shape){
		case SHAPE_CIRCLE:
			w = 2 * r;
			h = w;
			break;
		case SHAPE_RECT:
//			r = (int)(Math.sqrt(w / 2 * w / 2 + h / 2 * h / 2));
			break;
		}
		shape = s;
	}
	
	public void set_r(int _r){
		r = unscaled(_r);
		w = 2 * r;
		h = w;
	}
	
	public void set_wh(int _w, int _h){
		w = unscaled(_w);
		h = unscaled(_h);
		r = (int)(Math.sqrt(w / 2 * w / 2 + h / 2 * h / 2));
	}
	
	public void move_pos(int dx, int dy){
		pos.translate(unscaled(dx), unscaled(dy));
	}
	
	public int get_shape(){
		return shape;
	}
	
	public Point get_pos(){
		return scaled_p(pos);
	}
	
	public int get_r(){
		return scaled(r);
	}
	
	public int get_w(){
		return scaled(w);
	}
	
	public int get_h(){
		return scaled(h);
	}
	
	//alloy
	public int get_model_type(){
		return TYPE_NORMAL_STATE;
	}
	
	public void set_comment(String text){
		comment = text;
	}
	
	public String get_comment(){
		return comment;
	}
	
	public void set_base_state(Model_Base_State state){
		base_state = state;
	}
	
	public String get_base_state_name(){
		return base_state.get_state_name();
	}
	
	public Model_Base_State get_base_state(){
		return base_state;
	}
	
	public ArrayList<String> get_valiable_list(){
		return base_state.get_valiable_list();
	}

	@Override
	public String get_alloy() {
		String alloy = "";
		
		return alloy;
	}

	@Override
	public String get_xml() {
		String text = "";
		
		text += "\t\t<State model_type=\"TYPE_NORMAL_STATE\">\n";
		text += "\t\t\t<base_state_index><![CDATA[" + root.get_base_state_list().indexOf(base_state) + "]]></base_state_index>\n";
		text += "\t\t\t<pos.x><![CDATA[" + pos.x + "]]></pos.x>\n";
		text += "\t\t\t<pos.y><![CDATA[" + pos.y + "]]></pos.y>\n";
		text += "\t\t\t<r><![CDATA[" + r + "]]></r>\n";
		text += "\t\t\t<w><![CDATA[" + w + "]]></w>\n";
		text += "\t\t\t<h><![CDATA[" + h + "]]></h>\n";
		text += "\t\t\t<shape><![CDATA[" + shape + "]]></shape>\n";
		text += "\t\t\t<comment><![CDATA[" + comment + "]]></comment>\n";
		text += "\t\t</State>\n";
		
		return text;
	}

	@Override
	public void set_xml(Element xml) {
		if(is_exist_xml_tag(xml, "base_state_index")){
			int base_state_index = Integer.valueOf(get_xml_string(xml, "base_state_index"));
			if(base_state_index != -1){
				base_state = root.get_base_state_list().get(base_state_index);
	        }
	        else{
	        	base_state = root.get_root_state();
	        }
		}
		else{
			base_state = root.get_root_state();
		}
		
		if(pos == null){
			pos = new Point(0, 0);
		}
		if(is_exist_xml_tag(xml, "pos.x")) pos.x = Integer.valueOf(get_xml_string(xml, "pos.x"));
		if(is_exist_xml_tag(xml, "pos.y")) pos.y = Integer.valueOf(get_xml_string(xml, "pos.y"));
		if(is_exist_xml_tag(xml, "r")) r = Integer.valueOf(get_xml_string(xml, "r"));
		if(is_exist_xml_tag(xml, "w")) w = Integer.valueOf(get_xml_string(xml, "w"));
		if(is_exist_xml_tag(xml, "h")) h = Integer.valueOf(get_xml_string(xml, "h"));
		if(is_exist_xml_tag(xml, "shape")) shape = Integer.valueOf(get_xml_string(xml, "shape"));
		if(is_exist_xml_tag(xml, "comment")) comment = get_xml_string(xml, "comment");
	}
	

}
