import java.awt.Point;
import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class Model_Trans extends Model_Canvas_Elem {
	protected static final String DEFAULT_PRED_NAME = "event";
	protected static final int DEFAULT_OFFSET_X = 20;
	protected static final int DEFAULT_OFFSET_Y = 10;
	
	protected final String DEFAULT_RUN_NUM = "3";
	
	protected ArrayList<Point> point_list = null;
	protected int label_offset_x = DEFAULT_OFFSET_X;
	protected int label_offset_y = DEFAULT_OFFSET_Y;
	protected double src_point_rad = 0.0f;
	protected double dest_point_rad = 0.0f;

	protected Model_State src_state = null;
	protected Model_State dest_state = null;
	
	protected Model_Event event = null;
	protected String pred = "\t\n}";
	
	protected String comment = "";
	ArrayList<String> change_valiables = null;
	
	
	Model_Trans(Model_Root _root, Point start, Point end, float _scale){
		super(_root);
				
		change_valiables = new ArrayList<String>();
		point_list = new ArrayList<Point>();
		point_list.add(unscaled_p(start));
		point_list.add(unscaled_p(end));
	}
	
	Model_Trans(Model_Root _root, Element xml){
		super(_root);
		
		set_xml(xml);
	}
	
	Model_Trans(Model_Root _root, Element xml, ArrayList<Model_State> state_list){
		super(_root);
		
		this.set_xml(xml, state_list);
	}
	
	//図形
	public void move_trans(int dx, int dy){
		for(int i = 0; i < point_list.size(); i ++){
			point_list.get(i).translate(unscaled(dx), unscaled(dy));
		}
	}
	
	public void move_state(Model_State s, int dx, int dy){
		if((src_state == s && dest_state == s) || (src_state == null && dest_state == s) || (src_state == s && dest_state == null)){
			move_trans(dx, dy);
		}
		else{
			if(src_state == s) get_org_src_point().translate(unscaled(dx), unscaled(dy));
			if(dest_state == s) get_org_dest_point().translate(unscaled(dx), unscaled(dy));
		}
	}
	
	public void move_middle_point(Point p, int dx, int dy){
		assert(point_list.contains(p) && p != get_org_src_point() && p != get_org_dest_point());
		
		p.translate(unscaled(dx), unscaled(dy));
	}
	
	public void move_src_point(int dx, int dy){
		get_org_src_point().translate(unscaled(dx), unscaled(dy));
	}
	
	public void move_dest_point(int dx, int dy){
		get_org_dest_point().translate(unscaled(dx), unscaled(dy));
	}
	
	public void add_middle_point(Point new_p, Point prev_p){
		int index = point_list.indexOf(prev_p);
		point_list.add(index + 1, unscaled_p(new_p));
	}
	
	public void del_middle_point(Point p){
		point_list.remove(p);
	}

	public void connect_src_point(Model_State src, Point p, double rad){
		get_org_src_point().setLocation(unscaled_p(p));
		src_point_rad = rad;
		src_state = src;
	}
	
	public void connect_dest_point(Model_State dest, Point p, double rad){
		get_org_dest_point().setLocation(unscaled_p(p));
		dest_point_rad = rad;
		dest_state = dest;
	}
	
	public void unconnect_src_point(){
		src_state = null;
	}
	
	public void unconnect_dest_point(){
		dest_state = null;
	}
	
	public void set_src_point(Point p){
		get_org_src_point().setLocation(unscaled_p(p));
	}
	
	public void set_dest_point(Point p){
		get_org_dest_point().setLocation(unscaled_p(p));
	}
	
	public void move_label_offset(int dx, int dy){
		label_offset_x += unscaled(dx);
		label_offset_y += unscaled(dy);
	}
	
	public double get_src_point_rad(){
		return src_point_rad;
	}
	
	public double get_dest_point_rad(){
		return dest_point_rad;
	}
	
	public int get_label_offset_x(){
		return scaled(label_offset_x);
	}
	
	public int get_label_offset_y(){
		return scaled(label_offset_y);
	}
	
	public ArrayList<Point> get_point_list(){
		return scaled_p_list(point_list);
	}
	
	public ArrayList<Point> get_org_point_list(){
		return point_list;
	}
	
	public Point get_src_point(){
		return scaled_p(point_list.get(0));
	}
	
	public Point get_dest_point(){
		return scaled_p(point_list.get(point_list.size() - 1));
	}
	
	public Point get_org_src_point(){
		return point_list.get(0);
	}
	
	public Point get_org_dest_point(){
		return point_list.get(point_list.size() - 1);
	}
	
	public boolean is_connect_point_src(){
		if(get_src_state() != null){
			if(get_src_state().get_model_type() == Model_State.TYPE_CONNECT_POINT) return true;
		}
		
		return false;
	}
	
	public boolean is_connect_point_dest(){
		if(get_dest_state() != null){
			if(get_dest_state().get_model_type() == Model_State.TYPE_CONNECT_POINT) return true;
		}
		
		return false;
	}
	
	//alloy
	String get_src_dest(){
		String text = "";
		
		if(get_src_state() != null) {
			text += "\nSrc: " + get_src_state().get_base_state_name();
		}
		if(get_src_state() != null) {
			text += "\nDest: " + get_dest_state().get_base_state_name();
		}
		if(text.isEmpty() == false){
			text += "\n";
		}
		
		return text;
	}
	
	public String get_run_show_text(){
		return "run " + get_pred_full_name() + " for " + DEFAULT_RUN_NUM + get_but_text();
	}
	
	public String get_but_text(){
		if(get_src_state() != null && get_src_state().get_model_type() == Model_State.TYPE_INIT_STATE){
			return " but exactly 1 " + Model_Base_State.ROOT_STATE_NAME;
		}
		else return " but exactly 2 " + Model_Base_State.ROOT_STATE_NAME;
	}
	
	public String get_pred_param(){
		String text = "";
		if(get_src_state() == null || get_src_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			text += " ("  + get_src_param() + ", " + get_dest_param();
		}
		else text += " (" + get_dest_param();
		
		if(event == null){
			text += ")";
		}
		else{
			if(event.get_param().isEmpty()){
				text += ")";
			}
			else{
				text += ", " + event.get_param() + ")";
			}
		}
		
		return text;
	}
	
	public String get_pred_def(){
		String text = "pred " + get_pred_full_name() + get_pred_param() + " {";
		
		return text;
	}
	
	public String get_pred(){
		return pred;
	}
	
	public String get_pred_name(){
		if(event != null) return event.get_name();
		else return "???";
	}
	
	public String get_pred_full_name(){
		String text = new String(get_pred_name());
		
		if(get_src_state() != null && get_src_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			text += "__" + get_src_state().get_base_state_name();
		}
		if(get_dest_state() != null && get_dest_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			text += "__" + get_dest_state().get_base_state_name();
		}
		
		return text;
	}
	
	public String get_event_param(){
		if(event != null) return event.get_param();
		else return "";
	}
	
	public String get_default_pred(){
		String text = "";
		
		if(get_src_state() != null && get_src_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			text += "\t" + "s in " + get_src_state().get_base_state_name() + "\n";
		}
		if(get_dest_state() != null && get_dest_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			text += "\t" + "s' in " + get_dest_state().get_base_state_name() + "\n\n";
		}
		text += get_no_change_pred(); 
		
		
		return text;
	}
	
	public String get_no_change_pred(){
		String text = "";
		
		if(dest_state != null){
			ArrayList<String> vals = dest_state.get_valiable_list();
			for(int j = 0; j < vals.size(); j ++){
				if(is_no_change_valiable(vals.get(j))){
					String val = vals.get(j).replaceAll(":.*", "");
					val = val.replaceAll("\\[.*\\]", "");
					val = val.replaceAll(" ", "");
					text += "\ts'." + val + " = s." + val + "\n";
				}
			}
		}		
		
		return text;
	}
	
	public Model_State get_src_state(){
		return src_state;
	}
	
	public Model_State get_dest_state(){
		return dest_state;
	}
	
	public String get_label(){
		if(event != null) return event.get_full_name();
		else return "???";
	}
	
	public String get_comment(){
		return comment;
	}
	
	public Model_Event get_event(){
		return event;
	}
	
	public void set_comment(String text){
		comment = text;
	}
	
	public void set_pred(Model_Event _event, String _pred){
		event = _event;
		pred = _pred;
	}
	
	public void set_event(Model_Event e){
		event = e;
	}
	
	public void del_state(Model_State s){
		if(src_state == s) src_state = null;
		if(dest_state == s) dest_state = null;
	}
	
	public void set_change_valiable(ArrayList<String> list){		
		change_valiables = list;
	}
	
	public void add_change_valiable(String v){	
		for(int i = 0; i < change_valiables.size(); i ++){
			if(change_valiables.get(i).equals(v)){
				return;
			}
		}
		
		change_valiables.add(v);
	}
	
	public void del_change_valiable(String v){	
		for(int i = 0; i < change_valiables.size(); i ++){
			if(change_valiables.get(i).equals(v)){
				change_valiables.remove(i);
			}
		}
	}
	
	public boolean is_no_change_valiable(String s){
		for(int i = 0; i < change_valiables.size(); i ++){
			if(change_valiables.get(i).equals(s)){
				return false;
			}
		}
		
		return true;
	}

	public ArrayList<String> get_dest_state_valiables(){
		if(dest_state == null) return new ArrayList<String>();
		
		return dest_state.get_valiable_list();
	}
	
	public String get_src_param(){
		if(get_src_state() != null && get_src_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
			return "s : " + get_src_state().get_base_state_name();
		}
		else return "s : " + Model_Base_State.ROOT_STATE_NAME;
	}
	
	public String get_dest_param(){
		if(get_dest_state() != null){
			if(get_dest_state().get_model_type() == Model_State.TYPE_NORMAL_STATE){
				return "s' : " + get_dest_state().get_base_state_name();
			}
			else if(get_dest_state().get_model_type() == Model_State.TYPE_CONNECT_POINT){
				return "s' : " + get_dest_state().get_base_state_name();
			}
			else {
				return "s' : " + Model_Base_State.ROOT_STATE_NAME;
			}
		}
		else return "s' : " + Model_Base_State.ROOT_STATE_NAME;
	}

	@Override
	public String get_alloy() {
		if(is_connect_point_src() == true) return "";
		
		String alloy = "";
		
		if(get_comment().length() > 0){
			alloy += "/*\n" + insert_tab("遷移:\n" + get_label() + get_src_dest()) + "\n\n";
			alloy += get_comment() + "\n*/\n";
		}
		else{
			alloy += "/*\n" + insert_tab("遷移:\n" + get_label() + get_src_dest()) + "*/\n";
		}
		alloy += get_pred_def() + "\n" + get_default_pred();
		
		alloy += get_pred() + "\n";
		alloy += get_run_show_text() + "\n\n";
		
		return alloy;
	}

	@Override
	public String get_xml() {
		return get_xml(root.get_state_list());
	}
	
	public String get_xml(ArrayList<Model_State> state_list) {
		String text = "";
		
		text += "\t\t<Transition>\n";	
		text += "\t\t\t<event_index><![CDATA[" + root.get_event_list().indexOf(event) + "]]></event_index>\n";
		text += "\t\t\t<pred><![CDATA[" + pred + "]]></pred>\n";
		text += "\t\t\t<label_offset_x><![CDATA[" + label_offset_x + "]]></label_offset_x>\n";
		text += "\t\t\t<label_offset_y><![CDATA[" + label_offset_y + "]]></label_offset_y>\n";	
		text += "\t\t\t<src_state_index><![CDATA[" + state_list.indexOf(src_state) + "]]></src_state_index>\n";
		text += "\t\t\t<dest_state_index><![CDATA[" + state_list.indexOf(dest_state) + "]]></dest_state_index>\n";
		text += "\t\t\t<src_point_rad><![CDATA[" + src_point_rad + "]]></src_point_rad>\n";
		text += "\t\t\t<dest_point_rad><![CDATA[" + dest_point_rad + "]]></dest_point_rad>\n";
		text += "\t\t\t<comment><![CDATA[" + comment + "]]></comment>\n";
		
		text += "\t\t\t<point_list>\n";	
		for(int i = 0; i < point_list.size(); i ++){
			text += "\t\t\t\t<point>\n";	
			text += "\t\t\t\t\t<x><![CDATA[" + point_list.get(i).x + "]]></x>\n";
			text += "\t\t\t\t\t<y><![CDATA[" + point_list.get(i).y + "]]></y>\n";
			text += "\t\t\t\t</point>\n";	
		}
		text += "\t\t\t</point_list>\n";
		
		text += "\t\t\t<change_valiable_list>\n";	
		for(int i = 0; i < change_valiables.size(); i ++){
			text += "\t\t\t\t<change_valiable><![CDATA[" + change_valiables.get(i) + "]]></change_valiable>\n";
		}
		text += "\t\t\t</change_valiable_list>\n";
		
		text += "\t\t</Transition>\n";
		
		return text;
	}

	@Override
	public void set_xml(Element xml) {
		set_xml(xml, root.get_state_list());
	}
	
	public void set_xml(Element xml, ArrayList<Model_State> state_list) {
		if(is_exist_xml_tag(xml, "event_index")){
			int event_index = Integer.valueOf(get_xml_string(xml, "event_index"));
			if(event_index != -1){
	        	event = root.get_event_list().get(event_index);
	        }
	        else{
	        	event = null;
	        }
		}
		else{
			event = null;			
		}
		
		if(is_exist_xml_tag(xml, "pred")) pred = get_xml_string(xml, "pred");
		if(is_exist_xml_tag(xml, "label_offset_x")) label_offset_x = Integer.valueOf(get_xml_string(xml, "label_offset_x"));
		if(is_exist_xml_tag(xml, "label_offset_y")) label_offset_y = Integer.valueOf(get_xml_string(xml, "label_offset_y"));
		if(is_exist_xml_tag(xml, "src_point_rad")) src_point_rad = Double.valueOf(get_xml_string(xml, "src_point_rad"));
		if(is_exist_xml_tag(xml, "dest_point_rad")) dest_point_rad = Double.valueOf(get_xml_string(xml, "dest_point_rad"));
		if(is_exist_xml_tag(xml, "comment")) comment = get_xml_string(xml, "comment");
        
		if(is_exist_xml_tag(xml, "src_state_index")){
			int src_state_index = Integer.valueOf(get_xml_string(xml, "src_state_index"));
			if(src_state_index != -1){
	        	src_state = state_list.get(src_state_index);
	        }
	        else{
	        	src_state = null;
	        }
		}
		else{
			src_state = null;			
		}
		if(is_exist_xml_tag(xml, "dest_state_index")) {
	        int dest_state_index = Integer.valueOf(get_xml_string(xml, "dest_state_index"));
	        if(dest_state_index != -1){
	        	dest_state = state_list.get(dest_state_index);
	        }
		 	else{
		    	dest_state = null;
		    }
		}
		else{
	    	dest_state = null;
	    }
        
		if(point_list == null) point_list = new ArrayList<Point>();
        NodeList p_list = xml.getElementsByTagName("point");
		for(int i = 0; i < p_list.getLength(); i++){
			Element elem = (Element)p_list.item(i);
			int x = Integer.valueOf(get_xml_string(elem, "x"));
			int y = Integer.valueOf(get_xml_string(elem, "y"));
			point_list.add(new Point(x, y));
		}
		
		if(change_valiables == null) change_valiables = new ArrayList<String>();
		NodeList v_list = xml.getElementsByTagName("change_valiable");
		for(int i = 0; i < v_list.getLength(); i++){
			Element elem = (Element)v_list.item(i);
			change_valiables.add(elem.getFirstChild().getNodeValue());
		}
	}
}
