import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.ScrollPane;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.io.Serializable;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.css.Rect;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class MyCanvas extends JPanel implements MouseListener, MouseMotionListener, KeyListener, MouseWheelListener, ClipboardOwner{
	static final int MODE_NORMAL = 0;
	static final int MODE_ADD_STATE = 1;
	static final int MODE_ADD_TRANS1 = 2;
	static final int MODE_ADD_TRANS2 = 3;
	static final int MODE_ADD_MEMO = 4;
	static final int MODE_CANVAS_DRAG = 5;
	
	static final String DEFAULT_MEMO = "Memo";
	
	static final int DEFAULT_STATE_R = 30;
	static final int DEFAULT_CONNECT_POINT_R = 10;
	static final int DEFAULT_INITIAL_STATE_R = 15;
	static final int MIN_MARGIN_W = 50;
	static final int MIN_MARGIN_H = 50;
	static final int MAX_MARGIN_W = 100;
	static final int MAX_MARGIN_H = 100;
	static final Color DRAG_RECT = Color.BLUE;
	static final Color DRAG_RECT_LINE = Color.GRAY;
	
	private ArrayList<Canvas_State> focused_state_list;
	private ArrayList<Canvas_Trans> focused_trans_list;
	private ArrayList<Canvas_Memo> focused_memo_list;
	private ArrayList<Canvas_State> state_list;
	private ArrayList<Canvas_Trans> trans_list;
	private ArrayList<Canvas_Memo> memo_list;
	private int mode;
	private Canvas_State add_cs;
	private Canvas_Trans add_ct;
	private Canvas_Memo add_cm;
	private MyFrame frame;
	private Model_Root model_root;
	private boolean is_need_repaint = false;
	Dimension dim;
	Image buf = null;
	Graphics ct = null;
	private Point viewport_drag_start_point = null;
	private Point viewport_drag_end_point = null;
	JScrollPane scroll;
	
	protected int req_cursor = Cursor.DEFAULT_CURSOR;
	
	static final int SCALE_RATE = 10;
	static final int MIN_SCALE = 10;
	static final int MAX_SCALE = 400;
	
	static final float SCROLL_RATE = 0.1f;
	static final float DRAG_SCROLL_RATE = 1.0f;
	
	protected static final int DEFAULT_SCALE = 100;
	int scale = DEFAULT_SCALE;
	
	protected static final int PASTE_OFFSET_X = 100;
	protected static final int PASTE_OFFSET_Y = 100;
		
	MyCanvas(MyFrame _root_frame, Model_Root _data, JScrollPane _scroll){
		is_need_repaint = false;
		
		frame = _root_frame;
		model_root = _data;
		scroll = _scroll;
		
		model_root.set_scale(get_scale());
		
		state_list = new ArrayList<Canvas_State>();
		trans_list = new ArrayList<Canvas_Trans>();
		memo_list = new ArrayList<Canvas_Memo>();
		focused_state_list = new ArrayList<Canvas_State>();
		focused_trans_list = new ArrayList<Canvas_Trans>();
		focused_memo_list = new ArrayList<Canvas_Memo>();
		
		mode = MODE_NORMAL;
		add_cs = null;
		add_ct = null;
		add_cm = null;
		dim = getSize();
		
		viewport_drag_start_point = null;
		viewport_drag_end_point = null;
		
		this.setBackground(Color.WHITE);
		//this.setFocusable(true);
		this.addKeyListener(this);
/*		
		//サンプルの状態を追加
		model_root.add_base_state("State1", false);
		model_root.set_base_state_field(model_root.get_base_state_list().get(0), "\ta:Int\n}");
		model_root.set_base_state_field(model_root.get_base_state_list().get(1), "\tb:Int\n}");
		
		//サンプルのイベントを追加
		model_root.add_event("event0", "x:Int");
		model_root.add_event("event1", "y:Int");
			
		//サンプルの状態を追加
		Model_State s1 = new Model_State(model_root, new Point(50, 50), DEFAULT_STATE_R);
		s1.set_scale(get_scale());
		model_root.add_state(s1);
		state_list.add(new Canvas_State(this, model_root, s1, Canvas_State.MODE_NORMAL));
		
		Model_State s2 = new Model_State(model_root, new Point(100, 100), DEFAULT_STATE_R);
		s2.set_scale(get_scale());
		model_root.add_state(s2);
		state_list.add(new Canvas_State(this, model_root, s2, Canvas_State.MODE_NORMAL));
		((Model_Root)model_root).set_state_base_state(s2, model_root.get_base_state_list().get(1));
		
		//サンプルの遷移を追加
		Model_Trans t1 = new Model_Trans(model_root, new Point(80, 80), new Point(200, 200), get_scale());
		t1.set_scale(get_scale());
		model_root.add_trans(t1);
		trans_list.add(new Canvas_Trans(this, model_root, t1, Canvas_Trans.MODE_NORMAL));
		model_root.set_trans_event(t1, model_root.get_event_list().get(0));
		
		//サンプルの遷移を追加
		Model_Trans t2 = new Model_Trans(model_root, new Point(100, 150), new Point(200, 300), get_scale());
		t2.set_scale(get_scale());
		model_root.add_trans(t2);
		trans_list.add(new Canvas_Trans(this, model_root, t2, Canvas_Trans.MODE_NORMAL));
		model_root.set_trans_event(t2, model_root.get_event_list().get(0));
		
		//サンプルのメモを追加
		Model_Memo m1 = new Model_Memo(model_root, new Point(300, 80), "めも\nmemo");
		m1.set_scale(get_scale());
		model_root.add_memo(m1);
		memo_list.add(new Canvas_Memo(this, model_root, m1, Canvas_Memo.MODE_NORMAL));
		
		//サンプルのシナリオを追加
		model_root.add_scenario("scenario1");
		model_root.get_scenario_list().get(0).event_list.add(model_root.get_event_list().get(0));
		model_root.get_scenario_list().get(0).event_list.add(model_root.get_event_list().get(1));
*/		
		
		fit_size();
	}
	
	public String get_font_name(){
		return frame.get_font_name();
	}
	
	public int get_org_font_size(){
		return frame.get_org_font_size();
	}
	
	public void set_normal_mode(){
		mode = MODE_NORMAL;
		
		add_ct = null;
		add_cs = null;
		
		viewport_drag_start_point = null;
		viewport_drag_end_point = null;
		
		repaint();
	}
	
	public int get_org_scale(){
		return scale;
	}
	
	public float get_scale(){
		return scale / 100.0f;
	}
	
	public void zoom_in(){
		set_scale(scale + SCALE_RATE);
	}
	
	public void zoom_out(){
		set_scale(scale - SCALE_RATE);	
	}
	
	public void set_scale(int s){
		scale = s;
		if(s > MAX_SCALE) scale = MAX_SCALE;
		if(s < MIN_SCALE) scale = MIN_SCALE;
		
		model_root.set_scale(get_scale());
		fit_size();
		frame.update_status_bar();
		repaint();
	}
	
	public Point org_scale(Point _p){		
		return new Point((int)(_p.x / get_scale()), (int)(_p.y / get_scale()));
	}
	
	public void change_state_shape(Model_State model, int shape){
		Canvas_State s = get_state(model);
		int s_index = state_list.indexOf(s);
		
		switch(shape){
		case Model_State.SHAPE_CIRCLE:
			Canvas_State new_sc = new Canvas_State(this, model_root, s.get_model(), s.get_mode());
			state_list.remove(s);
			
			state_list.add(s_index, new_sc);
			if(focused_state_list.contains(s)){
				focused_state_list.remove(s);
				focused_state_list.add(new_sc);
			}
			
			break;
		case Model_State.SHAPE_RECT:
			Canvas_State_Rect new_sr = new Canvas_State_Rect(this, model_root, s.get_model(), s.get_mode());
			state_list.remove(s);
			state_list.add(s_index, new_sr);
			
			if(focused_state_list.contains(s)){
				focused_state_list.remove(s);
				focused_state_list.add(new_sr);
			}
			break;
		}
	}
	
	public Point get_max_pos(){
		Point max = new Point(0, 0);
		
		for (int i = 0; i < state_list.size(); i ++){
        	Point p = state_list.get(i).max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        if(add_cs != null){
        	Point p = add_cs.max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        for (int i = 0; i < trans_list.size(); i ++){
        	Point p = trans_list.get(i).max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        if(add_ct != null){
        	Point p = add_ct.max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        for (int i = 0; i < memo_list.size(); i ++){
        	Point p = memo_list.get(i).max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        if(add_cm != null){
        	Point p = add_cm.max_point();
        	if(p.x > max.x) max.x = p.x;
        	if(p.y > max.y) max.y = p.y;
        }
        
        return max;
	}
	
	public void fit_size(){
		this.setPreferredSize(get_fit_size());
    	scroll.invalidate();
	}
	
	public Dimension get_fit_size(){
		Point max = get_max_pos();
       
        int new_w, new_h;
        Dimension cur_size = this.getPreferredSize();
        Dimension vp_size = scroll.getSize();
        
        new_w = max.x + MIN_MARGIN_W;
        if(new_w < vp_size.width){
        	new_w = vp_size.width;
        }
        
        new_h = max.y + MIN_MARGIN_H;
        if(new_h < vp_size.height){
        	new_h = vp_size.height;
        }

        if(cur_size.width != new_w || cur_size.height != new_h){
        	this.setSize(new_w, new_h);
        	scroll.doLayout();
        }

		return new Dimension(new_w, new_h);
	}
	
	public Dimension getPreferredSize(){
		return new Dimension(this.getWidth(), this.getHeight());
	}
	
	public void copy(){
		if(mode != MODE_NORMAL) return;
		
		Clipboard clp = getToolkit().getSystemClipboard();
		
		if(focused_state_list.size() > 0 || focused_trans_list.size() > 0 || focused_memo_list.size() > 0){
			ClipBoard_Data clip_data = new ClipBoard_Data(focused_state_list, focused_trans_list, focused_memo_list, model_root.get_event_list());
			clp.setContents(clip_data , this);
		}
	}
	
	public void cut(){
		if(mode != MODE_NORMAL) return;
		
		copy();
		
		for (int i = 0; i < focused_state_list.size(); i ++){
			Canvas_State s = focused_state_list.get(i);
			frame.del_dialog(s.get_model());
			notify_del_state(s);
			state_list.remove(s);
			model_root.del_state(s.get_model());
        }
		focused_state_list.clear();
		
		for (int i = 0; i < focused_trans_list.size(); i ++){
			Canvas_Trans t = focused_trans_list.get(i);
			frame.del_dialog(t.get_model());
			model_root.del_trans(t.get_model());
			trans_list.remove(t);
        }
		focused_trans_list.clear();
		
		for (int i = 0; i < focused_memo_list.size(); i ++){
			Canvas_Memo m = focused_memo_list.get(i);
			frame.del_dialog(m.get_model());
			model_root.del_memo(m.get_model());
			memo_list.remove(m);
        }
		focused_memo_list.clear();
		
		fit_size();
		frame.add_undo_data_if_need();
		notify_update_focus();
		repaint();
	}
	
	public void paste(){
		if(mode != MODE_NORMAL) return;
		
		Clipboard clp = getToolkit().getSystemClipboard();
		if(clp.isDataFlavorAvailable(ClipBoard_Data.ASMFlavor) == false) return;
		
		Transferable t_data = clp.getContents(this);
		try { 
			String clip_data = (String)t_data.getTransferData(ClipBoard_Data.ASMFlavor);
			if(clip_data == null) return;
			
			DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbfactory.newDocumentBuilder();
			Document doc = builder.parse(new InputSource(new StringReader(clip_data)));
			
			ArrayList<Model_State> out_state_list = new ArrayList<Model_State>();
			ArrayList<Model_Trans> out_trans_list = new ArrayList<Model_Trans>();
			ArrayList<Model_Memo> out_memo_list = new ArrayList<Model_Memo>();
			model_root.parse_xml_and_get_models(doc, out_state_list, out_trans_list, out_memo_list);
			
			
			clear_focus();
			
			Point p = scroll.getViewport().getLocation();
			Dimension d = scroll.getViewport().getSize();
			
			//Point mouse_p = frame.getMousePosition();
			
			int offset_x = PASTE_OFFSET_X;
			int offset_y = PASTE_OFFSET_Y;
			
			for(int i = 0; i < out_state_list.size(); i ++){
				Model_State ms = out_state_list.get(i);
				ms.move_pos(offset_x, offset_y);
				model_root.add_state(ms);
				Canvas_State cs;
				if(ms.get_shape() == Model_State.SHAPE_CIRCLE){
					cs = new Canvas_State(this, model_root, ms, Canvas_State.MODE_FOCUSED);
				}
				else{
					cs = new Canvas_State_Rect(this, model_root, ms, Canvas_State.MODE_FOCUSED);
				}
				state_list.add(cs);
				focused_state_list.add(cs);
			}
			
			for(int i = 0; i < out_trans_list.size(); i ++){
				Model_Trans mt = out_trans_list.get(i);
				mt.move_trans(offset_x, offset_y);
				model_root.add_trans(mt);
				Canvas_Trans ct = new Canvas_Trans(this, model_root, mt, Canvas_Trans.MODE_FOCUSED_LINE);
				trans_list.add(ct);
				focused_trans_list.add(ct);
			}
			
			for(int i = 0; i < out_memo_list.size(); i ++){
				Model_Memo mm = out_memo_list.get(i);
				mm.move_pos(offset_x, offset_y);
				model_root.add_memo(mm);
				Canvas_Memo cm = new Canvas_Memo(this, model_root, mm, Canvas_Memo.MODE_FOCUSED);
				memo_list.add(cm);
				focused_memo_list.add(cm);
			}
		}
		catch (UnsupportedFlavorException e){
			return;
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
			return;
		} catch (SAXException e) {
			e.printStackTrace();
			return;
		}
		catch(Exception err) {}
		
		fit_size();
		frame.add_undo_data_if_need();
		notify_update_focus();
		repaint();
	}
	
	public void focused_move_front(){
		if(mode != MODE_NORMAL) return;
		
		for (int i = 0; i < focused_state_list.size(); i ++){
			Canvas_State cs = focused_state_list.get(i);
			state_list.remove(cs);
			state_list.add(cs);
			model_root.move_state_last(cs.get_model());
		}
		
		for (int i = 0; i < focused_trans_list.size(); i ++){
			Canvas_Trans ct = focused_trans_list.get(i);
			trans_list.remove(ct);
			trans_list.add(ct);
			model_root.move_trans_last(ct.get_model());
		}
		
		for (int i = 0; i < focused_memo_list.size(); i ++){
			Canvas_Memo cm = focused_memo_list.get(i);
			memo_list.remove(cm);
			memo_list.add(cm);
			model_root.move_memo_last(cm.get_model());
		}
		
		frame.add_undo_data_if_need();
		repaint();
	}
	
	public void focused_move_back(){
		if(mode != MODE_NORMAL) return;
		
		for (int i = focused_state_list.size() - 1; i >= 0; i --){
			Canvas_State cs = focused_state_list.get(i);
			state_list.remove(cs);
			state_list.add(0, cs);
			model_root.move_state_first(cs.get_model());
		}
		
		for (int i = focused_trans_list.size() - 1; i >= 0; i --){
			Canvas_Trans ct = focused_trans_list.get(i);
			trans_list.remove(ct);
			trans_list.add(0, ct);
			model_root.move_trans_first(ct.get_model());
		}
		
		for (int i = focused_memo_list.size() - 1; i >= 0; i --){
			Canvas_Memo cm = focused_memo_list.get(i);
			memo_list.remove(cm);
			memo_list.add(0, cm);
			model_root.move_memo_first(cm.get_model());
		}
		
		frame.add_undo_data_if_need();
		repaint();
	}
	
	public void set_new_data(Model_Root _data){
		model_root = _data;
		scale = DEFAULT_SCALE;
		
		model_root.set_scale(get_scale());
		
		for (int i = 0; i < model_root.get_state_list().size(); i ++){
			if(model_root.get_state_list().get(i).get_model_type() == Model_State.TYPE_CONNECT_POINT){
				state_list.add(new Canvas_ConnectPoint(this, model_root, model_root.get_state_list().get(i), Canvas_State.MODE_NORMAL));
			}
			else{
				Canvas_State cs;
				if(model_root.get_state_list().get(i).get_shape() == Model_State.SHAPE_CIRCLE){
					cs = new Canvas_State(this, model_root, model_root.get_state_list().get(i), Canvas_State.MODE_NORMAL);
				}
				else{
					cs = new Canvas_State_Rect(this, model_root, model_root.get_state_list().get(i), Canvas_State.MODE_NORMAL);
				}
				state_list.add(cs);
			}
		}
		for (int i = 0; i < model_root.get_trans_list().size(); i ++){
			trans_list.add(new Canvas_Trans(this, model_root, model_root.get_trans_list().get(i), Canvas_Trans.MODE_NORMAL));
		}
		for (int i = 0; i < model_root.get_memo_list().size(); i ++){
			memo_list.add(new Canvas_Memo(this, model_root, model_root.get_memo_list().get(i), Canvas_Memo.MODE_NORMAL));
		}
		
		clear_focus();
		mode = MODE_NORMAL;
		add_cs = null;
		add_ct = null;
		
		fit_size();
	}
	
	public void clear_data(){
        clear_focus();
        
		for (int i = 0; i < state_list.size(); i ++){
        	state_list.get(i).dispose();	
        }
		state_list.clear();
		
        if(add_cs != null){
        	add_cs.dispose();	
        	add_cs = null;
        }
        
        for (int i = 0; i < trans_list.size(); i ++){
        	trans_list.get(i).dispose();	
        }
        trans_list.clear();
        
        if(add_ct != null){
        	add_ct.dispose();	
        	add_ct = null;
        }
        
        for (int i = 0; i < memo_list.size(); i ++){
        	memo_list.get(i).dispose();	
        }
        memo_list.clear();
        
        if(add_cm != null){
        	add_cm.dispose();	
        	add_cm = null;
        }
       
		mode = MODE_NORMAL;
		scale = DEFAULT_SCALE;
        
		model_root = null;  
	}
	
	public void need_repaint(){
		is_need_repaint = true;
	}
	
	public void my_repaint(){
		if(is_need_repaint){
			repaint();
		}
	}
	
	public void repaint(){
		//fit_size();
		super.repaint();
		is_need_repaint = false;
	}
	
	public void setSize(Dimension d){
		super.setSize(d);
	}
	
    public void paintComponent(Graphics g) {
    	g.setColor(Color.white);
        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        
        for (int i = 0; i < state_list.size(); i ++){
        	state_list.get(i).paint(g);	
        }
        if(add_cs != null){
        	add_cs.paint(g);
        }
        for (int i = 0; i < trans_list.size(); i ++){
        	trans_list.get(i).paint_line(g);	
        }
        if(add_ct != null){
        	add_ct.paint_line(g);
        }
        
        for (int i = 0; i < trans_list.size(); i ++){
        	trans_list.get(i).paint_label(g);	
        }
        if(add_ct != null){
        	add_ct.paint_label(g);
        }
        
        for (int i = 0; i < memo_list.size(); i ++){
        	memo_list.get(i).paint(g);	
        }
        if(add_cm != null){
        	add_cm.paint(g);
        }
        
        Graphics2D g2 = (Graphics2D)g;
        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
        
        //ドラッグ領域
        if(viewport_drag_start_point != null && viewport_drag_end_point != null && mode != MODE_CANVAS_DRAG){
        	Rectangle r = get_rect(viewport_to_canvas(viewport_drag_start_point), viewport_to_canvas(viewport_drag_end_point));
        	BasicStroke stroke = new BasicStroke(
        			1.0f,		// 太さ
                    BasicStroke.CAP_BUTT,		// 端は残す
                    BasicStroke.JOIN_MITER,		// 継ぎ目は斜め
                    1.0f,						// 継ぎ目の限界
                    null,
                    0.0f);						// ダッシュの開始フェーズ
    		g2.setStroke(stroke); //線の種類を設定
	        g2.setColor(DRAG_RECT);
	        g2.fillRect(r.x, r.y, r.width, r.height);
	        g2.setColor(DRAG_RECT_LINE);
	        g2.drawRect(r.x, r.y, r.width, r.height);
        }
    }
    
    public MyFrame get_root_frame(){
    	return frame;
    }
    
    public void notify_update_focus(){
    	frame.notify_update_focus();
    }
    
    public void set_focused_index(ArrayList<Integer> focused_state_index_list, ArrayList<Integer> focused_trans_index_list, ArrayList<Integer> focused_memo_index_list){
    	for (int i = 0; i < focused_state_index_list.size(); i ++){
    		Canvas_State s = state_list.get(focused_state_index_list.get(i));
    		
    		focused_state_list.add(s);
    		s.setMode(Canvas_State.MODE_FOCUSED);
		}
    	
    	for (int i = 0; i < focused_trans_index_list.size(); i ++){
    		Canvas_Trans t = trans_list.get(focused_trans_index_list.get(i));
    		
    		focused_trans_list.add(t);
    		t.setMode(Canvas_Trans.MODE_FOCUSED_LINE);
		}
    	
    	for (int i = 0; i < focused_memo_index_list.size(); i ++){
    		Canvas_Memo m = memo_list.get(focused_memo_index_list.get(i));
    		
    		focused_memo_list.add(m);
    		m.setMode(Canvas_Memo.MODE_FOCUSED);
		}
    }
    
    public ArrayList<Integer> get_focused_state_index_list(){
    	ArrayList<Integer> list = new ArrayList<Integer>();
    	
    	for (int i = 0; i < focused_state_list.size(); i ++){
    		list.add(state_list.indexOf(focused_state_list.get(i)));
		}
    	
		return list;
	}
	
    public ArrayList<Integer> get_focused_trans_index_list(){
    	ArrayList<Integer> list = new ArrayList<Integer>();
    	
    	for (int i = 0; i < focused_trans_list.size(); i ++){
    		list.add(trans_list.indexOf(focused_trans_list.get(i)));
		}
    	
		return list;
	}
    
    public ArrayList<Integer> get_focused_memo_index_list(){
    	ArrayList<Integer> list = new ArrayList<Integer>();
    	
    	for (int i = 0; i < focused_memo_list.size(); i ++){
    		list.add(memo_list.indexOf(focused_memo_list.get(i)));
		}
    	
		return list;
	}
        
    public void setAddStateMode(){
    	if(mode == MODE_NORMAL){
    		mode = MODE_ADD_STATE;
    		Model_State s = new Model_State(model_root, new Point(0, 0), DEFAULT_STATE_R);
    		s.set_scale(get_scale());
    		add_cs = new Canvas_State(this, model_root, s, Canvas_State.MODE_ADDING);
    	}
    }
    
    public void setAddRootStateMode(){
    	if(mode == MODE_NORMAL){
    		mode = MODE_ADD_STATE;
    		Model_State s = new Model_State(model_root, new Point(0, 0), DEFAULT_STATE_R);
    		s.set_scale(get_scale());
    		add_cs = new Canvas_State(this, model_root, s, Canvas_State.MODE_ADDING);
    	}
    }
    
    public void setAddTransMode(){
    	if(mode == MODE_NORMAL){
    		mode = MODE_ADD_TRANS1;
    	}
    }
    
    public void setAddConnectPointMode(){
    	if(mode == MODE_NORMAL){
    		mode = MODE_ADD_STATE;
    		Model_State s = new Model_Connect_Point(model_root, new Point(0, 0), DEFAULT_CONNECT_POINT_R);
    		s.set_scale(get_scale());
    		add_cs = new Canvas_ConnectPoint(this, model_root, s, Canvas_State.MODE_ADDING);
    	}
    }
    
    public void setAddMemoMode(){
    	if(mode == MODE_NORMAL){
    		mode = MODE_ADD_MEMO;
    		Model_Memo m = new Model_Memo(model_root, new Point(0, 0), DEFAULT_MEMO);
    		m.set_scale(get_scale());
    		add_cm = new Canvas_Memo(this, model_root, m, Canvas_Memo.MODE_ADDING);
    	}
    }
    
    public Canvas_State get_hit_state(Point p){
/*   	for (int i = focused_state_list.size() - 1; i >= 0 ; i --){
    		Canvas_State cs = focused_state_list.get(i);	
    		if(cs.isHit(p)) return cs;
        }
*/    	for (int i = state_list.size() - 1; i >= 0 ; i --){
    		Canvas_State cs = state_list.get(i);	
    		if(cs.isHit(p)) return cs;
        }
    	return null;
    }
    
    private Canvas_Trans get_hit_trans(Point p){
/*    	for (int i = focused_trans_list.size() - 1; i >= 0 ; i --){
      		Canvas_Trans ct = focused_trans_list.get(i);	
    		if(ct.isHit(p)) return ct;
        }
*/      	for (int i = trans_list.size() - 1; i >= 0 ; i --){
      		Canvas_Trans ct = trans_list.get(i);	
    		if(ct.isHit(p)) return ct;
        }
    	return null;
    }
    
    private Canvas_Memo get_hit_memo(Point p){
 /*   	for (int i = focused_memo_list.size() - 1; i >= 0 ; i --){
      		Canvas_Memo cm = focused_memo_list.get(i);	
    		if(cm.isHit(p)) return cm;
        }
 */    	for (int i = memo_list.size() - 1; i >= 0 ; i --){
      		Canvas_Memo cm = memo_list.get(i);	
    		if(cm.isHit(p)) return cm;
        }
    	return null;
    }
    
    public Canvas_State get_state(Model_State s){
    	 for (int i = 0; i < state_list.size(); i ++){
    		 if(s == state_list.get(i).get_model()) return state_list.get(i);
         }
    	 return null;
    }
    
    public ArrayList<Canvas_State> get_state_list(){
    	return state_list;
    }
    
    private void clear_focus(){
    	for (int i = 0; i < focused_state_list.size(); i ++){
			focused_state_list.get(i).setMode(Canvas_State.MODE_NORMAL);
		}
		for (int i = 0; i < focused_trans_list.size(); i ++){
			focused_trans_list.get(i).setMode(Canvas_Trans.MODE_NORMAL);
		}
		for (int i = 0; i < focused_memo_list.size(); i ++){
			focused_memo_list.get(i).setMode(Canvas_Memo.MODE_NORMAL);
		}
		
		focused_state_list.clear();
		focused_trans_list.clear();
		focused_memo_list.clear();
    }
    
    private void unfocus(Canvas_Trans t){
    	t.setMode(Canvas_Trans.MODE_NORMAL);
    	focused_trans_list.remove(t);
    }
    
    private void unfocus(Canvas_State s){
    	s.setMode(Canvas_Trans.MODE_NORMAL);
    	focused_state_list.remove(s);
    }
    
    private void unfocus(Canvas_Memo m){
    	m.setMode(Canvas_Memo.MODE_NORMAL);
    	focused_memo_list.remove(m);
    }
    
    private void notify_del_state(Canvas_State cs){
    	for (int i = 0; i < trans_list.size(); i ++){
      		trans_list.get(i).del_state(cs);
        }
    }
    
    private Rectangle get_rect(Point p1, Point p2){
    	Rectangle r = new Rectangle();
    	
    	r.x = Math.min(p1.x, p2.x);
        r.y = Math.min(p1.y, p2.y);
        r.width = Math.abs(p1.x - p2.x);
        r.height = Math.abs(p1.y - p2.y);
              
    	return r;
    }
    
    public ArrayList<Canvas_Trans> get_focused_trans_list(){
    	return focused_trans_list;
    }
    
    public boolean is_multi_focused(){
    	if(focused_state_list.size() + focused_trans_list.size() + focused_memo_list.size() > 1) return true;
    	else return false;
    }
    
    protected void update_cursor(){
    	if(req_cursor != this.getCursor().getType()) this.setCursor(Cursor.getPredefinedCursor(req_cursor));
    }
    
    public void request_set_cursor(int cursor){
    	req_cursor = cursor;
    }
        
    protected void request_reset_cursor(){
    	request_set_cursor(Cursor.DEFAULT_CURSOR);
	}
    
    protected Point viewport_to_canvas(Point p){
    	Point vp = scroll.getViewport().getViewPosition();
    	
    	return new Point(p.x + vp.x, p.y + vp.y);
    }
    
    protected MouseEvent canvas_mouse_event(MouseEvent e){
    	Point vp = scroll.getViewport().getViewPosition();
    	//return e;
    	return new MouseEvent((Component) e.getSource(), e.getID(), e.getWhen(), e.getModifiers() | e.getModifiersEx(), e.getX() + vp.x, e.getY() + vp.y, e.getClickCount(), e.isPopupTrigger(), e.getButton());
    }
      
    // Action //
    public void mouseReleased(MouseEvent e){
    	request_reset_cursor();
    	MouseEvent new_e = canvas_mouse_event(e);
    	
    	switch (mode){
    	case MODE_NORMAL :
    		if(viewport_drag_start_point != null && viewport_drag_end_point != null){
	    		Rectangle drag_rect = get_rect(viewport_to_canvas(viewport_drag_start_point), viewport_to_canvas(viewport_drag_end_point));
	    		
	    		for (int i = 0; i < state_list.size(); i ++){
	    			if(state_list.get(i).is_in(drag_rect)) {
	    				focused_state_list.add(state_list.get(i));
	    				state_list.get(i).setMode(Canvas_State.MODE_FOCUSED);
	    			}
	            }
	    		for (int i = 0; i < trans_list.size(); i ++){
	    			if(trans_list.get(i).is_in(drag_rect)){
	    				focused_trans_list.add(trans_list.get(i));
	    				trans_list.get(i).setMode(Canvas_Trans.MODE_FOCUSED_LINE);
	    			}
	            }
	    		for (int i = 0; i < memo_list.size(); i ++){
	    			if(memo_list.get(i).is_in(drag_rect)){
	    				focused_memo_list.add(memo_list.get(i));
	    				memo_list.get(i).setMode(Canvas_Memo.MODE_FOCUSED);
	    			}
	            }
	    		
	    		//notify_update_focus();
	    		need_repaint();
	    	}
	    	else{
	    		for (int i = 0; i < focused_state_list.size(); i ++){
	    			focused_state_list.get(i).mouseReleased(new_e, is_multi_focused());
	            }
	    		for (int i = 0; i < focused_trans_list.size(); i ++){
	    			focused_trans_list.get(i).mouseReleased(new_e, is_multi_focused());
	            }
	    		for (int i = 0; i < focused_memo_list.size(); i ++){
	    			focused_memo_list.get(i).mouseReleased(new_e, is_multi_focused());
	            }
	    		
	    		if(add_ct != null) add_ct.mouseReleased(new_e, is_multi_focused());
	        	if(add_cs != null) add_cs.mouseReleased(new_e, is_multi_focused());
	        	if(add_cm != null) add_cm.mouseReleased(new_e, is_multi_focused());
	    	}  
    		break;
    	case MODE_ADD_STATE :
    		break;
    	case MODE_ADD_TRANS1 :
    		break;
    	case MODE_ADD_TRANS2 :
    		model_root.add_trans(add_ct.get_model());
    		trans_list.add(add_ct);
    		clear_focus();
    		focused_trans_list.add(add_ct);
    		
    		add_ct.mouseReleased(new_e, is_multi_focused());
    		add_ct.setMode(Canvas_Trans.MODE_FOCUSED_LINE);
         	add_ct = null;
    		mode = MODE_NORMAL;
    		break;
    	case MODE_ADD_MEMO :
    		break;
    	case MODE_CANVAS_DRAG:
    		request_set_cursor(Cursor.HAND_CURSOR);
    		break;
		}
    	
    	viewport_drag_start_point = null;
    	viewport_drag_end_point = null;
    	
		update_cursor();
    	fit_size();
    	frame.add_undo_data_if_need();
    	notify_update_focus();
    	my_repaint();
    }

    public void mousePressed(MouseEvent e){
    	request_reset_cursor();
    	MouseEvent new_e = canvas_mouse_event(e);
    	
    	//マウス左
    	if(new_e.getButton() == new_e.BUTTON1){
	    	switch (mode){
	    	case MODE_NORMAL :
	    		Canvas_Trans focused_trans = get_hit_trans(new_e.getPoint());
	    		Canvas_State focused_state = get_hit_state(new_e.getPoint());
	    		Canvas_Memo focused_memo = get_hit_memo(new_e.getPoint());
	    		
	    		//何も無い場所
	    		if(focused_trans == null && focused_state == null && focused_memo == null){
	        		if(new_e.isControlDown() == false){
	        			clear_focus();
	        			//notify_update_focus();
	        		}	
	        		viewport_drag_start_point = new Point(e.getPoint());
	        		viewport_drag_end_point = new Point(e.getPoint());
	    		}
	    		//Memoの上
	    		else if(focused_memo != null){
	    			//フォーカスされているMemoの上
	    			if(focused_memo_list.contains(focused_memo) == true){
	    				if(new_e.isControlDown() == true){
	            			unfocus(focused_memo);
	            		}	
	    				for (int i = 0; i < focused_memo_list.size(); i ++){
	    	    			focused_memo_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    				for (int i = 0; i < focused_trans_list.size(); i ++){
	    	    			focused_trans_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		for (int i = 0; i < focused_state_list.size(); i ++){
	    	    			focused_state_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		
	    	    		//notify_update_focus();
	    			}
	    			//フォーカスされてないMemoの上
	    			else{
	            		if(new_e.isControlDown() == false){
	            			clear_focus();
	            		}
	            		focused_memo_list.add(focused_memo);
	        			focused_memo.mousePressed(new_e, is_multi_focused());
	        			
	        			//notify_update_focus();
	    			}
	    		}
	    		//遷移の上
	    		else if(focused_trans != null){
	    			//フォーカスされている遷移の上
	    			if(focused_trans_list.contains(focused_trans) == true){
	    				if(new_e.isControlDown() == true){
	            			unfocus(focused_trans);
	            		}	
	    				for (int i = 0; i < focused_memo_list.size(); i ++){
	    	    			focused_memo_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		for (int i = 0; i < focused_trans_list.size(); i ++){
	    	    			focused_trans_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		for (int i = 0; i < focused_state_list.size(); i ++){
	    	    			focused_state_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		
	    	    		//notify_update_focus();
	    			}
	    			//フォーカスされてない遷移の上
	    			else{
	            		if(new_e.isControlDown() == false){
	            			clear_focus();
	            		}
	        			focused_trans_list.add(focused_trans);
	        			focused_trans.mousePressed(new_e, is_multi_focused());
	        			
	        			//notify_update_focus();
	    			}
	    		}
	    		//状態の上
	    		else{
	    			//フォーカスされている状態の上
	    			if(focused_state_list.contains(focused_state) == true){
	    				if(new_e.isControlDown() == true){
	            			unfocus(focused_state);
	            		}	
	    				for (int i = 0; i < focused_memo_list.size(); i ++){
	    	    			focused_memo_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    				for (int i = 0; i < focused_trans_list.size(); i ++){
	    	    			focused_trans_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		for (int i = 0; i < focused_state_list.size(); i ++){
	    	    			focused_state_list.get(i).mousePressed(new_e, is_multi_focused());
	    	            }
	    	    		
	    	    		//notify_update_focus();
	    			}
	    			//フォーカスされてない状態の上
	    			else{
	            		if(new_e.isControlDown() == false){
	            			clear_focus();
	            		}
	    				focused_state_list.add(focused_state);
	    				focused_state.mousePressed(new_e, is_multi_focused());
	    				
	    				//notify_update_focus();
	    			}
	    		}
	        	break;
	    	case MODE_ADD_STATE :
	    		model_root.add_state(add_cs.get_model());
	    		state_list.add(add_cs);
	    		clear_focus();
	    		focused_state_list.add(add_cs);
	    		add_cs.setMode(Canvas_State.MODE_FOCUSED);
	    		add_cs.mousePressed(new_e, is_multi_focused());
	         	add_cs = null;
	    		mode = MODE_NORMAL;
	    		break;
	    	case MODE_ADD_TRANS1 :
	    		Model_Trans t = new Model_Trans(model_root, new_e.getPoint(), new_e.getPoint(), get_scale());
	    		add_ct = new Canvas_Trans(this, model_root, t, Canvas_Trans.MODE_ADDING);
	    		mode = MODE_ADD_TRANS2;
	    		break;
	    	case MODE_ADD_TRANS2 :
	    		break;
	    	case MODE_ADD_MEMO :
	    		model_root.add_memo(add_cm.get_model());
	    		memo_list.add(add_cm);
	    		clear_focus();
	    		focused_memo_list.add(add_cm);
	    		add_cm.setMode(Canvas_Memo.MODE_FOCUSED);
	    		add_cm.mousePressed(new_e, is_multi_focused());
	         	add_cm = null;
	    		mode = MODE_NORMAL;
	    		break;
	    	case MODE_CANVAS_DRAG:
	    		viewport_drag_start_point = new Point(e.getPoint());
	    		viewport_drag_end_point = new Point(e.getPoint());
    			request_set_cursor(Cursor.HAND_CURSOR);
	    		break;
	    	}
	    
	    	update_cursor();
			requestFocus();
	    	my_repaint();
	    	frame.add_undo_data_if_need();
	    	notify_update_focus();
    	}
    	//マウス右
    	else if(new_e.getButton() == new_e.BUTTON3){
    		this.set_normal_mode();
    	}
    }
  
    public void mouseClicked(MouseEvent e){
    	request_reset_cursor();
    	MouseEvent new_e = canvas_mouse_event(e);
    	
    	Canvas_Trans focused_trans = get_hit_trans(new_e.getPoint());
		Canvas_State focused_state = get_hit_state(new_e.getPoint());
		Canvas_Memo focused_memo = get_hit_memo(new_e.getPoint());
		
		if(focused_memo != null) focused_memo.mouseClicked(new_e, is_multi_focused());
		else if(focused_trans != null) focused_trans.mouseClicked(new_e, is_multi_focused());
		else if(focused_state != null) focused_state.mouseClicked(new_e, is_multi_focused());

		update_cursor();
    	my_repaint();
    	frame.add_undo_data_if_need();
    }
	public void mouseEntered(MouseEvent e){
		request_reset_cursor();
		MouseEvent new_e = canvas_mouse_event(e);
		
		update_cursor();
		my_repaint();
	}
	public void mouseExited(MouseEvent e){
		request_reset_cursor();
		MouseEvent new_e = canvas_mouse_event(e);
		
		update_cursor();
		my_repaint();
	}
	
	public void mouseDragged(MouseEvent e){
		request_reset_cursor();
		MouseEvent new_e = canvas_mouse_event(e);
		
		switch (mode){
    	case MODE_NORMAL :
    		if(viewport_drag_start_point != null && viewport_drag_end_point != null){
    			viewport_drag_end_point = new Point(e.getPoint());
    			need_repaint();
    		}
    		else{
    			for (int i = 0; i < focused_state_list.size(); i ++){
    				focused_state_list.get(i).mouseDragged(new_e, is_multi_focused());
    			}
    			for (int i = 0; i < focused_trans_list.size(); i ++){
    				focused_trans_list.get(i).mouseDragged(new_e, is_multi_focused());
    			}
    			for (int i = 0; i < focused_memo_list.size(); i ++){
    				focused_memo_list.get(i).mouseDragged(new_e, is_multi_focused());
    			}
    		}
    		break;
    	case MODE_ADD_STATE :
    		break;
    	case MODE_ADD_TRANS1 :
    		break;
    	case MODE_ADD_TRANS2 :
    		add_ct.set_dest_pos(new_e.getPoint());
    		break;
    	case MODE_ADD_MEMO :
    		break;
    	case MODE_CANVAS_DRAG :
			Point vp = scroll.getViewport().getViewPosition();  			
			vp.x -= (int)((e.getX() - viewport_drag_end_point.x) * DRAG_SCROLL_RATE);
			vp.y -= (int)((e.getY() - viewport_drag_end_point.y) * DRAG_SCROLL_RATE);
			Dimension d = scroll.getViewport().getSize();
		    this.scrollRectToVisible(new Rectangle(vp, d));
		    viewport_drag_end_point.setLocation(e.getPoint());
 		    request_set_cursor(Cursor.HAND_CURSOR);
 		    break;
		}
		
		update_cursor();
		my_repaint();
	}
	public void mouseMoved(MouseEvent e){
		request_reset_cursor();
		MouseEvent new_e = canvas_mouse_event(e);
		
		switch (mode){
    	case MODE_NORMAL :
    		if(viewport_drag_start_point != null && viewport_drag_end_point != null){
    			viewport_drag_end_point.setLocation(e.getPoint());
    			need_repaint();
    		}
    		
    		for (int i = 0; i < focused_trans_list.size(); i ++){
    			focused_trans_list.get(i).mouseMoved(new_e, is_multi_focused());
            }
    		for (int i = 0; i < focused_state_list.size(); i ++){
    			focused_state_list.get(i).mouseMoved(new_e, is_multi_focused());
            }
    		for (int i = 0; i < focused_memo_list.size(); i ++){
    			focused_memo_list.get(i).mouseMoved(new_e, is_multi_focused());
            }
    		
    		break;
    	case MODE_ADD_STATE :
    		add_cs.set_pos_without_trans(new_e.getPoint());
    		break;
    	case MODE_ADD_MEMO :
    		add_cm.set_pos(new_e.getPoint());
    		break;
    	case MODE_CANVAS_DRAG :
    		request_set_cursor(Cursor.HAND_CURSOR);
    		break;
		}
		
		update_cursor();
		my_repaint();
	}

	public void keyPressed(KeyEvent e) {
		request_reset_cursor();
		
		switch(e.getKeyCode()){
		case KeyEvent.VK_SPACE :
			switch (mode){
	    	case MODE_NORMAL :
	    		mode = MODE_CANVAS_DRAG;
	    		request_set_cursor(Cursor.DEFAULT_CURSOR);
	    	case MODE_ADD_STATE :
	    		break;
	    	case MODE_ADD_TRANS1 :
	    		break;
	    	case MODE_ADD_TRANS2 :
	    		break;
	    	case MODE_ADD_MEMO :
	    		break;
	    	case MODE_CANVAS_DRAG:
	    		request_set_cursor(Cursor.HAND_CURSOR);
	    		break;
			}
			break;
			
		case KeyEvent.VK_DELETE :
			switch (mode){
	    	case MODE_NORMAL :
	    		for (int i = 0; i < focused_state_list.size(); i ++){
	    			Canvas_State s = focused_state_list.get(i);
	    			frame.del_dialog(s.get_model());
	    			notify_del_state(s);
	    			state_list.remove(s);
	    			model_root.del_state(s.get_model());
	            }
	    		focused_state_list.clear();
	    		for (int i = 0; i < focused_trans_list.size(); i ++){
	    			Canvas_Trans t = focused_trans_list.get(i);
	    			frame.del_dialog(t.get_model());
	    			model_root.del_trans(t.get_model());
					trans_list.remove(t);
	            }
	    		focused_trans_list.clear();
	    		for (int i = 0; i < focused_memo_list.size(); i ++){
	    			Canvas_Memo m = focused_memo_list.get(i);
	    			frame.del_dialog(m.get_model());
	    			model_root.del_memo(m.get_model());
					memo_list.remove(m);
	            }
	    		focused_memo_list.clear();
	    	case MODE_ADD_STATE :
	    		break;
	    	case MODE_ADD_TRANS1 :
	    		break;
	    	case MODE_ADD_TRANS2 :
	    		break;
	    	case MODE_ADD_MEMO :
	    		break;
	    	case MODE_CANVAS_DRAG:
	    		break;
			}
		break;
		}
		
		update_cursor();
		my_repaint();
		frame.add_undo_data_if_need();
	}

	public void keyReleased(KeyEvent e){
		request_reset_cursor();
		
		switch(e.getKeyCode()){
		case KeyEvent.VK_SPACE :
			switch (mode){
	    	case MODE_NORMAL :
	    		break;
	    	case MODE_ADD_STATE :
	    		break;
	    	case MODE_ADD_TRANS1 :
	    		break;
	    	case MODE_ADD_TRANS2 :
	    		break;
	    	case MODE_ADD_MEMO :
	    		break;
	    	case MODE_CANVAS_DRAG:
	    		mode = MODE_NORMAL;
	    		request_set_cursor(Cursor.DEFAULT_CURSOR);
	    		break;
			}
			break;
		}

		update_cursor();
		my_repaint();
	}
	public void keyTyped(KeyEvent e){
		update_cursor();
		my_repaint();
	}
	
	public int multiline_string_width(Graphics g, String text){
		int max_w = 0;
		
		String [] lines = text.split("\n", -1);
		FontMetrics fm = g.getFontMetrics();
		
		for(int i = 0; i < lines.length; i ++){
			int w = 0;
			if(lines[i].length() > 0){
				w = fm.stringWidth(lines[i]);
			}
			max_w = Math.max(max_w, w);
		}
		
		return max_w;
	}
	
	public int multiline_string_height(Graphics g, String text){
		String [] lines = text.split("\n", -1);
		
		int ascent, descent;
		FontMetrics fm = g.getFontMetrics();
		ascent = fm.getAscent();
		descent = fm.getDescent();
		
		return lines.length * ascent + descent;
	}
	
	public void draw_multiline_string(Graphics g, Point p, String text, boolean is_centering){
		String [] lines = text.split("\n", -1);
		
		//フォントサイズを取得
		int ascent, descent;
		FontMetrics fm = g.getFontMetrics();
		ascent = fm.getAscent();
		descent = fm.getDescent();
		
		int [] line_w_list = new int [lines.length];
		int draw_y = p.y + ascent;
		int max_w = 0;
		
		for(int i = 0; i < lines.length; i ++){
			if(lines[i].length() > 0){
				line_w_list[i] = fm.stringWidth(lines[i]);
			}
			else{
				line_w_list[i] = 0;
			}
			max_w = Math.max(max_w, line_w_list[i]);
		}
		
		for(int i = 0; i < lines.length; i ++){
			if(is_centering){
				g.drawString(lines[i], p.x + (max_w - line_w_list[i]) / 2, draw_y);
			}
			else{
				g.drawString(lines[i], p.x, draw_y);
			}
			draw_y += ascent;
		}
	}

	public void mouseWheelMoved(MouseWheelEvent e) {
		if(e.isAltDown()){
			set_scale(scale -= e.getWheelRotation() * SCALE_RATE);
		}
		else{		
			JScrollBar bar = scroll.getVerticalScrollBar();
            int barMax = bar.getMaximum();
            int barMin = bar.getMinimum();
            int barLen = bar.getVisibleAmount();
            float rate = (barMax - barLen) * SCROLL_RATE / get_scale();
            int cur = bar.getValue();
            cur += rate * e.getWheelRotation();
            bar.setValue(cur);
		}
		
	}

	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		// TODO Auto-generated method stub
		
	}
}
