import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.MouseInputAdapter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import javax.xml.parsers.*;
import org.w3c.dom.*;

public class MyFrame extends JFrame implements MouseListener, KeyListener, Dialog_Progress_Listener, ActionListener{
	private static final String DEFAULT_TITLE = "名称未設定";
	private static final String SETTING_FILE = "Stage_for_Alloy_setting.ini";
	public static final String TMP_ALLOY_CHECK_FILE = "tmp_alloy_compiler_check.als";
	public static final String TMP_ALLOY_END_FILE = "tmp_alloy_compiler_end.txt";
	public static final String TMP_ALLOY_RESULT_FILE = "tmp_alloy_compiler_result.txt";
	private JPopupMenu popup = null;
	private Dialog_Global g_state_dialog = null;
	private Dialog_Event_List g_event_list_dialog = null;
	private Dialog_Base_State_List g_base_state_list_dialog = null;
	private Dialog_Error_Alloy g_error_alloy_dialog = null;
	private Dialog_State_Event_Matrix g_matrix_dialog = null;
	private Dialog_Simulation g_simulation_dialog = null;
	private Dialog_Scenario_List g_scenario_dialog = null;
	private MyCanvas canvas = null;
	private Model_Root model_root = null;
	private String current_file_path = null;
	private String current_export_path = null;
	JLabel status_bar = null;
	
	private ArrayList<Dialog_Trans> trans_dialog_list = null;
	private ArrayList<Dialog_State> state_dialog_list = null;
	private ArrayList<Dialog_Memo> memo_dialog_list = null;
	
	private ArrayList<Undo_Data> undo_buffer = null;
	private int undo_buffer_pos = -1;
	
	JButton b_add_state;
	JButton b_add_trans;
	JButton b_add_point;
	JButton b_add_memo;
	JButton b_zoom_in;
	JButton b_zoom_out;
	JButton b_show_global_def;
	JButton b_show_base_state_list;
	JButton b_show_event_list;
	
	JMenu file_menu;
	JMenu edit_menu;
	JMenu setting_menu;
	JMenu check_menu;
	
	JMenuItem new_menu;
	JMenuItem open_menu;
	JMenuItem save_menu;
	JMenuItem save_as_menu;
	JMenuItem alloy_menu;
	JMenuItem alloy_as_menu;
	
	JMenuItem undo_menu;
	JMenuItem redo_menu;
	JMenuItem copy_menu;
	JMenuItem cut_menu;
	JMenuItem paste_menu;
	JMenuItem move_front_menu;
	JMenuItem move_back_menu;
	
	JMenuItem font_menu;
	JMenuItem show_alloy_menu;
	JMenuItem syntax_type_check_menu;
	JMenuItem instance_check_menu;
	JMenuItem matrix_menu;
	JMenuItem simulation_menu;
	JMenuItem scenario_menu;
	
	boolean is_need_add_undo = false;
	boolean is_need_save = false;
	
	private static final int MAX_UNDO_BUFFER = 100;
	
	String font_name = "";
	int font_size = DEFAULT_FONT_SIZE;
	
	protected static final int DEFAULT_FONT_SIZE = 12;
	
	private boolean is_enable_add_undo = false;
	
	volatile Dialog_Progress d_progress = null;
	
	MyFrame(){
		// 外観を設定します
	    try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception e) {
			
		}
		
		//モデル作成		
		model_root = new Model_Root(this);
		
		d_progress = new Dialog_Progress(this);
		
		//設定ファイルの読み込み
		open_setting_file();
		
		//UI作成
		{		
			this.setTitle(DEFAULT_TITLE);
			
			Container contentPane = getContentPane(); 
		
			//ダイアログ
			create_dialogs();
			
			undo_buffer = new ArrayList<Undo_Data>();
		  
		  	//キャンバス追加
			JScrollPane scrollpane = new JScrollPane();
			
			canvas = new MyCanvas(this, model_root, scrollpane);
		
			scrollpane.getViewport().addMouseListener(canvas);
			scrollpane.getViewport().addMouseMotionListener(canvas);
			scrollpane.getViewport().addMouseWheelListener(canvas);
			
			scrollpane.setViewportView(canvas);
			scrollpane.setBorder(new EmptyBorder(0, 0, 0, 0));
			contentPane.add(scrollpane, BorderLayout.CENTER);
			
			//ツールバー追加
			JToolBar toolbar = new JToolBar();
			contentPane.add(toolbar, BorderLayout.NORTH);   
				
			toolbar.setFloatable(false);
			b_add_state = new JButton("◯ 状態");
			b_add_state.setFocusable(false);
			b_add_state.addActionListener(this);
			toolbar.add(b_add_state);
			
			toolbar.add(Box.createRigidArea(new Dimension(20,1)));
			
			b_add_trans = new JButton("→ 遷移");
			b_add_trans.setFocusable(false);
			b_add_trans.addActionListener(this);
			toolbar.add(b_add_trans);
			
			b_add_point = new JButton("○ 接続点");
			b_add_point.setFocusable(false);
			b_add_point.addActionListener(this);
			//toolbar.add(b_add_point);
			
			toolbar.add(Box.createRigidArea(new Dimension(20,1)));
			
			b_add_memo = new JButton("□ メモ");
			b_add_memo.setFocusable(false);
			b_add_memo.addActionListener(this);
			toolbar.add(b_add_memo);
			
			toolbar.add(Box.createRigidArea(new Dimension(20,1)));
			
			b_zoom_in = new JButton("+ 拡大");
			b_zoom_in.setFocusable(false);
			b_zoom_in.addActionListener(this);
			toolbar.add(b_zoom_in);
			
			b_zoom_out = new JButton("- 縮小");
			b_zoom_out.setFocusable(false);
			b_zoom_out.addActionListener(this);
			toolbar.add(b_zoom_out);
			
			toolbar.add(Box.createGlue());
				
			b_show_global_def = new JButton("共通の定義");
			b_show_global_def.setFocusable(false);
			b_show_global_def.addActionListener(this);
			toolbar.add(b_show_global_def);
			
			toolbar.add(Box.createRigidArea(new Dimension(20,1)));
			
			b_show_base_state_list = new JButton("状態の一覧");
			b_show_base_state_list.setFocusable(false);
			b_show_base_state_list.addActionListener(this);
			toolbar.add(b_show_base_state_list);
			
			toolbar.add(Box.createRigidArea(new Dimension(20,1)));
			
			b_show_event_list = new JButton("イベントの一覧");
			b_show_event_list.setFocusable(false);
			b_show_event_list.addActionListener(this);
			toolbar.add(b_show_event_list);
			
	/*		    
			//右クリックメニュー
			btn.addMouseListener(this);
			popup = new JPopupMenu();
			
			JMenuItem updateMenuItem = new JMenuItem("最新の情報に更新");
			JMenuItem propertyMenuItem = new JMenuItem("画像のプロパティ");
			popup.add(updateMenuItem);
			popup.add(propertyMenuItem);
	*/		
			//メニュー追加
			JMenuBar menubar = new JMenuBar();
			setJMenuBar(menubar);		
			
			file_menu = new JMenu("ファイル");
			menubar.add(file_menu);
			{
				new_menu = new JMenuItem("新規作成");
				new_menu.addActionListener(this);
				//new_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl N"));
				file_menu.add(new_menu);
				
				open_menu = new JMenuItem("開く");
				open_menu.addActionListener(this);
				open_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl O"));
				file_menu.add(open_menu);
				
				save_menu = new JMenuItem("保存");
				save_menu.addActionListener(this);
				save_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl S"));
				file_menu.add(save_menu);
	
				save_as_menu = new JMenuItem("名前を付けて保存");
				save_as_menu.addActionListener(this);
				file_menu.add(save_as_menu);
								
				alloy_menu = new JMenuItem("Alloy 出力");
				alloy_menu.addActionListener(this);
				alloy_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl E"));
				file_menu.add(alloy_menu);
				
				alloy_as_menu = new JMenuItem("名前をつけて Alloy 出力");
				alloy_as_menu.addActionListener(this);
				file_menu.add(alloy_as_menu);
			}

			edit_menu = new JMenu("編集");
			menubar.add(edit_menu);
			{
				undo_menu = new JMenuItem("元に戻す");
				undo_menu.addActionListener(this);
				undo_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl Z"));
				edit_menu.add(undo_menu);
				
				redo_menu = new JMenuItem("やり直し");
				redo_menu.addActionListener(this);
				redo_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl Y"));
				edit_menu.add(redo_menu);
				
				copy_menu = new JMenuItem("コピー");
				copy_menu.addActionListener(this);
				copy_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl C"));
				edit_menu.add(copy_menu);
				
				cut_menu = new JMenuItem("切り取り");
				cut_menu.addActionListener(this);
				cut_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl X"));
				edit_menu.add(cut_menu);
				
				paste_menu = new JMenuItem("貼り付け");
				paste_menu.addActionListener(this);
				paste_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl V"));
				edit_menu.add(paste_menu);
				
				move_front_menu = new JMenuItem("最前面に移動");
				move_front_menu.addActionListener(this);
				move_front_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl F"));
				edit_menu.add(move_front_menu);
				
				move_back_menu = new JMenuItem("最背面に移動");
				move_back_menu.addActionListener(this);
				move_back_menu.setAccelerator(KeyStroke.getKeyStroke("ctrl B"));
				edit_menu.add(move_back_menu);
			}
			
			setting_menu = new JMenu("設定");
			menubar.add(setting_menu);
			{
				font_menu = new JMenuItem("フォント設定");
				font_menu.addActionListener(this);
				setting_menu.add(font_menu);
			}			
			
			check_menu = new JMenu("検査");
			menubar.add(check_menu);
			{
				show_alloy_menu = new JMenuItem("Alloy 表示");
				show_alloy_menu.addActionListener(this);
				show_alloy_menu.setAccelerator(KeyStroke.getKeyStroke("F5"));
				check_menu.add(show_alloy_menu);
				
				syntax_type_check_menu = new JMenuItem("構文・型チェック");
				syntax_type_check_menu.addActionListener(this);
				syntax_type_check_menu.setAccelerator(KeyStroke.getKeyStroke("F6"));
				check_menu.add(syntax_type_check_menu);
				
				instance_check_menu = new JMenuItem("Instance チェック");
				instance_check_menu.addActionListener(this);
				instance_check_menu.setAccelerator(KeyStroke.getKeyStroke("F7"));
				check_menu.add(instance_check_menu);
				
				matrix_menu = new JMenuItem("状態遷移表");
				matrix_menu.addActionListener(this);
				matrix_menu.setAccelerator(KeyStroke.getKeyStroke("F8"));
				check_menu.add(matrix_menu);
				
				simulation_menu = new JMenuItem("シミュレーション");
				simulation_menu.addActionListener(this);
				simulation_menu.setAccelerator(KeyStroke.getKeyStroke("F9"));
				check_menu.add(simulation_menu);
				
				scenario_menu = new JMenuItem("シナリオテスト");
				scenario_menu.addActionListener(this);
				scenario_menu.setAccelerator(KeyStroke.getKeyStroke("F10"));
				check_menu.add(scenario_menu);
			}
			
			//ステータスバー
			status_bar = new JLabel("", JLabel.LEFT);
			update_status_bar();
			this.add(status_bar, BorderLayout.SOUTH);
			
			this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			this.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent e) {
					System.out.println("windowClosing");
					maybeExit();
				}
				@Override
				public void windowClosed(WindowEvent e) {
					System.out.println("windowClosed");
					System.exit(0); 
				}
			});
			
			clear_undo_data();
			is_need_save = false;
			update_title();
			
			is_enable_add_undo = true;
		}
		
		update_font();
	}
	
	private void open_setting_file(){
		try{
			FileReader in = new FileReader(SETTING_FILE);
	        BufferedReader br = new BufferedReader(in);

	        String line;
			
	        line = br.readLine();
			if(line != null){
				font_name = line;
			}

			line = br.readLine();
			if(line != null){
				font_size = Integer.valueOf(line);
			}

			br.close();
			in.close();		
		}
		catch(IOException e){
			font_name = Font.SANS_SERIF;
			font_size = DEFAULT_FONT_SIZE;
			
			save_setting_file();
		}
	}
	
	private void save_setting_file(){
		try{
	        File file = new File(SETTING_FILE);
	        FileWriter filewriter = new FileWriter(file);

	        filewriter.write(font_name + "\n" + Integer.toString(font_size));

	        filewriter.close();
		}
		catch(IOException e){
			
		}
	}
	
	public void syntax_type_check(){
		is_cancel_exec_alloy = false;
		Check_Type_Syntax_Thread thread = new Check_Type_Syntax_Thread(this);
		thread.start();
		d_progress.setTitle("構文・型チェック");
		d_progress.set_message("チェック中...");
		d_progress.setVisible(true);
	}
	
	protected void syntax_type_check_local(){
		String alloy = model_root.get_alloy();
		
		Alloy_result ret = new Alloy_result();
		ret = exec_alloy(alloy, "instance_check", "");
		
		d_progress.hide_after_show();
		
		if(is_cancel_exec_alloy == false){
			if(ret.ret != Alloy_result.CHECK_OK){
				show_Dialog_Error_Alloy("構文・型チェック NG", alloy, ret.ret_text);
			}
			else{
				JOptionPane.showMessageDialog(this, "構文・型チェック OK", "結果", JOptionPane.PLAIN_MESSAGE);
			}
		}
		
		is_cancel_exec_alloy = false;
	}
	
	public void instance_check(){
		is_cancel_exec_alloy = false;
		Check_Instance_Thread thread = new Check_Instance_Thread(this);
		thread.start();
		d_progress.setTitle("インスタンスチェック");
		d_progress.set_message("チェック中...");
		d_progress.setVisible(true);
	}
	
	protected void instance_check_local(){
		String alloy = model_root.get_alloy();
		
		Alloy_result ret = new Alloy_result();
		ret = exec_alloy(alloy, "instance_check", "");
		
		d_progress.hide_after_show();
		
		if(is_cancel_exec_alloy == false){
			if(ret.ret == Alloy_result.SYNTAX_TYPE_ERROR){
				show_Dialog_Error_Alloy("構文・型チェック NG", alloy, ret.ret_text);
			}
			else if(ret.ret == Alloy_result.NO_INSTANCE_FOUND){
				show_Dialog_Error_Alloy("インスタンスチェック NG", alloy, ret.ret_text);
			}
			else if(ret.ret == Alloy_result.ALLOY_INTERNAL_ERROR){
				show_Dialog_Error_Alloy("インスタンスチェック NG", alloy, ret.ret_text);
			}
			else{
				JOptionPane.showMessageDialog(this, "インスタンスチェック OK", "結果", JOptionPane.PLAIN_MESSAGE);
			}
		}
	}
	
	public void show_alloy(){
		delete_tmp_files();
		
		//tmp alloy file
		String alloy_text = model_root.get_alloy();
		try{
	        File file = new File(TMP_ALLOY_CHECK_FILE);
	        FileWriter filewriter = new FileWriter(file);
	        filewriter.write(alloy_text);
	        filewriter.close();
		}
		catch(IOException e){
			
		}

		try {
		    Runtime rt = Runtime.getRuntime();
		    
		    String name = System.getProperty("os.name");
		    
		    if (name.equals("Mac OS X")){
		    	rt.exec("open -a Alloy4 " + TMP_ALLOY_CHECK_FILE);
		    }
		    else{
		    	rt.exec("java -jar alloy4.jar " + TMP_ALLOY_CHECK_FILE);
		    }		    	
		} 
		catch (IOException ex) {
			ex.printStackTrace();
		} 

	}
	
	
	
	public void show_matrix(){
		g_matrix_dialog.set_model(model_root);
		g_matrix_dialog.show();
		g_matrix_dialog.instance_check();	
	}
	
	public void show_simulation(){
		g_simulation_dialog.set_model(model_root);
		g_simulation_dialog.show();		
	}
	
	public void show_scenario(){
		g_scenario_dialog.set_model(model_root);
		g_scenario_dialog.show();		
	}
	
	public void set_font(String name, int size){
		font_name = name;
		font_size = size;
		notify_update_data();
		
		update_font();
		
		repaint();
		
		save_setting_file();
	}
	
	public Font get_font(){
		return new Font(get_font_name(), Font.PLAIN, get_org_font_size());
	}
	
	protected void update_font(){
		Font font = get_font();
		
		file_menu.setFont(font);
		edit_menu.setFont(font);
		setting_menu.setFont(font);
		check_menu.setFont(font);
		
		new_menu.setFont(font);
		open_menu.setFont(font);
		save_menu.setFont(font);
		save_as_menu.setFont(font);
		alloy_menu.setFont(font);
		alloy_as_menu.setFont(font);
		
		undo_menu.setFont(font);
		redo_menu.setFont(font);
		copy_menu.setFont(font);
		cut_menu.setFont(font);
		paste_menu.setFont(font);
		move_front_menu.setFont(font);
		move_back_menu.setFont(font);
		
		font_menu.setFont(font);
		show_alloy_menu.setFont(font);
		syntax_type_check_menu.setFont(font);
		instance_check_menu.setFont(font);
		matrix_menu.setFont(font);
		simulation_menu.setFont(font);
		scenario_menu.setFont(font);
	
		b_add_state.setFont(font);
		b_add_trans.setFont(font);
		b_add_point.setFont(font);
		b_add_memo.setFont(font);
		b_zoom_in.setFont(font);
		b_zoom_out.setFont(font);
		b_show_global_def.setFont(font);
		b_show_base_state_list.setFont(font);
		b_show_event_list.setFont(font);
		status_bar.setFont(font);
		
		this.setFont(font);

		Enumeration<Object> enumeration = UIManager.getDefaults().keys();
		while(enumeration.hasMoreElements()){
		    Object key = enumeration.nextElement();
		    Object value = UIManager.get(key);
		    if (value instanceof java.awt.Font){
		        UIManager.put(key.toString(), font);
		    }
		}
	}
	
	public String get_font_name(){
		return font_name;
	}
	
	public int get_org_font_size(){
		return font_size;
	}

	private void maybeExit(){
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		if(is_need_save == false){
			dispose();
			return;
		}
		Object[] options = { "保存", "終了", "キャンセル" };
		int retValue = JOptionPane.showOptionDialog(this,
		        "ファイルが保存されていません。終了しますか？",
		        "Exit Options",
		        JOptionPane.YES_NO_CANCEL_OPTION,
		        JOptionPane.WARNING_MESSAGE, null, options, options[0]);
		if(retValue==JOptionPane.YES_OPTION){
			save_as_data_xml();			
		}
		else if(retValue==JOptionPane.NO_OPTION){
			dispose();
			return;
		}
		else if(retValue==JOptionPane.CANCEL_OPTION){
			
		}
	}
	
	public void update_status_bar(){
		status_bar.setText(" Zoom : " + canvas.get_org_scale() + "%");
	}
	
	public void zoom_in(){
		canvas.zoom_in();
	}
	
	public void zoom_out(){
		canvas.zoom_out();
	}
	
	public Dialog_State get_state_dialog(Model_State model){
		for(int i = 0; i < state_dialog_list.size(); i ++){
			Dialog_State d = state_dialog_list.get(i);
			if(d.get_model() == model) return d;
		}
		
		return null;
	}
	
	public Dialog_Trans get_trans_dialog(Model_Trans model){
		for(int i = 0; i < trans_dialog_list.size(); i ++){
			Dialog_Trans d = trans_dialog_list.get(i);
			if(d.get_model() == model) return d;
		}
		
		return null;
	}
	
	public Dialog_Memo get_memo_dialog(Model_Memo model){
		for(int i = 0; i < memo_dialog_list.size(); i ++){
			Dialog_Memo d = memo_dialog_list.get(i);
			if(d.get_model() == model) return d;
		}
		
		return null;
	}
	
	public void show_state_dialog(Model_State model){
		Dialog_State d = get_state_dialog(model);
		//無かったら作る
		if(d == null){
			d = new Dialog_State(this, model_root, model);
			state_dialog_list.add(d);
			//Undo情報に追加
			update_undo_dialog_info();
		}
		
		//あったらshow
		d.show(true);
	}
	
	public void show_trans_dialog(Model_Trans model){
		Dialog_Trans d = get_trans_dialog(model);
		//無かったら作る
		if(d == null){
			d = new Dialog_Trans(this, model_root, model);
			trans_dialog_list.add(d);
			//Undo情報に追加
			update_undo_dialog_info();
		}
		
		//あったらshow
		d.show(true);
	}
	
	public void show_memo_dialog(Model_Memo model){
		Dialog_Memo d = get_memo_dialog(model);
		//無かったら作る
		if(d == null){
			d = new Dialog_Memo(this, model_root, model);
			memo_dialog_list.add(d);
			//Undo情報に追加
			update_undo_dialog_info();
		}
		
		//あったらshow
		d.show(true);
	}
	
	private void update_undo_dialog_info(){
		undo_buffer.get(undo_buffer.size() - 1).set_dialog_info(get_state_dialog_info_list(), get_trans_dialog_info_list(), get_memo_dialog_info_list());
	}
	
	public void notify_update_data(Model_Root _model, int flag){
		
	}
	
	public void notify_update_data(Model_State _model, int flag){
		
	}
	
	private void update_title(){
		String new_title = "";
		if(current_file_path == null){
			new_title = DEFAULT_TITLE;
		}
		else{
			new_title = current_file_path;
		}
		
		if(is_need_save){
			new_title += " *";
		}
		
		if(new_title.equalsIgnoreCase(this.getTitle()) == false){
			this.setTitle(new_title);
		}
	}
	
	public void notify_update_focus(){
		Undo_Data undo_data = undo_buffer.get(undo_buffer_pos);
		undo_data.set_focused_index(canvas.get_focused_state_index_list(), canvas.get_focused_trans_index_list(), canvas.get_focused_memo_index_list());
	}
	
	public void notify_update_data(){
		if(canvas != null) canvas.need_repaint();
		if(g_state_dialog != null){
			if(g_state_dialog.isShowing()) g_state_dialog.update_data();
		}
		if(g_event_list_dialog != null){
			if(g_event_list_dialog.isShowing()) g_event_list_dialog.update_data();
		}
		if(g_base_state_list_dialog != null){
			if(g_base_state_list_dialog.isShowing()) g_base_state_list_dialog.update_data();
		}
		if(g_scenario_dialog != null){
			if(g_scenario_dialog.isShowing()) g_scenario_dialog.update_data();
		}
		
		for(int i = 0; i < trans_dialog_list.size(); i ++){
			Dialog_Trans d = trans_dialog_list.get(i);
			d.update_data();
		}
		
		for(int i = 0; i < state_dialog_list.size(); i ++){
			Dialog_State d = state_dialog_list.get(i);
			d.update_data();
		}
		
		for(int i = 0; i < memo_dialog_list.size(); i ++){
			Dialog_Memo d = memo_dialog_list.get(i);
			d.update_data();
		}
		
		if(is_enable_add_undo){
			is_need_add_undo = true;
			is_need_save = true;
			update_title();
		}
	}
	
	public MyCanvas get_canvas(){
		return canvas;
	}
	
	//オープンダイアログ
    String openfile(FilenameFilter filter){
        FileDialog fd=new FileDialog(new Frame(),"File Select");
        fd.setFilenameFilter(filter);
        fd.show();
        String fullpath = "";
        if(fd.getDirectory() != null && fd.getFile() != null) fullpath =fd.getDirectory()+fd.getFile();
        fd.dispose();
        return fullpath;
    }
    
    //セーブダイアログ
    String writefile(FilenameFilter filter){
        FileDialog fd=new FileDialog(this,"File Save",FileDialog.SAVE);
        fd.setFilenameFilter(filter);
        fd.show();
        String fullpath = "";
        if(fd.getDirectory() != null && fd.getFile() != null) fullpath =fd.getDirectory()+fd.getFile();
        fd.dispose();
        return fullpath;
    }
    
    //Exportダイアログ
    String exportfile(FilenameFilter filter){
        FileDialog fd=new FileDialog(this,"Alloy Export",FileDialog.SAVE);
        fd.setFilenameFilter(filter);
        fd.show();
        String fullpath = "";
        if(fd.getDirectory() != null && fd.getFile() != null) fullpath =fd.getDirectory()+fd.getFile();
        fd.dispose();
        return fullpath;
    }
    
    private int get_state_model_index(Model_State model){
    	return model_root.get_state_list().indexOf(model); 
    }
    
    private int get_trans_model_index(Model_Trans model){
    	return model_root.get_trans_list().indexOf(model); 
    }
    
    private int get_memo_model_index(Model_Memo model){
    	return model_root.get_memo_list().indexOf(model); 
    }
    
    public void del_dialog(Model_State model){
    	for(int i = 0; i < state_dialog_list.size(); i ++){
    		Dialog_State d = state_dialog_list.get(i);
    		if(d.get_model() == model){
    			d.dispose();
    			state_dialog_list.remove(d);
    			return;
    		}
    	}
    }
    
    public void del_dialog(Model_Trans model){
    	for(int i = 0; i < trans_dialog_list.size(); i ++){
    		Dialog_Trans d = trans_dialog_list.get(i);
    		if(d.get_model() == model){
    			d.dispose();
    			trans_dialog_list.remove(d);
    			return;
    		}
    	}
    }
    
    public void del_dialog(Model_Memo model){
    	for(int i = 0; i < memo_dialog_list.size(); i ++){
    		Dialog_Memo d = memo_dialog_list.get(i);
    		if(d.get_model() == model){
    			d.dispose();
    			memo_dialog_list.remove(d);
    			return;
    		}
    	}
    }
    
    private ArrayList<Undo_Dialog_Info> get_state_dialog_info_list(){
    	ArrayList<Undo_Dialog_Info> list = new ArrayList<Undo_Dialog_Info>();
    	
    	for(int i = 0; i < state_dialog_list.size(); i ++){
    		Dialog_State d = state_dialog_list.get(i);
    		Model_State model = d.get_model();
    		
    		Undo_Dialog_Info info = new Undo_Dialog_Info(d, get_state_model_index(model));
    		
			list.add(info);
		}    	
    	
    	return list;
    }
    
    private ArrayList<Undo_Dialog_Info> get_trans_dialog_info_list(){
    	ArrayList<Undo_Dialog_Info> list = new ArrayList<Undo_Dialog_Info>();
    	
    	for(int i = 0; i < trans_dialog_list.size(); i ++){
    		Dialog_Trans d = trans_dialog_list.get(i);
    		Model_Trans model = d.get_model();
    		
    		Undo_Dialog_Info info = new Undo_Dialog_Info(d, get_trans_model_index(model));
    		
			list.add(info);
		}    	
    	
    	return list;
    }
    
    private ArrayList<Undo_Dialog_Info> get_memo_dialog_info_list(){
    	ArrayList<Undo_Dialog_Info> list = new ArrayList<Undo_Dialog_Info>();
    	
    	for(int i = 0; i < memo_dialog_list.size(); i ++){
    		Dialog_Memo d = memo_dialog_list.get(i);
    		Model_Memo model = d.get_model();
    		
    		Undo_Dialog_Info info = new Undo_Dialog_Info(d, get_memo_model_index(model));
    		
			list.add(info);
		}    	
    	
    	return list;
    }
    
    private int undo_info_model_index(ArrayList<Undo_Dialog_Info> dialog_info_list, Dialog d){
    	for(int i = 0; i < dialog_info_list.size(); i ++){
    		Undo_Dialog_Info info = dialog_info_list.get(i);
    		
    		if(info.get_dialog() == d) return info.get_model_index();
    	}
    	return -1;
    }
    
    private void set_model_to_dialog(ArrayList<Undo_Dialog_Info> state_dialog_info_list, ArrayList<Undo_Dialog_Info> trans_dialog_info_list, ArrayList<Undo_Dialog_Info> memo_dialog_info_list){
    	g_state_dialog.set_model(model_root, model_root.get_global_state());
    	g_event_list_dialog.set_model(model_root);
    	g_base_state_list_dialog.set_model(model_root);
    	
    	for(int i = 0; i < memo_dialog_list.size(); i ++){
    		Dialog_Memo d = memo_dialog_list.get(i);
    		
    		//UndoInfoにないDialogは削除
    		int index = undo_info_model_index(memo_dialog_info_list, d);
    		if(index == -1){
    			d.dispose();
    			memo_dialog_list.remove(d);
    			i --;
    		}
    		//modelをセット
    		else{
    			d.set_model(model_root, model_root.get_memo_list().get(index));
    		}
    	}
    	
    	for(int i = 0; i < trans_dialog_list.size(); i ++){
    		Dialog_Trans d = trans_dialog_list.get(i);
    		
    		//UndoInfoにないDialogは削除
    		int index = undo_info_model_index(trans_dialog_info_list, d);
    		if(index == -1){
    			d.dispose();
    			trans_dialog_list.remove(d);
    			i --;
    		}
    		//modelをセット
    		else{
    			d.set_model(model_root, model_root.get_trans_list().get(index));
    		}
    	}
    	
    	for(int i = 0; i < state_dialog_list.size(); i ++){
    		Dialog_State d = state_dialog_list.get(i);
    		
    		//UndoInfoにないDialogは削除
    		int index = undo_info_model_index(state_dialog_info_list, d);
    		if(index == -1){
    			d.dispose();
    			state_dialog_list.remove(d);
    			i --;
    		}
    		//modelをセット
    		else{
    			d.set_model(model_root, model_root.get_state_list().get(index));
    		}
    	}
    }
    
    private void create_dialogs(){
		g_state_dialog = new Dialog_Global(this, model_root, model_root.get_global_state());
		g_event_list_dialog = new Dialog_Event_List(this, model_root);
		g_base_state_list_dialog = new Dialog_Base_State_List(this, model_root);
		g_error_alloy_dialog = new Dialog_Error_Alloy(this, "", "", "");
		g_matrix_dialog = new Dialog_State_Event_Matrix(this, model_root);
		g_simulation_dialog = new Dialog_Simulation(this, model_root);
		g_scenario_dialog = new Dialog_Scenario_List(this, model_root);
		
		trans_dialog_list = new ArrayList<Dialog_Trans>();
		state_dialog_list = new ArrayList<Dialog_State>();
		memo_dialog_list = new ArrayList<Dialog_Memo>();
	}
    
    private void close_all_dialog(){
    	g_state_dialog.dispose();
    	g_state_dialog = null;
    	
    	g_event_list_dialog.dispose();
    	g_event_list_dialog = null;
    	
    	g_base_state_list_dialog.dispose();
    	g_base_state_list_dialog = null;
    	
    	g_error_alloy_dialog.dispose();
    	g_error_alloy_dialog = null;
    	
    	g_matrix_dialog.dispose();
    	g_matrix_dialog = null;
    	
    	g_simulation_dialog.dispose();
    	g_simulation_dialog = null;
    	
    	g_scenario_dialog.dispose();
    	g_scenario_dialog = null;
    	
    	for(int i = 0; i < trans_dialog_list.size(); i ++){
    		trans_dialog_list.get(i).dispose();
    	}
    	trans_dialog_list.clear();
    	for(int i = 0; i < state_dialog_list.size(); i ++){
    		state_dialog_list.get(i).dispose();
    	}
    	state_dialog_list.clear();
    	for(int i = 0; i < memo_dialog_list.size(); i ++){
    		memo_dialog_list.get(i).dispose();
    	}
    	memo_dialog_list.clear();
    }
    
    public void clear_undo_data(){
    	undo_buffer.clear();
    	undo_buffer_pos = -1;
    	add_undo_data();
    	update_undo_redo_menu();
    }
    
    public void update_undo_redo_menu(){
    	if(undo_buffer_pos == 0) undo_menu.setEnabled(false);
    	else undo_menu.setEnabled(true);
    	
    	if(undo_buffer_pos == undo_buffer.size() - 1) redo_menu.setEnabled(false);
    	else redo_menu.setEnabled(true);
    }  
    
    public void add_undo_data_if_need(){
    	if(is_need_add_undo) add_undo_data();
    }
    
    private void add_undo_data(){
    	try {
    		if(undo_buffer.size() == MAX_UNDO_BUFFER){
    			undo_buffer.remove(0);
    			undo_buffer_pos --;
    		}
    		while(undo_buffer.size() - 1 > undo_buffer_pos){
    			undo_buffer.remove(undo_buffer.size() - 1);
    		}
    		
	    	ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(); 
	    	ObjectOutputStream outObject = new ObjectOutputStream(byteArrayOutputStream);
	    	outObject.writeObject(model_root);
	    	outObject.flush();
			outObject.close();
			
			Undo_Data undo_data = new Undo_Data(byteArrayOutputStream.toByteArray());
			undo_data.set_focused_index(canvas.get_focused_state_index_list(), canvas.get_focused_trans_index_list(), canvas.get_focused_memo_index_list());
			undo_data.set_dialog_info(get_state_dialog_info_list(), get_trans_dialog_info_list(), get_memo_dialog_info_list());
			undo_buffer.add(undo_data);
			undo_buffer_pos ++;
			update_undo_redo_menu();
			is_need_add_undo = false;
    	}
    	catch (IOException e) {
			JOptionPane.showMessageDialog(this, "Undoデータが保存できませんでした。", "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		}
    }
    
    public void undo(){
    	canvas.set_normal_mode();
    	
    	if(undo_buffer_pos == 0) return;
    	
    	try {
    		Undo_Data undo_data = undo_buffer.get(undo_buffer_pos - 1);
    		ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(undo_data.get_data());
			ObjectInputStream inObject = new ObjectInputStream(byteArrayInputStream);
			
			is_enable_add_undo = false;
			
			//現データをクリア
			int scale = canvas.scale;
			canvas.clear_data();
			
			//オブジェクトの読み込み
			model_root = (Model_Root)inObject.readObject();
			model_root.set_frame(this);
			
			//データをセット
			canvas.set_new_data(model_root);
			canvas.set_focused_index(undo_data.get_focused_state_index_list(), undo_data.get_focused_trans_index_list(), undo_data.get_focused_memo_index_list());
			canvas.set_scale(scale);
			
			set_model_to_dialog(undo_data.get_state_dialog_list(), undo_data.get_trans_dialog_list(), undo_data.get_memo_dialog_list());
						
			inObject.close();
			
			undo_buffer_pos --;
			update_undo_redo_menu();
			
			is_enable_add_undo = true;
    	}
    	catch (IOException e) {
    		is_enable_add_undo = true;
			JOptionPane.showMessageDialog(this, "Undoデータが読み込めませんでした。", "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		} catch (ClassNotFoundException e) {
			is_enable_add_undo = true;
			JOptionPane.showMessageDialog(this, "Undoデータが読み込めませんでした。", "エラー", JOptionPane.ERROR_MESSAGE);
		}
    }
    
    public void redo(){
    	canvas.set_normal_mode();
    	
    	if(undo_buffer_pos == undo_buffer.size() - 1) return;
    	
    	try {
    		Undo_Data undo_data = undo_buffer.get(undo_buffer_pos + 1);
    		ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(undo_data.get_data());
			ObjectInputStream inObject = new ObjectInputStream(byteArrayInputStream);
			
			is_enable_add_undo = false;
			
			//現データをクリア
			canvas.clear_data();
			
			//オブジェクトの読み込み
			model_root = (Model_Root)inObject.readObject();
			model_root.set_frame(this);
			
			//データをセット
			canvas.set_new_data(model_root);
			canvas.set_focused_index(undo_data.get_focused_state_index_list(), undo_data.get_focused_trans_index_list(), undo_data.get_focused_memo_index_list());
			canvas.repaint();
			set_model_to_dialog(undo_data.get_state_dialog_list(), undo_data.get_trans_dialog_list(), undo_data.get_memo_dialog_list());
			
			inObject.close();
			
			undo_buffer_pos ++;
			update_undo_redo_menu();
			
			is_enable_add_undo = true;
    	}
    	catch (IOException e) {
    		is_enable_add_undo = true;
			JOptionPane.showMessageDialog(this, "IOException Undoデータが読み込めませんでした。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		} catch (ClassNotFoundException e) {
			is_enable_add_undo = true;
			JOptionPane.showMessageDialog(this, "ClassNotFoundException Undoデータが読み込めませんでした。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
		}
    }
	
	public void new_data(){
		is_enable_add_undo = false;
		
		//現データをクリア
		close_all_dialog();
		canvas.clear_data();
	
		model_root = new Model_Root(this);
		
		//データをセット
		create_dialogs();
		
		canvas.set_new_data(model_root);
		canvas.repaint();
		current_file_path = null;
		current_export_path = null;
		
		clear_undo_data();
		
		is_need_save = false;
		update_title();
		
		is_enable_add_undo = true;
	}
		
	public void save_data_xml(){
		if(current_file_path == null) {
			save_as_data_xml();
		}
		else{
			canvas.set_normal_mode();
			
			try {
				String xml = model_root.get_xml();
				PrintWriter outFile = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(current_file_path),"Shift-JIS"))); 
				outFile.write(xml);
				outFile.flush();
				outFile.close();
			} 
			catch (IOException e) {
				JOptionPane.showMessageDialog(this, "IOException ファイルが保存できませんでした。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		
		is_need_save = false;
		update_title();
	}
	
	public void save_as_data_xml(){		
		canvas.set_normal_mode();
		
		try {
			String file_path = writefile(new FilenameFilter(){
				public boolean accept(File dir, String name){
					return (name.endsWith(".xml"));
			    }
			});
			if(file_path == "") return;
			if(file_path.endsWith(".xml") == false){
				file_path += ".xml";
			}
			
			String xml = model_root.get_xml();
			PrintWriter outFile = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file_path),"Shift-JIS"))); 
			outFile.write(xml);
			outFile.flush();
			outFile.close();
			
			current_file_path = file_path;	
			is_need_save = false;
			update_title();
		} 
		catch (IOException e) {
			JOptionPane.showMessageDialog(this, "IOException ファイルが保存できませんでした。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	
	public void open_data_xml(){
		DocumentBuilderFactory dbfactory = null;
		DocumentBuilder builder = null;
		Document doc = null;
		File file = null;
		
		canvas.set_normal_mode();
		
		String file_path = openfile(new FilenameFilter(){
		    public boolean accept(File dir, String name){
		        return (name.endsWith(".xml"));
		    }
		  });
		if(file_path == "") return;
		
		try {	    
		    // ドキュメントビルダーファクトリを生成
		    dbfactory = DocumentBuilderFactory.newInstance();
		    // ドキュメントビルダーを生成
		    builder = dbfactory.newDocumentBuilder();
		    // パースを実行してDocumentオブジェクトを取得
		    file = new File(file_path);
		    doc = builder.parse(file);
		    
		    is_enable_add_undo = false;
			
			//現データをクリア
			close_all_dialog();
			canvas.clear_data();
			
			//(9)オブジェクトの読み込み
			model_root = new Model_Root(this, doc);
			model_root.set_frame(this);
			
			//データをセット
			create_dialogs();
			canvas.set_new_data(model_root);
			canvas.repaint();
			
			clear_undo_data();
			
			is_need_save = false;
			update_title();
			
			is_enable_add_undo = true;
		} 
		catch (IOException e) {
			is_enable_add_undo = true;
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "IOException ファイルが壊れています。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);		
			return;
		} catch (ParserConfigurationException e) {
			is_enable_add_undo = true;
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "ParserConfigurationException ファイルが壊れています。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		} catch (SAXException e) {
			is_enable_add_undo = true;
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "SAXException ファイルが壊れています。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		current_file_path = file_path;
		this.setTitle(file_path);
		repaint();
	}
	
	protected void export_alloy_local(String file_path){
		canvas.set_normal_mode();
		
		try {
			if(file_path == null || file_path == "") return;
			if(file_path.endsWith(".als") == false){
				file_path += ".als";
			}
		
			FileWriter filewriter = new FileWriter(file_path);
			filewriter.write(model_root.get_alloy());
			filewriter.flush();
			filewriter.close();
			
			current_export_path = file_path;
		} 
		catch (IOException e) {
			JOptionPane.showMessageDialog(this, "IOException ファイルが保存できませんでした。\n" + e.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	
	public void export_alloy(){
		if(current_export_path != null){
			export_alloy_local(current_export_path);
		}
		else{
			export_alloy_as();
		}
	}
	
	public void export_alloy_as(){
		String file_path = exportfile(new FilenameFilter(){
		    public boolean accept(File dir, String name){
		        return (name.endsWith(".als"));
		    }
		  });
		if(file_path == "") return;
	
		export_alloy_local(file_path);
	}
	
	private static boolean checkBeforeWritefile(File file){
		if (file.exists()){
			if (file.isFile() && file.canWrite()){
				return true;
			}
		}
		else{
			return true;
		}
	
		return false;
	}

	public void show_Dialog_Global(){
		canvas.set_normal_mode();
		
		g_state_dialog.show();
	}
	
	public void show_Dialog_EventList(){
		canvas.set_normal_mode();
		
		g_event_list_dialog.show();
	}
	
	public void show_Dialog_EventList(Model_Event e){
		canvas.set_normal_mode();
		
		g_event_list_dialog.show(e);
	}
	
	public void show_Dialog_Base_State_List(){
		canvas.set_normal_mode();
		
		g_base_state_list_dialog.show();
	}
	
	public void show_Dialog_Base_State_List(Model_Base_State s){
		canvas.set_normal_mode();
		
		g_base_state_list_dialog.show(s);
	}
	
	public void show_Dialog_Error_Alloy(String _title, String _alloy_text, String _error_text){
		canvas.set_normal_mode();
		
		g_error_alloy_dialog.set_source(_title, _alloy_text, _error_text);
		g_error_alloy_dialog.setVisible(true);
	}

	// Action
	public void mouseReleased(MouseEvent e){
		showPopup(e);
	}
	
	public void mousePressed(MouseEvent e){
		showPopup(e);
	}
	
	public void mouseClicked(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	
	public void keyPressed(KeyEvent e) {}
	
	private void showPopup(MouseEvent e){
		if (e.isPopupTrigger()) {
			popup.show(e.getComponent(), e.getPoint().x, e.getPoint().y);
		}
	}

	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	public void font_setting(){
		Dialog_Font d = new Dialog_Font(this, model_root);
		d.show();
	}
	
	public void delete_file(String path){
		File file = new File(path);

	    if (file.exists()){
	    	if (file.delete()){
	    		//System.out.println("ファイルを削除しました");
	    	}
	    	else{
	    		//System.out.println("ファイルの削除に失敗しました");
	    	}
	    }
	}
	
	public void delete_tmp_files(){
		delete_file(MyFrame.TMP_ALLOY_CHECK_FILE);
		delete_file(MyFrame.TMP_ALLOY_RESULT_FILE);
		delete_file(MyFrame.TMP_ALLOY_END_FILE);
	}
	
	public String get_file_text(String path) throws IOException{
		String text = "";
		
		try{
            FileReader fr = new FileReader(path);
            BufferedReader br = new BufferedReader(fr);
        
            while(true){
            	String tmp = br.readLine();
            	if(tmp == null) break;
            	text += tmp + "\n";
            }
            br.close();
        }
        catch(IOException e){
        	throw e;
        }
        
		return text;
	}
	
	boolean is_end_alloy(){
		File file = new File(TMP_ALLOY_END_FILE);

	    return file.exists();
	}
	
	volatile Boolean is_cancel_exec_alloy = false;
	public void cancel_exec_alloy(){
		is_cancel_exec_alloy = true;
	}
	
	public Alloy_result exec_alloy(String alloy, String exec_type, String command){
		is_cancel_exec_alloy = false;
		delete_tmp_files();
		
		//tmp alloy file
		try{
	        File file = new File(MyFrame.TMP_ALLOY_CHECK_FILE);
	        FileWriter filewriter = new FileWriter(file);
	        filewriter.write(alloy);
	        filewriter.close();
		}
		catch(IOException e){
			
		}
	
		Alloy_result result = new Alloy_result();
		int ret = 0;
		try {
		    Runtime rt = Runtime.getRuntime();
		    String arg = " " + exec_type;
		    if(command.isEmpty() == false){
		    	arg += " " + command;
		    }
		    Process p = rt.exec("java -jar alloy_compiler.jar " + MyFrame.TMP_ALLOY_CHECK_FILE + arg);
		    
		    while(true){
		    	if(is_cancel_exec_alloy){
		    		//is_cancel_exec_alloy = false;
		    		result.ret = Alloy_result.CANCELED;
		    		delete_tmp_files();
		    		return result;
		    	}
		    	
		    	try{
		    		ret = p.exitValue();
		    		break;
		    	}
		    	catch(IllegalThreadStateException e){
		    		Thread.sleep(1000);
		    	}
		    }
		    
		    result.ret_text = get_file_text(MyFrame.TMP_ALLOY_RESULT_FILE);
		} 
		catch (IOException ex) {
			result.ret_text = ex.getMessage();
		} 
		catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(ret == 0){
			result.ret = Alloy_result.CHECK_OK;
		}
		else if(ret == 1){
			result.ret = Alloy_result.SYNTAX_TYPE_ERROR;
		}
		else if(ret == 2){
			result.ret = Alloy_result.NO_INSTANCE_FOUND;
		}
		else if(ret == 3 || ret == 4){
			JOptionPane.showMessageDialog(this, "内部エラー(" + ret + ")", "結果", JOptionPane.ERROR_MESSAGE);
			result.ret = Alloy_result.ALLOY_INTERNAL_ERROR;
		}
		
		//delete_tmp_files();
		//is_cancel_exec_alloy = false;
		
		return result;
	}

	public void progress_cancel_clicked() {
		this.cancel_exec_alloy();
	}

	public void actionPerformed(ActionEvent e) {		
		if(e.getSource() == b_add_state) {
			canvas.setAddStateMode();
		}
		if(e.getSource() == b_add_trans) {
			canvas.setAddTransMode();
		}
		if(e.getSource() == b_add_point) {
			canvas.setAddConnectPointMode();
		}
		if(e.getSource() == b_add_memo) {
			canvas.setAddMemoMode();
		}
		if(e.getSource() == b_zoom_in) {
			zoom_in();
		}
		if(e.getSource() == b_zoom_out) {
			zoom_out();
		}
		if(e.getSource() == b_show_global_def) {
			show_Dialog_Global();
		}
		if(e.getSource() == b_show_base_state_list) {
			show_Dialog_Base_State_List();
		}
		if(e.getSource() == b_show_event_list) {
			show_Dialog_EventList();
		}
		if(e.getSource() == new_menu) {
			new_data();
		}
		if(e.getSource() == open_menu) {
			open_data_xml();
		}
		if(e.getSource() == save_menu) {
			save_data_xml();
		}
		if(e.getSource() == save_as_menu) {
			save_as_data_xml();
		}
		if(e.getSource() == alloy_menu) {
			export_alloy();
		}
		if(e.getSource() == alloy_as_menu) {
			export_alloy_as();
		}
		if(e.getSource() == undo_menu) {
			undo();
		}
		if(e.getSource() == redo_menu) {
			redo();
		}
		if(e.getSource() == copy_menu) {
			canvas.copy();
		}
		if(e.getSource() == cut_menu) {
			canvas.cut();
		}
		if(e.getSource() == paste_menu) {
			canvas.paste();
		}
		if(e.getSource() == move_front_menu) {
			canvas.focused_move_front();
		}
		if(e.getSource() == move_back_menu) {
			canvas.focused_move_back();
		}
		if(e.getSource() == font_menu) {
			font_setting();
		}
		if(e.getSource() == show_alloy_menu) {
			show_alloy();
		}
		if(e.getSource() == syntax_type_check_menu) {
			syntax_type_check();
		}
		if(e.getSource() == instance_check_menu) {
			instance_check();
		}
		if(e.getSource() == matrix_menu) {
			show_matrix();
		}
		if(e.getSource() == simulation_menu) {
			show_simulation();
		}
		if(e.getSource() == scenario_menu) {
			show_scenario();
		}
	}
}

class Alloy_result{
	public static int CANCELED = -1;
	public static int CHECK_OK = 0;
	public static int SYNTAX_TYPE_ERROR = 1;
	public static int NO_INSTANCE_FOUND = 2;
	public static int ALLOY_INTERNAL_ERROR = 3;
	
	protected int ret;
	protected String ret_text = "";
}


class Check_Type_Syntax_Thread extends Thread{
	MyFrame owner;
	
	Check_Type_Syntax_Thread(MyFrame _owner){
		super();
		owner = _owner;
	}
	
	public void run() {
		owner.syntax_type_check_local();
	}
}

class Check_Instance_Thread extends Thread{
	MyFrame owner;
	
	Check_Instance_Thread(MyFrame _owner){
		super();
		owner = _owner;
	}
	
	public void run() {
		owner.instance_check_local();
	}
}
