/* Alloy Analyzer 4 -- Copyright (c) 2006-2009, Felix Chang
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package edu.mit.csail.sdg.alloy4whole;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import edu.mit.csail.sdg.alloy4.A4Reporter;
import edu.mit.csail.sdg.alloy4.ConstList;
import edu.mit.csail.sdg.alloy4.Err;
import edu.mit.csail.sdg.alloy4.ErrorWarning;
import edu.mit.csail.sdg.alloy4.SafeList;
import edu.mit.csail.sdg.alloy4compiler.ast.Command;
import edu.mit.csail.sdg.alloy4compiler.ast.Module;
import edu.mit.csail.sdg.alloy4compiler.ast.Sig;
import edu.mit.csail.sdg.alloy4compiler.ast.Sig.PrimSig;
import edu.mit.csail.sdg.alloy4compiler.ast.Sig.SubsetSig;
import edu.mit.csail.sdg.alloy4compiler.parser.CompUtil;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Options;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;
import edu.mit.csail.sdg.alloy4compiler.translator.TranslateAlloyToKodkod;
import edu.mit.csail.sdg.alloy4viz.VizGUI;

/** This class demonstrates how to access Alloy4 via the compiler methods. */

public final class alloy_compiler {

    /*
     * Execute every command in every file.
     *
     * This method parses every file, then execute every command.
     *
     * If there are syntax or type errors, it may throw
     * a ErrorSyntax or ErrorType or ErrorAPI or ErrorFatal exception.
     * You should catch them and display them,
     * and they may contain filename/line/column information.
     */
    public static void main(String[] args) throws Err {
    	int ret = 0;
    	String ret_text = "";
    	
    	if (args.length == 0){
    	//	system_exit(0, ret_text);
    	}
    	
        // The visualizer (We will initialize it to nonnull when we visualize an Alloy solution)
        VizGUI viz = null;

        // Alloy4 sends diagnostic messages and progress reports to the A4Reporter.
        // By default, the A4Reporter ignores all these events (but you can extend the A4Reporter to display the event for the user)
        A4Reporter rep = new A4Reporter() {
            // For example, here we choose to display each "warning" by printing it to System.out
            @Override public void warning(ErrorWarning msg) {
                //System.out.print("Relevance Warning:\n"+(msg.toString().trim())+"\n\n");
                //System.out.flush();
            }
        };
        
        String filename = args[0];
        //String filename = "tmp_alloy_compiler_check.als";
        String check_type = "";
        if(args.length >= 2){
        	check_type = args[1];
        }  
        
       //check_type = "command";
   
        //\E^`FbN
        Module world = null;
    	try {
            // Parse+typecheck the model
            world = CompUtil.parseEverything_fromFile(rep, null, filename);            
        }
    	catch (Err ex) {
    		ret_text += ex.getMessage();
    		system_exit(1, ret_text);
		}
  
    	//instance found `FbN
    	if(check_type.equals("instance_check")){
    		try {
	            // Choose some default options for how you want to execute the commands
	            A4Options options = new A4Options();
	            options.solver = A4Options.SatSolver.SAT4J;
	
	            for (Command command: world.getAllCommands()) {
	                // Execute the command
	            	ret_text += "== "+command+": ==\n";
	                A4Solution ans = TranslateAlloyToKodkod.execute_command(rep, world.getAllReachableSigs(), command, options);
	                if (ans.satisfiable()) {
	                	ret_text += "Instance found.\n\n";
	                }
	                else{
	                	ret_text += "No instance found.\n\n";
	                	ret = 2;
	                }
	                
	            }
    		}
        	catch (Err ex) {
        		ret_text += ex.getMessage();
        		system_exit(3, ret_text);
    		}
    	}
    	
    	//command `FbN
    	if(check_type.equals("command")){
    		String command_text = args[2];
    		//String command_text = "simulation";
    		try {
	            // Choose some default options for how you want to execute the commands
	            A4Options options = new A4Options();
	            options.solver = A4Options.SatSolver.SAT4J;
	            
	            Command command = null;
	            for(int i = 0; i < world.getAllCommands().size(); i ++){
	            	if(world.getAllCommands().get(i).label.indexOf(command_text) != -1){
	            		command = world.getAllCommands().get(i);
	            		break;
	            	}
	            }
	            if(command == null){
	            	ret_text += "no maching command : " + command_text + "\n";
	            	system_exit(4, ret_text);
	            }
	            
                // Execute the command
                A4Solution ans = TranslateAlloyToKodkod.execute_command(rep, world.getAllReachableSigs(), command, options);
                if (ans.satisfiable()) {
                	try{
	                	File file = new File("tmp_alloy_compiler_result.txt");
	                	ans.writeXML(new PrintWriter(file), null, null);
	                	create_end_file();
	                	System.exit(0);
	                }
	        		catch(IOException e){
	                	system_exit(3, e.getMessage());
	                }
                }
                else{
                	ret_text += "No instance found.\n\n";
                	system_exit(2, ret_text);
                }

    		}
        	catch (Err ex) {
        		ret_text += ex.getMessage();
        		system_exit(3, ret_text);
    		}
    	}
    	
    	if(check_type.equals("get_sig_list")){
            SafeList<Sig> sig_list = world.getAllSigs();
            for(int i = 0; i < sig_list.size(); i ++){
            	String def = "";
            	Sig sig = sig_list.get(i);
            	
            	if(sig.isAbstract != null) def = def + "abstract ";
            	if(sig.isEnum != null) continue;
            	if(sig.isLone != null) continue;
            	if(sig.isMeta != null) continue;
            	if(sig.isOne != null) continue;
            	def += "sig " + sig.label;
             	
            	if(sig.isSubset != null) continue;
            	else{
            		if(sig.isTopLevel() == false){
                		Sig parent = ((PrimSig)sig).parent;
                		def += " extends " + parent.label;
                	}            		
            	}
            	
            	ret_text += def + "\n";
            }
            system_exit(0, ret_text);
            
    	}
    	
    	system_exit(ret, ret_text);
    }
    
    static void system_exit(int ret, String ret_text){
    	create_result_file(ret_text);
    	create_end_file();
        System.exit(ret);
    }
    
    static void create_result_file(String ret){
		try{
	        File file = new File("tmp_alloy_compiler_result.txt");
	        FileWriter filewriter = new FileWriter(file);
	        filewriter.write(ret);
	        filewriter.close();
		}
		catch(IOException e){
			
		}
    }
    
    static void create_end_file(){
		try{
	        File file = new File("tmp_alloy_compiler_end.txt");
	        FileWriter filewriter = new FileWriter(file);
	        filewriter.close();
		}
		catch(IOException e){
			
		}
    }
}
